<?php

namespace ShopWP\API\Admin\Webhooks;

use ShopWP\Utils;

if (!defined('ABSPATH')) {
	exit;
}

class Queries {

   public static function default_admin_api_product_schema($selective_sync = [], $sync_by_collections = false) {

      if (empty($selective_sync)) {

         $syncing_collections = false;
         $syncing_products    = true;
         $syncing_options     = true;
         $syncing_tags        = true;
         $syncing_variants    = true;
         $syncing_metafields  = true;
         $syncing_images      = true;

      } else {

         $syncing_collections = empty($selective_sync['collections']) ? false : true;
         $syncing_products    = empty($selective_sync['products']) ? false : true;
         $syncing_options     = empty($selective_sync['options']) ? false : true;
         $syncing_tags        = empty($selective_sync['tags']) ? false : true;
         $syncing_variants    = empty($selective_sync['variants']) ? false : true;
         $syncing_metafields  = empty($selective_sync['metafields']) ? false : true;
         $syncing_images      = empty($selective_sync['images']) ? false : true;
         
      }

      return '
         id
         title
         ' . ($syncing_collections && empty($sync_by_collections) ? "
         collections {
            edges {
               node {
                  id
                  title
                  description
                  descriptionHtml
                  handle
                  image {
                     id
                     width
                     height
                     altText
                     url                     
                  }
                  productsCount
                  publishedOnCurrentPublication
                  ruleSet {
                     appliedDisjunctively
                     rules {
                     column
                     condition
                     relation
                     }
                  }
                  seo {
                     description
                     title
                  }
                  sortOrder
                  updatedAt                  
               }
            }
         }" : "") .
         ($syncing_products ? "
         createdAt
         customProductType
         defaultCursor
         description
         descriptionHtml
         handle
         hasOnlyDefaultVariant
         hasOutOfStockVariants
         isGiftCard
         legacyResourceId
         mediaCount
         onlineStorePreviewUrl
         onlineStoreUrl
         priceRangeV2 {
            maxVariantPrice {
               amount
               currencyCode
            }
            minVariantPrice {
               amount
               currencyCode
            }
         }
         productType
         publishedAt
         publishedOnCurrentPublication
         requiresSellingPlan
         seo {
            description
            title
         }
         status
         totalInventory
         totalVariants
         tracksInventory
         updatedAt
         vendor" : "") .
         ($syncing_images ? "
         featuredImage {
            altText
            id
            height
            url
            width
         }" : "") .
         ($syncing_options ? "
         options {
            id
            name
            values
            position
         }" : "") . ($syncing_tags ? "
         tags
         " : "") . ($syncing_images ? "
         media(first: 250) {
            edges {
               node {
                  alt
                  mediaContentType
                  preview {
                     image {
                        width
                        height
                        altText
                        id
                        url
                     }
                  }
                  ...on ExternalVideo {
                     id
                     embedUrl
                  }
                  ...on MediaImage {
                     image {
                        width
                        height
                        altText
                        id
                        url
                     }
                  }
                  ...on Video {
                     sources {
                        url
                        mimeType
                        format
                        height
                        width
                     }
                  }
               }
            }
         },
         " : "") . ($syncing_metafields ? "
         metafields(first: 250) {
            edges {
               node {
                  id
                  key
                  value
                  type
                  namespace
                  description
               }
            }
         }
         " : "") . ($syncing_variants ? "
         variants(first: 100) {
            edges {
               node {
                  id
                  availableForSale
                  barcode
                  compareAtPrice
                  createdAt
                  defaultCursor
                  displayName
                  image {
                     id
                     width
                     height
                     altText
                     url
                  }
                  inventoryItem {
                     tracked
                  }
                  inventoryPolicy
                  inventoryQuantity
                  legacyResourceId
                  position
                  price
                  product {
                     title
                  }
                  selectedOptions {
                     name
                     value
                  }
                  sellableOnlineQuantity
                  sellingPlanGroupCount
                  sku
                  taxCode
                  taxable
                  title
                  updatedAt
                  weight
                  weightUnit
               }
            }
         }
         " : "") . '';
   }

   public function graph_query_get_current_bulk_operation() {
      return [
         "query" => 'query {
            currentBulkOperation {
               id
               status
               errorCode
               createdAt
               completedAt
            }
         }'
      ];
   }

   public function graph_query_get_bulk_operation($bulk_id) {
      return [
         "query" => 'query {
            node(id: "' . $bulk_id . '") {
               ... on BulkOperation {
                  url
                  partialDataUrl
                  errorCode
                  fileSize
                  query
                  status
               }
            }
         }
         ',
      ];
   }

   public function graph_query_cancel_bulk_operation($bulk_id) {
      return [
         "query" => 'mutation {
            bulkOperationCancel(id: "' . $bulk_id . '") {
               bulkOperation {
                  status
               }
               userErrors {
                  field
                  message
               }
            }
         }
         ',
      ];
   }

   
   public function graph_query_create_bulk_webhook_subscription($callback_url) {
      return [
         "query" => 'mutation {
            webhookSubscriptionCreate(
               topic: BULK_OPERATIONS_FINISH
               webhookSubscription: {
                  format: JSON,
                  callbackUrl: "' . $callback_url . '"
               }
            ) {
               userErrors {
                  field
                  message
               }
               webhookSubscription {
                  id
               }
            }
         }'
      ];
   }

   public function graph_query_delete_bulk_webhook_subscription($webhook_id) {
      return [
         "query" => 'mutation webhookSubscriptionDelete($id: ID!) {
            webhookSubscriptionDelete(id: $id) {
               deletedWebhookSubscriptionId
               userErrors {
                  field
                  message
               }
            }
         }',
         "variables" => [
            'id' => $webhook_id
         ]
      ];
   }

   public function graph_query_get_all_webhook_subscriptions($params) {

      $topics        = isset($params['topics']) ? $params['topics'] : null;
      $callbackUrl   = isset($params['callbackUrl']) ? $params['callbackUrl'] : null;
      $cursor        = isset($params['cursor']) ? $params['cursor'] : null; 

      return [
         "query" => 'query($callbackUrl: URL, $cursor: String, $topics: [WebhookSubscriptionTopic!]) {
               webhookSubscriptions(first: 250, after: $cursor, callbackUrl: $callbackUrl, topics: $topics) {
                  edges {
                     node {
                        id
                        endpoint
                        createdAt
                        topic
                     }
                  },
                  pageInfo {
                     hasNextPage
                     endCursor
                  }
               }
            }',
         "variables" => [
            'callbackUrl'  => $callbackUrl,
            'cursor'       => $cursor,
            'topics'       => $topics 
         ]
      ];
   }

   public function get_orders_query($selective_sync) {
      
      if (!empty($selective_sync['orders'])) {
         $orders_query = '
            orders {
               edges {
                  node {
                     id
                     app {
                        name
                     }
                     billingAddress {
                        address1
                        address2
                        city
                        company
                        coordinatesValidated
                        country
                        countryCodeV2
                        firstName
                        formatted
                        formattedArea
                        id
                        lastName
                        latitude
                        longitude
                        name
                        phone
                        province
                        provinceCode
                        zip
                     }
                     billingAddressMatchesShippingAddress
                     canMarkAsPaid
                     canNotifyCustomer
                     cancelReason
                     cancelledAt
                     capturable
                     cartDiscountAmountSet {
                        presentmentMoney {
                           amount
                           currencyCode
                        }
                        shopMoney {
                           amount
                           currencyCode
                        }
                     }
                     channelInformation {
                        app {
                           title
                        }
                        channelDefinition {
                           channelName
                           id
                           svgIcon
                        }
                        channelId
                        id
                     }
                     clientIp
                     closed
                     closedAt
                     confirmed
                     createdAt
                     currencyCode
                     currentCartDiscountAmountSet {
                        presentmentMoney {
                           amount
                           currencyCode
                        }
                        shopMoney {
                           amount
                           currencyCode
                        }
                     }
                     currentSubtotalLineItemsQuantity
                     currentSubtotalPriceSet {
                        presentmentMoney {
                           amount
                           currencyCode
                        }
                        shopMoney {
                           amount
                           currencyCode
                        }
                     }
                     currentTaxLines {
                        priceSet  {
                           presentmentMoney {
                              amount
                              currencyCode
                           }
                           shopMoney {
                              amount
                              currencyCode
                           }
                        }
                        rate
                        ratePercentage
                        title
                     }
                     currentTotalDiscountsSet {
                        presentmentMoney {
                           amount
                           currencyCode
                        }
                        shopMoney {
                           amount
                           currencyCode
                        }
                     }
                     currentTotalDutiesSet {
                        presentmentMoney {
                           amount
                           currencyCode
                        }
                        shopMoney {
                           amount
                           currencyCode
                        }
                     }
                     currentTotalPriceSet {
                        presentmentMoney {
                           amount
                           currencyCode
                        }
                        shopMoney {
                           amount
                           currencyCode
                        }
                     }
                     currentTotalTaxSet {
                        presentmentMoney {
                           amount
                           currencyCode
                        }
                        shopMoney {
                           amount
                           currencyCode
                        }
                     }
                     currentTotalWeight
                     customAttributes {
                        key
                        value
                     }
                     customer {
                        id
                     }
                     customerAcceptsMarketing
                     customerJourneySummary {
                        firstVisit {
                           id
                           landingPage
                        }
                     }
                     customerLocale
                     discountCode
                     discountCodes
                     displayAddress {
                        address1
                        address2
                        city
                        company
                        coordinatesValidated
                        country
                        countryCodeV2
                        firstName
                        formatted
                        formattedArea
                        id
                        lastName
                        latitude
                        longitude
                        name
                        phone
                        province
                        provinceCode
                        zip
                     }
                     displayFinancialStatus
                     displayFulfillmentStatus
                     disputes {
                        id
                     }
                     edited
                     email
                     estimatedTaxes
                     fulfillable
                     fulfillments {
                        id
                     }
                     fullyPaid
                     hasTimelineComment
                     legacyResourceId
                     merchantEditable
                     merchantEditableErrors
                     name
                     netPaymentSet {
                        presentmentMoney {
                           amount
                           currencyCode
                        }
                        shopMoney {
                           amount
                           currencyCode
                        }
                     }
                     note
                     originalTotalDutiesSet {
                        presentmentMoney {
                           amount
                           currencyCode
                        }
                        shopMoney {
                           amount
                           currencyCode
                        }
                     }
                     originalTotalPriceSet {
                        presentmentMoney {
                           amount
                           currencyCode
                        }
                        shopMoney {
                           amount
                           currencyCode
                        }
                     }
                     paymentCollectionDetails {
                        additionalPaymentCollectionUrl
                     }
                     phone
                     physicalLocation {
                        id
                     }
                     presentmentCurrencyCode
                     processedAt
                     publication {
                        id
                     }
                     refundable
                     refunds(first: 10) {
                        id
                     }
                     registeredSourceUrl
                     requiresShipping
                     restockable
                     riskLevel
                     risks {
                        display
                        level
                        message
                     }
                     shippingAddress {
                        address1
                        address2
                        city
                        company
                        coordinatesValidated
                        country
                        countryCodeV2
                        firstName
                        formatted
                        formattedArea
                        id
                        lastName
                        latitude
                        longitude
                        name
                        phone
                        province
                        provinceCode
                        zip
                     }
                     shippingLine {
                        id
                     }
                     sourceIdentifier
                     subtotalLineItemsQuantity
                     subtotalPriceSet {
                        presentmentMoney {
                           amount
                           currencyCode
                        }
                        shopMoney {
                           amount
                           currencyCode
                        }
                     }
                     tags
                     taxLines {
                        title
                     }
                     taxesIncluded
                     test
                     totalCapturableSet {
                        presentmentMoney {
                           amount
                           currencyCode
                        }
                        shopMoney {
                           amount
                           currencyCode
                        }
                     } 
                     totalDiscountsSet {
                        presentmentMoney {
                           amount
                           currencyCode
                        }
                        shopMoney {
                           amount
                           currencyCode
                        }
                     }
                     totalOutstandingSet {
                        presentmentMoney {
                           amount
                           currencyCode
                        }
                        shopMoney {
                           amount
                           currencyCode
                        }
                     }
                     totalPriceSet {
                        presentmentMoney {
                           amount
                           currencyCode
                        }
                        shopMoney {
                           amount
                           currencyCode
                        }
                     }
                     totalReceivedSet {
                        presentmentMoney {
                           amount
                           currencyCode
                        }
                        shopMoney {
                           amount
                           currencyCode
                        }
                     }
                     totalRefundedSet {
                        presentmentMoney {
                           amount
                           currencyCode
                        }
                        shopMoney {
                           amount
                           currencyCode
                        }
                     }
                     totalRefundedShippingSet {
                        presentmentMoney {
                           amount
                           currencyCode
                        }
                        shopMoney {
                           amount
                           currencyCode
                        }
                     }
                     totalShippingPriceSet {
                        presentmentMoney {
                           amount
                           currencyCode
                        }
                        shopMoney {
                           amount
                           currencyCode
                        }
                     }
                     totalTaxSet {
                        presentmentMoney {
                           amount
                           currencyCode
                        }
                        shopMoney {
                           amount
                           currencyCode
                        }
                     }
                     totalTipReceivedSet {
                        presentmentMoney {
                           amount
                           currencyCode
                        }
                        shopMoney {
                           amount
                           currencyCode
                        }
                     }
                     totalWeight
                     transactions {
                        accountNumber
                        amountSet {
                           presentmentMoney {
                              amount
                              currencyCode
                           }
                           shopMoney {
                              amount
                              currencyCode
                           }
                        } 
                        id
                        gateway
                        kind
                        status
                        order {
                           id
                        }
                        status 
                     }
                     unpaid
                     updatedAt
                  }
               }
            }
         ';

      } else {
         $orders_query = '';
      }

      return $orders_query;

   }

   public function get_customers_query($selective_sync) {
      
      if (!empty($selective_sync['customers'])) {
         $customers_query = '
            customers {
               edges {
                  node {
                     id
                     amountSpent {
                        amount
                        currencyCode
                     }
                     averageOrderAmountV2 {
                        amount
                        currencyCode
                     }
                     canDelete
                     createdAt
                     displayName
                     email
                     emailMarketingConsent {
                        consentUpdatedAt
                        marketingOptInLevel
                        marketingState
                     }
                     firstName
                     hasTimelineComment
                     image {
                        id
                        width
                        height
                        altText
                        url
                     }
                     lastName
                     legacyResourceId
                     lifetimeDuration
                     locale
                     multipassIdentifier
                     note
                     numberOfOrders
                     phone
                     productSubscriberStatus
                     smsMarketingConsent {
                        consentCollectedFrom
                        consentUpdatedAt
                        marketingOptInLevel
                        marketingState
                     }
                     state
                     statistics {
                        predictedSpendTier
                     }
                     tags
                     taxExempt
                     taxExemptions
                     unsubscribeUrl
                     updatedAt
                     validEmailAddress
                     verifiedEmail
                  }
               }
            }
         ';

      } else {
         $customers_query = '';
      }

      return $customers_query;

   }

   public function create_collections_query($sync_by_collections) {
      $result = array_reduce($sync_by_collections, function($accumulator, $collection) {

         $title = $collection['title'];

         $accumulator .= "(title:" . $title . ") OR ";

         return $accumulator;

      }, '');

      $final_collections_query = preg_replace('/ OR $/', '', $result);

      // return $final_collections_query . ' AND published_status:\'shopwp:visible\'';
      return $final_collections_query;

   }

   public function get_collections_products_query($products_schema, $is_syncing_products) {
      if ($is_syncing_products) {
         return '
            products {
               edges {
                  node {
                     ' . $products_schema . '
                  }
               }
            }
         ';
      } else {
         return '';
      }
   }

   public function create_collections_and_products_queries($products_schema, $sync_by_collections, $is_syncing_products) {
      return [
         'collections'  => $this->create_collections_query($sync_by_collections),
         'products'     => $this->get_collections_products_query($products_schema, $is_syncing_products)
      ];
   }

   public function get_collections_query($products_schema, $selective_sync, $sync_by_collections, $is_syncing_products = true, $single = false) {

      $queries = $this->create_collections_and_products_queries($products_schema, $sync_by_collections, $is_syncing_products);
      
      if ($single) {
         $args = 'first: 1, query: "' . $queries['collections'] . '"';

      } else {
         $args = 'query: "' . $queries['collections'] . '"';
      }

      return '
         collections(' . $args . ') {
            edges {
               node {
                  id
                  title
                  ' . ($selective_sync['collections'] ? "
                  description
                  descriptionHtml
                  handle
                  image {
                     id
                     width
                     height
                     altText
                     url                     
                  }
                  productsCount
                  publishedOnCurrentPublication
                  ruleSet {
                     appliedDisjunctively
                     rules {
                        column
                        condition
                        relation
                     }
                  }
                  seo {
                     description
                     title
                  }
                  sortOrder
                  updatedAt                  
                  " : "")
                  . $queries['products'] . '
               }
            }
         }
      ';       
   }

   public function graph_query_create_bulk_products_operation($queries) {
      return [
         "query" => 'mutation {
            bulkOperationRunQuery(
               query: """
                  {
                     ' . $queries['products'] . '
                     ' . $queries['customers'] . '
                     ' . $queries['orders'] . '
                  }
               """
            ) {
               bulkOperation {
                  id
                  status
                  objectCount
                  rootObjectCount
                  completedAt
               }
               userErrors {
                  field
                  message
               }
            }
         }'
      ];
   }

}