<?php

namespace ShopWP\API\Admin\Webhooks;

if (!defined('ABSPATH')) {
	exit;
}

class Webhooks {

    public $GraphQL;
    public $Admin_Queries;
    public $DB_Settings_Syncing;
    public $DB_Settings_General;

    public function __construct($GraphQL, $Admin_Queries, $DB_Settings_Syncing, $DB_Settings_General) {
        
        $this->GraphQL = $GraphQL;
        $this->Admin_Queries = $Admin_Queries;
        $this->DB_Settings_Syncing = $DB_Settings_Syncing;
        $this->DB_Settings_General = $DB_Settings_General;

        add_filter('cron_schedules', [$this, 'shopwp_bulk_cron_recheck_interval' ]);
        add_filter('cron_schedules', [$this, 'shopwp_auto_sync_recheck_interval' ]);

    }

    public function shopwp_bulk_cron_recheck_interval($schedules) { 
        
        $schedules['shopwp_bulk_operation_recheck_interval'] = [
            'interval' => 10,
            'display'  => esc_html__( 'Every 10 Seconds' ) 
        ];

        return $schedules;

    }

    public function shopwp_auto_sync_recheck_interval($schedules) { 

        $seconds = $this->DB_Settings_General->col('sync_cron_interval', 'int');

        $schedules['shopwp_auto_sync_recheck_interval'] = [
            'interval' => $seconds,
            'display'  => esc_html__( 'ShopWP auto sync interval' ) 
        ];

        return $schedules;

    }
    
    public function maybe_schedule_cron($cron_callback, $cron_interval) {
        
         if (defined('DISABLE_WP_CRON')) {

            if (DISABLE_WP_CRON === 'true' || DISABLE_WP_CRON === true) {

                $this->DB_Settings_Syncing->log(__('WP-CRON has been disabled by this WordPress installation. Unable to poll for bulk op completion.', 'shopwp'), 'warning');

                return false;
            }
        }

        if ( !\wp_next_scheduled($cron_callback)) {

            $this->DB_Settings_Syncing->log(__('Cron not scheduled, scheduling new cron ...', 'shopwp'));

            $scheduled = \wp_schedule_event( time(), $cron_interval, $cron_callback );

            return $scheduled;

        }

        return \wp_next_scheduled($cron_callback);

    }

    public function unschedule_all_crons() {

        $this->DB_Settings_Syncing->log(__('Removing all crons ...', 'shopwp'));

        $remove_cron_1 = \wp_clear_scheduled_hook('shopwp_bulk_recheck_cron_hook');
        $remove_cron_2 = \wp_clear_scheduled_hook('shopwp_auto_sync_cron_callback');

        return [
            'shopwp_bulk_recheck_cron_hook' => $remove_cron_1,
            'shopwp_auto_sync_cron_callback' => $remove_cron_2
        ];
    }

    public function admin_api_get_bulk_operation($bulk_id) {

        return $this->GraphQL->graphql_api_request(
            $this->Admin_Queries->graph_query_get_bulk_operation($bulk_id),
            'admin',
            [
                'mutation_key' => false,
                'data_key' => 'node', 
                'user_errors_key' => 'userErrors'
            ]
        );

    }

    public function admin_api_create_bulk_webhook_subscription($callback_url) {
        return $this->GraphQL->graphql_api_request(
            $this->Admin_Queries->graph_query_create_bulk_webhook_subscription($callback_url),
            'admin',
            [
                'mutation_key' => 'webhookSubscriptionCreate',
                'data_key' => 'webhookSubscription', 
                'user_errors_key' => 'userErrors'
            ]
        );

    }

    public function admin_api_delete_bulk_webhook_subscription($webhook_id) {

        return $this->GraphQL->graphql_api_request(
            $this->Admin_Queries->graph_query_delete_bulk_webhook_subscription($webhook_id),
            'admin',
            [
                'mutation_key' => 'webhookSubscriptionDelete',
                'data_key' => 'deletedWebhookSubscriptionId', 
                'user_errors_key' => 'userErrors'
            ]
        );

    }

    public function admin_api_get_all_webhook_subscriptions($params = []) {

        return $this->GraphQL->graphql_api_request(
            $this->Admin_Queries->graph_query_get_all_webhook_subscriptions($params),
            'admin',
            [
                'mutation_key' => false,
                'data_key' => 'webhookSubscriptions', 
                'user_errors_key' => 'userErrors'
            ]
        );

    }

    public function admin_api_create_bulk_op($queries) {
        return $this->GraphQL->graphql_api_request(
            $this->Admin_Queries->graph_query_create_bulk_products_operation($queries),
            'admin',
            [
                'mutation_key' => 'bulkOperationRunQuery',
                'data_key' => 'bulkOperation', 
                'user_errors_key' => 'userErrors'
            ]
        );

    }

    public function admin_api_cancel_bulk_operation($bulk_id) {
        return $this->GraphQL->graphql_api_request(
            $this->Admin_Queries->graph_query_cancel_bulk_operation($bulk_id),
            'admin',
            [
                'mutation_key' => 'bulkOperationCancel',
                'data_key' => 'bulkOperation', 
                'user_errors_key' => 'userErrors'
            ]
        );
    }

    public function admin_api_get_current_bulk_operation() {

        return $this->GraphQL->graphql_api_request(
            $this->Admin_Queries->graph_query_get_current_bulk_operation(),
            'admin',
            [
                'mutation_key' => false,
                'data_key' => 'currentBulkOperation', 
                'user_errors_key' => 'userErrors'
            ]
        );

    }

}


