<?php

namespace ShopWP\API\Settings;

use ShopWP\Options;
use ShopWP\Transients;
use ShopWP\Utils;
use ShopWP\Utils\Data;

if (!defined('ABSPATH')) {
    exit();
}

class General extends \ShopWP\API
{
    public $DB_Settings_General;
    public $DB_Settings_Syncing;

    public function __construct(
        $DB_Settings_General,
        $DB_Settings_Syncing
    ) {
        $this->DB_Settings_General  = $DB_Settings_General;
        $this->DB_Settings_Syncing  = $DB_Settings_Syncing;
    }

    public function massage_styles_selection($new_settings)
    {
        if (
            $new_settings['styles_all'] === 0 &&
            $new_settings['styles_core'] === 0 &&
            $new_settings['styles_grid'] === 0
        ) {
            return 1;
        }

        return (int) $new_settings['styles_all'];
    }

    public function maybe_update_overrides_js_file($js_overrides_new, $js_overrides_current) {

        $overrides_filepath = trim(SHOPWP_PLUGIN_DIR_PATH . 'public/templates/javascript/user-overrides.js');

        if (empty($js_overrides_new) || empty($js_overrides_new['data'])) {

            $size = filesize($overrides_filepath);

            if ($size === false) {
                Utils::log([
                    'thing_to_log'  => 'Unable to determine the size of user-overrides.js. Does it exist?',
                    'log_level'     => 'error',
                    'file'          => 'shopwp-debug'
                ]);
            }

            // Current file has something in it, delete it
            if ($size !== false && $size > 0) {

                $success = file_put_contents(
                    $overrides_filepath,
                    '',
                );

                if ($success === false) {
                    Utils::log([
                        'thing_to_log'  => 'Unable to empty the user-overrides.js file.',
                        'log_level'     => 'error',
                        'file'          => 'shopwp-debug'
                    ]);
                }
            }

        } else {

            if (empty($js_overrides_current)) {

                $success = file_put_contents(
                    $overrides_filepath,
                    $js_overrides_new['data'],
                );

                if ($success === false) {
                    Utils::log([
                        'thing_to_log'  => 'Unable to update the user-overrides.js file.',
                        'log_level'     => 'error',
                        'file'          => 'shopwp-debug'
                    ]);
                }

            } else {

                if (!str_contains($js_overrides_current['data'], $js_overrides_new['data'])) {

                    $success = file_put_contents(
                        $overrides_filepath,
                        $js_overrides_new['data'],
                    );

                    if ($success === false) {
                        Utils::log([
                            'thing_to_log'  => 'Unable to update the user-overrides.js file after something changed.',
                            'log_level'     => 'error',
                            'file'          => 'shopwp-debug'
                        ]); 
                    }                    
                }
            }
        }

    }

    public function update_settings($request)
    {
        
        
        $settings = $request->get_param('settings');
        $flush_rewrite_rules = $request->get_param('flushRewriteRules');

        unset($settings['rechargeApiKey']); // needed so the masked values aren't added back
        unset($settings['yotpoApiKey']);
        unset($settings['yotpoApiSecretKey']);
        unset($settings['yotpoUtoken']);

        $new_settings = Utils::convert_settings_shape($settings, true);

        $js_overrides_new       = \maybe_unserialize($new_settings['javascript_overrides']);
        $js_overrides_current   = \maybe_unserialize($this->DB_Settings_General->col('javascript_overrides'));

        $this->maybe_update_overrides_js_file($js_overrides_new, $js_overrides_current);


        if (empty($new_settings)) {
            return $this->handle_response([
                'response' => Utils::wp_error([
                    'message_lookup' =>
                        __('Error while attempting to gather settings before saving.', 'shopwp'),
                    'call_method' => __METHOD__,
                    'call_line' => __LINE__,
                ]),
            ]);
        }

        $results = $this->DB_Settings_General->update_general($new_settings);

        Transients::delete_cached_settings();

        if ($flush_rewrite_rules) {
            update_option('shopwp_should_flush_rewrite_rules', 1, false);
        }

        return $this->handle_response([
            'response' => $results,
        ]);
    }

    public function register_route_settings()
    {
        return register_rest_route(
            SHOPWP_SHOPIFY_API_NAMESPACE,
            '/settings',
            [
                [
                    'methods' => \WP_REST_Server::CREATABLE,
                    'callback' => [$this, 'update_settings'],
                    'permission_callback' => [$this, 'pre_process'],
                ],
            ]
        );
    }

    public function init()
    {
        add_action('rest_api_init', [$this, 'register_route_settings']);
    }
}
