<?php

namespace ShopWP\API\Storefront\Cart;

if (!defined('ABSPATH')) {
	exit;
}

class Queries {

   public function default_cart_schema($cart_data = false) {

      if ($cart_data && !empty($cart_data['metafields'])) {
         $metafields = $cart_data['metafields'];
         
      } else {
         $metafields = false;
      }

      return '
         id
         note
         checkoutUrl
         createdAt
         updatedAt
         totalQuantity
         buyerIdentity {
            countryCode
            email
            phone
         }
         attributes {
            key
            value
         }
         cost {
            checkoutChargeAmount {
               amount
               currencyCode               
            }
            subtotalAmount {
               amount
               currencyCode
            }
            subtotalAmountEstimated
            totalAmount {
               amount
               currencyCode
            }
            totalAmountEstimated
            totalDutyAmount {
               amount
               currencyCode
            }
            totalDutyAmountEstimated
            totalTaxAmount {
               amount
               currencyCode
            }
            totalTaxAmountEstimated
         }
         discountCodes {
            applicable
            code
         }
         discountAllocations {
            discountedAmount {
               amount
               currencyCode
            }
         }
         lines(first: 250) {
            edges {
               node {
                  id
                  merchandise {
                     ... on ProductVariant {
                        product {
                           title
                           handle
                        }
                        availableForSale
                        compareAtPrice {
                           amount
                           currencyCode
                        }
                        currentlyNotInStock
                        id
                        image {
                           width
                           height
                           altText
                           id
                           originalSrc
                           transformedSrc
                        }' . $metafields . '
                        price {
                           amount
                           currencyCode
                        }
                        quantityAvailable
                        requiresShipping
                        selectedOptions {
                           name 
                           value
                        }
                        sku
                        title
                        weight
                        weightUnit
                     }
                  }
                  quantity
                  sellingPlanAllocation {
                     priceAdjustments {
                        price {
                           amount
                           currencyCode
                        }
                     }
                     sellingPlan {
                        description
                        id
                        name
                        recurringDeliveries
                        options {
                           name
                           value
                        }
                        priceAdjustments {
                           adjustmentValue 
                           orderCount
                        }
                        
                     }
                  }
                  attributes {
                     key
                     value
                  }
                  discountAllocations {
                     discountedAmount {
                        amount
                        currencyCode
                     }
                  }
                  cost {
                     amountPerQuantity {
                        amount
                        currencyCode
                     }
                     compareAtAmountPerQuantity {
                        amount
                        currencyCode
                     }
                     subtotalAmount {
                        amount
                        currencyCode
                     }
                     totalAmount {
                        amount
                        currencyCode
                     }
                  }
               }
            }
         }
      ';
   }

   public function graph_query_apply_discount($data) {
      return [
         "query" => 'mutation cartDiscountCodesUpdate($cartId: ID!, $discountCodes: [String!], $language: LanguageCode, $country: CountryCode) @inContext(country: $country, language: $language) {
               cartDiscountCodesUpdate(cartId: $cartId, discountCodes: $discountCodes) {
                  cart {
                     ' . $this->default_cart_schema($data) . '
                  }
                  userErrors {
                     field
                     message
                  }
               }
            }
         ',
         "variables" => [
            'cartId'          => $data['cartId'],
            'discountCodes'   => $data['discountCodes'],
            'language'        => $data['buyerIdentity']['language'],
            'country'         => $data['buyerIdentity']['countryCode']
         ]
      ];
   }

   public function graph_query_create_checkout($cart_data) {
      return [
         "query" => 'mutation checkoutCreate($cartInput: CheckoutCreateInput!) {
            checkoutCreate(input: $cartInput) {
               checkout {
                  id
               }
               checkoutUserErrors {
                  code
                  field
                  message
               }
            }
          }',
         "variables" => [
            'cartInput' => [
               "lineItems" => $cart_data['lines']
            ]
         ]
      ];
   }

   public function graph_query_add_lineitems($data) {
      return [
         "query" => 'mutation cartLinesAdd($cartId: ID!, $lines: [CartLineInput!]!, $language: LanguageCode, $country: CountryCode) @inContext(country: $country, language: $language) {
            cartLinesAdd(cartId: $cartId, lines: $lines) {
               cart {
                  ' . $this->default_cart_schema($data) . '
               }
               userErrors {
                  field
                  code
                  message
               }
            }
         }',
         "variables" => [
            'cartId'       => $data['cartId'],
            'lines'        => $data['lines'],
            'language'     => $data['buyerIdentity']['language'],
            'country'      => $data['buyerIdentity']['countryCode']
         ]
      ];

   }

   public function graph_query_remove_lineitems($data) {
      return [
         "query" => 'mutation cartLinesRemove($cartId: ID!, $lineIds: [ID!]!, $language: LanguageCode, $country: CountryCode) @inContext(country: $country, language: $language) {
               cartLinesRemove(cartId: $cartId, lineIds: $lineIds) {
                  cart {
                     ' . $this->default_cart_schema($data) . '
                  }
                  userErrors {
                     field
                     message
                  }
               }
            }
         ',
         "variables" => [
            'cartId'       => $data['cartId'],
            'lineIds'      => $data['lineIds'],
            'language'     => $data['buyerIdentity']['language'],
            'country'      => $data['buyerIdentity']['countryCode']
         ]
      ];

   }

   public function graph_query_update_lineitems($data) {

      return [
         "query" => 'mutation cartLinesUpdate($cartId: ID!, $lines: [CartLineUpdateInput!]!, $language: LanguageCode, $country: CountryCode) @inContext(country: $country, language: $language) {
               cartLinesUpdate(cartId: $cartId, lines: $lines) {
                  cart {
                     ' . $this->default_cart_schema($data) . '
                  }
                  userErrors {
                     field
                     message
                  }
               }
            }
         ',
         "variables" => [
            'cartId'       => $data['cartId'],
            'lines'        => $data['lines'],
            'language'     => $data['buyerIdentity']['language'],
            'country'      => $data['buyerIdentity']['countryCode']
         ]
      ];

   }

   public function graph_query_update_note($data) {
      return [
         "query" => 'mutation cartNoteUpdate($cartId: ID!, $note: String!, $language: LanguageCode, $country: CountryCode) @inContext(country: $country, language: $language) {
               cartNoteUpdate(cartId: $cartId, note: $note) {
                  cart {
                     ' . $this->default_cart_schema($data) . '
                  }
                  userErrors {
                     field
                     message
                  }
               }
            }
         ',
         "variables" => [
            'cartId'       => $data['cartId'],
            'note'         => $data['note'],
            'language'     => $data['buyerIdentity']['language'],
            'country'      => $data['buyerIdentity']['countryCode']
         ]
      ];

   }   
   

   public function graph_query_get_cart($data) {
      return [
         "query" => 'query($id: ID!, $language: LanguageCode, $country: CountryCode) @inContext(country: $country, language: $language) {
            cart(id: $id) {
               ' . $this->default_cart_schema($data) . '
            }
          }',
         "variables" => [
            'id'           => $data['id'],
            'language'     => $data['buyerIdentity']['language'],
            'country'      => $data['buyerIdentity']['country'],
         ]
      ];
   }

   public function graph_query_create_cart($data, $language, $country) {
      return [
         "query" => 'mutation cartCreate($lines: [CartLineInput!], $note: String, $attributes: [AttributeInput!]!, $buyerIdentity: CartBuyerIdentityInput
, $discountCodes: [String!], $language: LanguageCode, $country: CountryCode) @inContext(country: $country, language: $language) {
            cartCreate(input: {
               lines: $lines, 
               note: $note, 
               attributes: $attributes, 
               discountCodes: $discountCodes, 
               buyerIdentity: $buyerIdentity
            }) 
            {
               cart {
                  ' . $this->default_cart_schema($data) . '
               }
               userErrors {
                  code
                  field
                  message
               }
            }
          }',
         "variables" => [
            "lines"           => empty($data['lines']) ? [] : $data['lines'],
            "note"            => empty($data['note']) ? '' : $data['note'],
            "attributes"      => empty($data['attributes']) ? [] : $data['attributes'],
            "discountCodes"   => empty($data['discountCodes']) ? [] : $data['discountCodes'],
            'buyerIdentity'   => $data['buyerIdentity'],
            'language'        => $language,
            'country'         => $country,
         ]
      ];
   }

   public function graph_query_update_cart_attributes($data) {
      return [
         "query" => 'mutation cartAttributesUpdate($attributes: [AttributeInput!]!, $cartId: ID!, $language: LanguageCode, $country: CountryCode) @inContext(country: $country, language: $language) {
            cartAttributesUpdate(cartId: $cartId, attributes: $attributes) 
            {
               cart {
                  ' . $this->default_cart_schema($data) . '
               }
               userErrors {
                  code
                  field
                  message
               }
            }
          }',
         "variables" => [
            "cartId"       => $data['cartId'],
            "attributes"   => $data['attributes'],
            'language'     => $data['buyerIdentity']['language'],
            'country'      => $data['buyerIdentity']['countryCode'],
         ]
      ];

   }

   public function graph_query_update_buyer_identity($data, $language) {
      return [
         "query" => 'mutation cartBuyerIdentityUpdate($buyerIdentity: CartBuyerIdentityInput!, $cartId: ID!, $language: LanguageCode, $country: CountryCode) @inContext(country: $country, language: $language) {
               cartBuyerIdentityUpdate(buyerIdentity: $buyerIdentity, cartId: $cartId) {
                  cart {
                     ' . $this->default_cart_schema($data) . '
                  }
                  userErrors {
                     field
                     message
                  }
               }
            }
         ',
         "variables" => [
            "cartId"          => $data['cartId'],
            "buyerIdentity"   => $data['buyerIdentity'],
            'language'     => $language,
            'country'      => $data['buyerIdentity']['countryCode'],
         ]
      ];

   }   

}