<?php

namespace ShopWP\API\Syncing;

use ShopWP\Messages;
use ShopWP\Utils;

if (!defined('ABSPATH')) {
    exit();
}

class Indicator extends \ShopWP\API
{
    public $DB_Settings_Syncing;
    public $Filesystem;

    public function __construct($DB_Settings_Syncing, $Filesystem)
    {
        $this->DB_Settings_Syncing = $DB_Settings_Syncing;
        $this->Filesystem = $Filesystem;
    }

    public function set_syncing_indicator($request)
    {

        $syncing_on = $request->get_param('syncing');

        if (!is_bool($syncing_on)) {

            $wp_err = $this->DB_Settings_Syncing->api_error('Unable to turn syncing on due to type error.', __METHOD__, __LINE__);

            if (is_wp_error($wp_err)) {
                $this->DB_Settings_Syncing->log($wp_err->get_error_message(), 'error');
            }
            
            return wp_send_json_error($wp_err);
        }

        if ($syncing_on) {
            
            // Reset any previous sync
            $this->DB_Settings_Syncing->reset_syncing_notices();
            $this->DB_Settings_Syncing->reset_all_syncing_status();
            $this->DB_Settings_Syncing->reset_all_syncing_totals();
            $this->DB_Settings_Syncing->delete_log();

            $this->DB_Settings_Syncing->log(__('Syncing has been turned on', 'shopwp'));

        } else {

            $this->DB_Settings_Syncing->log(__('Syncing has been turned off', 'shopwp'));
            $this->DB_Settings_Syncing->expire_sync();
            
        }

        $toggle_syncing_result = $this->DB_Settings_Syncing->toggle_syncing($syncing_on);

        // If the DB update was unsuccessful ...
        if (!$toggle_syncing_result) {

            $wp_err = $this->DB_Settings_Syncing->api_error(__('Error: Unable to change syncing status due to database error.', 'shopwp'), __METHOD__, __LINE__);

            if (is_wp_error($wp_err)) {
                $this->DB_Settings_Syncing->log($wp_err->get_error_message(), 'error');
            }

            return wp_send_json_error($wp_err);
            
        }

        return $this->handle_response([
            'response' => $toggle_syncing_result,
        ]);
    }

    public function register_route_syncing_indicator()
    {
        return register_rest_route(
            SHOPWP_SHOPIFY_API_NAMESPACE,
            '/syncing/indicator',
            [
                [
                    'methods' => \WP_REST_Server::CREATABLE,
                    'callback' => [$this, 'set_syncing_indicator'],
                    'permission_callback' => [$this, 'pre_process'],
                ],
            ]
        );
    }

    public function init()
    {
        add_action('rest_api_init', [
            $this,
            'register_route_syncing_indicator',
        ]);
    }
}
