<?php

namespace ShopWP\API\Syncing;

use ShopWP\Messages;

if (!defined('ABSPATH')) {
    exit();
}

class Status extends \ShopWP\API
{
    public $DB_Settings_Syncing;
    public $DB_Settings_Connection;

    public function __construct($DB_Settings_Syncing, $DB_Settings_Connection)
    {
        $this->DB_Settings_Syncing      = $DB_Settings_Syncing;
        $this->DB_Settings_Connection   = $DB_Settings_Connection;
    }

    public function calc_syncing_total($totals) {
        return array_reduce($totals, function($carry, $current) {
            return $carry + $current;
        }, 0);
    }

    public function get_has_connection($connection_table_data)
    {
        if (empty($connection_table_data)) {
            return false;
        }

        if (
            empty($connection_table_data->api_password) ||
            empty($connection_table_data->domain)
        ) {
            return false;
        }

        return true;
    }

    public function get_syncing_status_data() {

        $syncing_table              = $this->DB_Settings_Syncing->get();
        $connection_table           = $this->DB_Settings_Connection->get();
        $uploads_dir                = \wp_get_upload_dir();

        $total_to_process           = $this->DB_Settings_Syncing->syncing_totals($syncing_table);
        $total_processed            = $this->DB_Settings_Syncing->syncing_current_amounts($syncing_table);

        $path                       = $uploads_dir['basedir'] . '/shopwp.log';
        $log_full                   = @file_get_contents($uploads_dir['basedir'] . '/shopwp.log', false, null);

        if ($log_full === false) {
            $log = __('No syncing log found, try refreshing the page.', 'shopwp');

        } else {
            $log = substr($log_full, -10000);
        }

        $final_total_to_process     = $this->calc_syncing_total($total_to_process);
        $final_total_processed      = $this->calc_syncing_total($total_processed);

        $is_syncing                 = (bool) $syncing_table->is_syncing;
        $next_cron_time             = \wp_next_scheduled('shopwp_auto_sync_cron_callback');

        if (!empty($next_cron_time) && !$is_syncing) {
            $current_step_text = __('Waiting for next sync ...', 'shopwp');

        } else {
            $current_step_text = $this->DB_Settings_Syncing->get_current_syncing_step_text();
        }

        return [
            'isSyncing'                     => $is_syncing,
            'currentStepText'               => $current_step_text,
            'totalToProcess'                => $final_total_to_process,
            'totalProcessed'                => $final_total_processed,
            'percentCompletedRemoval'       => $this->get_percent_completed_removal(),
            'syncingErrors'                 => $this->get_syncing_errors(),
            'syncingWarnings'               => $this->get_syncing_warnings(),
            'percentCompleted'              => $this->DB_Settings_Syncing->calc_percent_completed($total_processed, $total_to_process),            
            'syncLog'                       => $log,
            'syncingTotals'                 => $total_to_process,
            'syncingCurrentAmounts'         => $total_processed,
            'nextCronTime'                  => $next_cron_time,
            'hasConnection'                 => $this->get_has_connection($connection_table),
            'syncLogUrl'                    => $uploads_dir['baseurl'] . '/shopwp.log',
            'currentlyActiveTool'           => empty($syncing_table->currently_active_tool) ? false : $syncing_table->currently_active_tool
        ];
    }

    public function get_syncing_status($request)
    {
        
        $sync_data = $this->get_syncing_status_data();

        return \wp_send_json_success($sync_data);

    }

    public function get_syncing_warnings() {
        $any_errors = \maybe_unserialize($this->DB_Settings_Syncing->get_column_single('syncing_warnings'));

        if (empty($any_errors) || empty($any_errors[0]->syncing_warnings)) {
            return false;
        }
                
        $warnings = \maybe_unserialize($any_errors[0]->syncing_warnings);

        return $warnings['warning'];
    }

    public function get_syncing_errors() {

        $any_errors = \maybe_unserialize($this->DB_Settings_Syncing->get_column_single('syncing_errors'));

        if (empty($any_errors) || empty($any_errors[0]->syncing_errors)) {
            return false;
        }
                
        $errors = \maybe_unserialize($any_errors[0]->syncing_errors);

        return $errors['error'];

    }

    public function get_percent_completed_removal() {

        if (!$this->DB_Settings_Syncing->is_syncing()) {
            return 100;
        }

        $percent_completed_removal = $this->DB_Settings_Syncing->col('percent_completed_removal', 'int');

        return $percent_completed_removal;

    }

    public function get_syncing_status_media()
    {
        if (!$this->DB_Settings_Syncing->is_syncing()) {
            return true;
        }

        return $this->DB_Settings_Syncing->col('finished_media', 'bool');

    }

    public function register_route_syncing_status()
    {
        return register_rest_route(
            SHOPWP_SHOPIFY_API_NAMESPACE,
            '/syncing/status',
            [
                [
                    'methods' => \WP_REST_Server::CREATABLE,
                    'callback' => [$this, 'get_syncing_status'],
                    'permission_callback' => [$this, 'pre_process'],
                ],
            ]
        );
    }

    public function expire_sync()
    {
        $result = $this->handle_response([
            'response' => $this->DB_Settings_Syncing->expire_sync(),
        ]);

        update_option('shopwp_should_flush_rewrite_rules', 1);
        do_action('shopwp_processing_completed');
       
        return $result;
    }

    public function init()
    {
        add_action('rest_api_init', [$this, 'register_route_syncing_status']);
    }
}
