<?php

namespace ShopWP;

use ShopWP\Utils;
use ShopWP\Options;
use ShopWP\Transients;

if (!defined('ABSPATH')) {
    exit();
}

class Activator
{

    public $DB_Settings_Connection;
    public $DB_Settings_General;
    public $DB_Settings_License;
    public $DB_Settings_Syncing;
    public $DB_Shop;
    public $DB_Products;
    public $DB_Variants;
    public $DB_Collects;
    public $DB_Options;
    public $DB_Collections;
    public $DB_Images;
    public $DB_Tags;
    public $DB_Customers;
    public $DB_Orders;
    public $DB_Metafields;
    public $Async_Processing_Database;
    public $Compatibility;    
    public static $_instance = null;
    public $DB;

    public function __construct(
        $DB_Settings_Connection,
        $DB_Settings_General,
        $DB_Settings_License,
        $DB_Settings_Syncing,
        $DB_Shop,
        $DB_Products,
        $DB_Variants,
        $DB_Collects,
        $DB_Options,
        $DB_Collections,
        $DB_Images,
        $DB_Tags,
        $DB_Customers,
        $DB_Orders,
        $DB_Metafields,
        $Async_Processing_Database,
        $Compatibility
    ) {

        $this->DB_Settings_Connection       = $DB_Settings_Connection;
        $this->DB_Settings_General          = $DB_Settings_General;
        $this->DB_Settings_License          = $DB_Settings_License;
        $this->DB_Settings_Syncing          = $DB_Settings_Syncing;
        $this->DB_Shop                      = $DB_Shop;
        $this->DB_Products                  = $DB_Products;
        $this->DB_Variants                  = $DB_Variants;
        $this->DB_Collects                  = $DB_Collects;
        $this->DB_Options                   = $DB_Options;
        $this->DB_Collections               = $DB_Collections;
        $this->DB_Images                    = $DB_Images;
        $this->DB_Tags                      = $DB_Tags;
        $this->DB_Customers                 = $DB_Customers;
        $this->DB_Orders                    = $DB_Orders;
        $this->DB_Metafields                = $DB_Metafields;
        $this->DB                           = $DB_Settings_General; // alias only        
        $this->Async_Processing_Database    = $Async_Processing_Database;
        $this->Compatibility                = $Compatibility;
    }

	public static function instance() {

		if ( is_null( self::$_instance ) ) {
			self::$_instance = new self(
                Factories\DB\Settings_Connection_Factory::build(),
                Factories\DB\Settings_General_Factory::build(),
                Factories\DB\Settings_License_Factory::build(),
                Factories\DB\Settings_Syncing_Factory::build(),
                Factories\DB\Shop_Factory::build(),
                Factories\DB\Products_Factory::build(),
                Factories\DB\Variants_Factory::build(),
                Factories\DB\Collects_Factory::build(),
                Factories\DB\Options_Factory::build(),
                Factories\DB\Collections_Factory::build(),
                Factories\DB\Images_Factory::build(),
                Factories\DB\Tags_Factory::build(),
                Factories\DB\Customers_Factory::build(),
                Factories\DB\Orders_Factory::build(),
                Factories\DB\Metafields_Factory::build(),
                Factories\Processing\Database_Factory::build(),
                Factories\Compatibility\Manager_Factory::build(),
            );
		}

		return self::$_instance;
	}

    public function create_db_tables()
    {
        $results = [];

        $results['DB_Settings_Connection']  = $this->DB_Settings_Connection->create_table();
        $results['DB_Settings_General']     = $this->DB_Settings_General->create_table();
        $results['DB_Settings_License']     = $this->DB_Settings_License->create_table();
        $results['DB_Settings_Syncing']     = $this->DB_Settings_Syncing->create_table();
        $results['DB_Shop']                 = $this->DB_Shop->create_table();
        $results['DB_Products']             = $this->DB_Products->create_table();
        $results['DB_Variants']             = $this->DB_Variants->create_table();
        $results['DB_Collects']             = $this->DB_Collects->create_table();
        $results['DB_Options']              = $this->DB_Options->create_table();
        $results['DB_Collections']          = $this->DB_Collections->create_table();
        $results['DB_Images']               = $this->DB_Images->create_table();
        $results['DB_Tags']                 = $this->DB_Tags->create_table();
        $results['DB_Customers']            = $this->DB_Customers->create_table();
        $results['DB_Orders']               = $this->DB_Orders->create_table();
        $results['DB_Metafields']           = $this->DB_Metafields->create_table();

        return $results;
    }

    public function set_default_table_values()
    {
        $results = [];

        $results['DB_Settings_General'] = $this->DB_Settings_General->init();
        $results['DB_Settings_Syncing'] = $this->DB_Settings_Syncing->init();

        return $results;
    }

    public function bootstrap_tables()
    {
        $results = [];

        $results['create_db_tables'] = $this->create_db_tables();
        $results[
            'set_default_table_values'
        ] = $this->set_default_table_values();

        return $results;
    }

    public function get_ready()
    {   

        $results = [];

        $skip_mu                                            = apply_filters('shopwp_skip_compatibility', false);
        $results['bootstrap_wp_cache_flush']                = wp_cache_flush();
        $results['bootstrap_transient_delete_all_cache']    = Transients::delete_plugin_cache();
        $results['bootstrap_tables']                        = $this->bootstrap_tables();

        if ($skip_mu) {
            $results['bootstrap_compatibility']             = $this->Compatibility->delete_compatibility_mu();
        } else {
            $results['bootstrap_compatibility']             = $this->Compatibility->make_compatibility_mu();
        }
        
        $results['bootstrap_default_pages']                 = $this->maybe_create_default_pages();
        $results['flush_rewrite_rules']                     = update_option('shopwp_should_flush_rewrite_rules', 1, false);

        return $results;
    }

    public function bootstrap_blogs()
    {
        $blog_ids = \get_sites(['fields' => 'ids']);
        $results = [];

        foreach ($blog_ids as $blog_id) {
            $results[] = $this->bootstrap_blog($blog_id);
        }

        return $results;
    }

    /*

	Runs when the plugin is activated as a result of register_activation_hook.

	Runs for both Free and Pro versions

	*/
    public function on_plugin_activate($network_wide)
    {
        
        if (is_multisite() && $network_wide) {
            return $this->bootstrap_blogs();
        }

        return $this->get_ready();
    }

    public function bootstrap_blog($blog_id)
    {
        switch_to_blog($blog_id);

        // Bootstraps tables, creates CPTs, and flushes rewrites
        $ready_result = $this->get_ready();

        restore_current_blog();

        return $ready_result;
    }

    public function create_default_products_page()
    {
        $default_page_title = 'ShopWP Products';
        $existing_page = \get_page_by_title($default_page_title);

        if (!empty($existing_page)) {
            return $existing_page->ID;
        }

        return \wp_insert_post([
            'post_title' => \wp_strip_all_tags($default_page_title),
            'post_status' => 'publish',
            'post_author' => \get_current_user_id(),
            'post_type' => 'page',
            'post_name' => SHOPWP_DEFAULT_PRODUCTS_PAGE_SLUG,
        ]);
    }

    public function create_default_collections_page()
    {
        $default_page_title = 'ShopWP Collections';
        $existing_page = \get_page_by_title($default_page_title);

        if (!empty($existing_page)) {
            return $existing_page->ID;
        }

        return \wp_insert_post([
            'post_title' => \wp_strip_all_tags($default_page_title),
            'post_status' => 'publish',
            'post_author' => \get_current_user_id(),
            'post_type' => 'page',
            'post_name' => SHOPWP_DEFAULT_COLLECTIONS_PAGE_SLUG,
        ]);
    }

    public function maybe_create_default_pages()
    {

        /*
        
        Check if the default posts already exist
        
        */
        $existing_plp_post = get_posts([
            'post_type' => 'shopwp_shortcodes',
            'post_status'   => 'any',
            'meta_query' => [
                [
                    'key' => 'layout_type',
                    'value' => 'template-plp',
                ]
            ]
        ]);

        $existing_pdp_post = get_posts([
            'post_type'     => 'shopwp_shortcodes',
            'post_status'   => 'any',
            'meta_query' => [
                [
                    'key' => 'layout_type',
                    'value' => 'template-pdp',
                ]
            ]
        ]);

        // Create default posts if they don't exist
        if (empty($existing_plp_post)) {
            $plp_post_id = wp_insert_post([
                'post_type' => 'shopwp_shortcodes',
                'post_title' => 'Template PLP',
                'post_content' => '<!-- wp:columns {"align":"wide"} -->
                    <div class="wp-block-columns alignwide"><!-- wp:column -->
                    <div class="wp-block-column"><!-- wp:shopwp/products -->
                    <div data-wpshopify-component="true" data-wpshopify-component-id="" data-wpshopify-component-type="products" data-wpshopify-skeleton-type="" data-wpshopify-payload-settings=""></div>
                    <!-- /wp:shopwp/products --></div>
                    <!-- /wp:column --></div>
                    <!-- /wp:columns -->',
                'post_status' => 'publish'
            ]);

            if ($plp_post_id) {
                add_post_meta($plp_post_id, 'layout_type', 'template-plp', true);
            }
        }

        if (empty($existing_pdp_post)) {
            
            $pdp_post_id = wp_insert_post([
                'post_type' => 'shopwp_shortcodes',
                'post_title' => 'Template PDP',
                'post_status' => 'publish',
                'post_content' => '<!-- wp:columns {"align":"wide"} -->
                    <div class="wp-block-columns alignwide"><!-- wp:column -->
                    <div class="wp-block-column"><!-- wp:shopwp/images {"settingsId":"eyJxdWVyeSI6ImlkOlwiMjIxNjg3ODE0NTU4NFwiIiwibGltaXQiOjEsImV4Y2x1ZGVzIjpbImJ1eS1idXR0b24iLCJkZXNjcmlwdGlvbiIsInByaWNpbmciLCJ0aXRsZSJdLCJpdGVtc1BlclJvdyI6MSwicGFnaW5hdGlvbiI6ZmFsc2UsImxpbmtUbyI6Im5vbmUifQ=="} -->
                    <div data-wpshopify-component="true" data-wpshopify-component-id="" data-wpshopify-component-type="products" data-wpshopify-skeleton-type="" data-wpshopify-payload-settings="eyJxdWVyeSI6ImlkOlwiMjIxNjg3ODE0NTU4NFwiIiwibGltaXQiOjEsImV4Y2x1ZGVzIjpbImJ1eS1idXR0b24iLCJkZXNjcmlwdGlvbiIsInByaWNpbmciLCJ0aXRsZSJdLCJpdGVtc1BlclJvdyI6MSwicGFnaW5hdGlvbiI6ZmFsc2UsImxpbmtUbyI6Im5vbmUifQ=="></div>
                    <!-- /wp:shopwp/images --></div>
                    <!-- /wp:column -->

                    <!-- wp:column -->
                    <div class="wp-block-column"><!-- wp:shopwp/title {"settingsId":"eyJxdWVyeSI6ImlkOlwiMjIxNjg3ODE0NTU4NFwiIiwibGltaXQiOjEsImV4Y2x1ZGVzIjpbImJ1eS1idXR0b24iLCJpbWFnZXMiLCJkZXNjcmlwdGlvbiIsInByaWNpbmciXSwiaXRlbXNQZXJSb3ciOjEsInBhZ2luYXRpb24iOmZhbHNlLCJsaW5rVG8iOiJub25lIn0="} -->
                    <div data-wpshopify-component="true" data-wpshopify-component-id="" data-wpshopify-component-type="products" data-wpshopify-skeleton-type="" data-wpshopify-payload-settings="eyJxdWVyeSI6ImlkOlwiMjIxNjg3ODE0NTU4NFwiIiwibGltaXQiOjEsImV4Y2x1ZGVzIjpbImJ1eS1idXR0b24iLCJpbWFnZXMiLCJkZXNjcmlwdGlvbiIsInByaWNpbmciXSwiaXRlbXNQZXJSb3ciOjEsInBhZ2luYXRpb24iOmZhbHNlLCJsaW5rVG8iOiJub25lIn0="></div>
                    <!-- /wp:shopwp/title -->

                    <!-- wp:shopwp/pricing {"settingsId":"eyJxdWVyeSI6ImlkOlwiMjIxNjg3ODE0NTU4NFwiIiwibGltaXQiOjEsImV4Y2x1ZGVzIjpbImJ1eS1idXR0b24iLCJpbWFnZXMiLCJkZXNjcmlwdGlvbiIsInRpdGxlIl0sIml0ZW1zUGVyUm93IjoxLCJwYWdpbmF0aW9uIjpmYWxzZSwibGlua1RvIjoibm9uZSJ9"} -->
                    <div data-wpshopify-component="true" data-wpshopify-component-id="" data-wpshopify-component-type="products" data-wpshopify-skeleton-type="" data-wpshopify-payload-settings="eyJxdWVyeSI6ImlkOlwiMjIxNjg3ODE0NTU4NFwiIiwibGltaXQiOjEsImV4Y2x1ZGVzIjpbImJ1eS1idXR0b24iLCJpbWFnZXMiLCJkZXNjcmlwdGlvbiIsInRpdGxlIl0sIml0ZW1zUGVyUm93IjoxLCJwYWdpbmF0aW9uIjpmYWxzZSwibGlua1RvIjoibm9uZSJ9"></div>
                    <!-- /wp:shopwp/pricing -->

                    <!-- wp:shopwp/description {"settingsId":"eyJxdWVyeSI6ImlkOlwiMjIxNjg3ODE0NTU4NFwiIiwibGltaXQiOjEsImV4Y2x1ZGVzIjpbImJ1eS1idXR0b24iLCJpbWFnZXMiLCJwcmljaW5nIiwidGl0bGUiXSwiaXRlbXNQZXJSb3ciOjEsInBhZ2luYXRpb24iOmZhbHNlLCJsaW5rVG8iOiJub25lIn0="} -->
                    <div data-wpshopify-component="true" data-wpshopify-component-id="" data-wpshopify-component-type="products" data-wpshopify-skeleton-type="" data-wpshopify-payload-settings="eyJxdWVyeSI6ImlkOlwiMjIxNjg3ODE0NTU4NFwiIiwibGltaXQiOjEsImV4Y2x1ZGVzIjpbImJ1eS1idXR0b24iLCJpbWFnZXMiLCJwcmljaW5nIiwidGl0bGUiXSwiaXRlbXNQZXJSb3ciOjEsInBhZ2luYXRpb24iOmZhbHNlLCJsaW5rVG8iOiJub25lIn0="></div>
                    <!-- /wp:shopwp/description -->

                    <!-- wp:shopwp/buy-button {"settingsId":"eyJxdWVyeSI6ImlkOlwiMjIxNjg3ODE0NTU4NFwiIiwibGltaXQiOjEsImV4Y2x1ZGVzIjpbImltYWdlcyIsImRlc2NyaXB0aW9uIiwicHJpY2luZyIsInRpdGxlIl0sIml0ZW1zUGVyUm93IjoxLCJwYWdpbmF0aW9uIjpmYWxzZSwibGlua1RvIjoibm9uZSIsImFkZFRvQ2FydEJ1dHRvblRleHQiOiJBZGQgdG8gY2FydCJ9"} -->
                    <div data-wpshopify-component="true" data-wpshopify-component-id="" data-wpshopify-component-type="products" data-wpshopify-skeleton-type="" data-wpshopify-payload-settings="eyJxdWVyeSI6ImlkOlwiMjIxNjg3ODE0NTU4NFwiIiwibGltaXQiOjEsImV4Y2x1ZGVzIjpbImltYWdlcyIsImRlc2NyaXB0aW9uIiwicHJpY2luZyIsInRpdGxlIl0sIml0ZW1zUGVyUm93IjoxLCJwYWdpbmF0aW9uIjpmYWxzZSwibGlua1RvIjoibm9uZSIsImFkZFRvQ2FydEJ1dHRvblRleHQiOiJBZGQgdG8gY2FydCJ9"></div>
                    <!-- /wp:shopwp/buy-button --></div>
                    <!-- /wp:column --></div>
                    <!-- /wp:columns -->'
            ]);

            if ($pdp_post_id) {
                add_post_meta($pdp_post_id, 'layout_type', 'template-pdp', true);
            }
        }

    }

    /*

	Runs when a new blog is created within a multi-site setup. NOT when activated network wide.

	*/
    public function on_blog_create(
        $blog_id,
        $user_id,
        $domain,
        $path,
        $site_id,
        $meta
    ) {
        if (Utils::is_network_wide()) {
            $this->bootstrap_blog($blog_id);
        }
    }

    public function on_blog_create_from_wp_site($blog)
    {
        if (is_object($blog) && isset($blog->blog_id)) {
            if (Utils::is_network_wide()) {
                $this->bootstrap_blog($blog->blog_id);
            }
        }
    }

    /*

	Deletes custom tables when blog is deleted

	$tables is an array containing a list of table names in string format

	*/
    public function on_blog_delete($tables)
    {

        $tables[] = SHOPWP_TABLE_NAME_SETTINGS_CONNECTION;
        $tables[] = SHOPWP_TABLE_NAME_SETTINGS_GENERAL;
        $tables[] = SHOPWP_TABLE_NAME_SETTINGS_LICENSE;
        $tables[] = SHOPWP_TABLE_NAME_SETTINGS_SYNCING;
        $tables[] = SHOPWP_TABLE_NAME_SHOP;
        $tables[] = SHOPWP_TABLE_NAME_PRODUCTS;
        $tables[] = SHOPWP_TABLE_NAME_VARIANTS;
        $tables[] = SHOPWP_TABLE_NAME_COLLECTS;
        $tables[] = SHOPWP_TABLE_NAME_OPTIONS;
        $tables[] = SHOPWP_TABLE_NAME_COLLECTIONS_CUSTOM;
        $tables[] = SHOPWP_TABLE_NAME_COLLECTIONS_SMART;
        $tables[] = SHOPWP_TABLE_NAME_IMAGES;
        $tables[] = SHOPWP_TABLE_NAME_TAGS;
        $tables[] = SHOPWP_TABLE_NAME_CUSTOMERS;
        $tables[] = SHOPWP_TABLE_NAME_ORDERS;

        return $tables;
    }

    public function is_dev_version($current_version_number)
    {
        return \version_compare($current_version_number, '10.0.0', '>=');
    }

    public function is_outdated($current, $new)
    {
        return \version_compare($current, $new, '<');
    }

    /*

    Runs when the plugin updates.

    Will only run once since we're updating the plugin version after everything gets executed.

    TODO: This functions gets executed many times lol

    */
    public function on_plugins_loaded()
    {
        
        $new_version_number = SHOPWP_NEW_PLUGIN_VERSION;

        $current_version_number = $this->DB_Settings_General->col(
            'plugin_version',
            'string'
        );

        if (!$current_version_number) {
            return;
        }

        if (\is_wp_error($current_version_number)) {
            Utils::log([
                'thing_to_log'  => $current_version_number->get_error_message(),
                'log_level'     => 'error',
                'file'          => 'shopwp-debug'
            ]);
            return;
        }

        if (
            $this->is_dev_version($current_version_number) ||
            $this->is_outdated($current_version_number, $new_version_number)
        ) {

            $this->Async_Processing_Database->sync_table_deltas();

            $this->DB_Settings_General->update_plugin_version(
                $new_version_number
            );

            Transients::delete_plugin_cache();
            Options::delete('shopwp_migration_needed');

        }
        
    }

    public function init()
    {
        add_action('plugins_loaded', [$this, 'on_plugins_loaded']);
        add_filter('wpmu_drop_tables', [$this, 'on_blog_delete']);

        if (is_multisite()) {

            global $wp_version;

            if (version_compare($wp_version, '5.1', '<')) {
                add_action('wpmu_new_blog', [$this, 'on_blog_create'], 10, 6);
            } else {
                add_action('wp_initialize_site', [
                    $this,
                    'on_blog_create_from_wp_site',
                ]);
            }
        }
        // plugins_loaded loads before activation before before init
    }
}