<?php

namespace ShopWP;

use ShopWP\Utils;


if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

if ( class_exists( 'ShopWP_Permalink_Settings', false ) ) {
	return new ShopWP_Permalink_Settings();
}

class ShopWP_Permalink_Settings {

	private $product_permalink = '';
	private $collection_permalink = '';
	public $DB_Settings_General;

	public function __construct() {
		$this->settings_init();
		$this->settings_save();
	}

	public function settings_init() {
		add_settings_section( 'shopwp-permalink', __( 'ShopWP Product permalinks', 'shopwp' ), array( $this, 'settings' ), 'permalink' );

		add_settings_field(
			'shopwp_product_base_slug',
			__( 'Product tag base', 'shopwp' ),
			array( $this, 'product_tag_slug_input' ),
			'permalink',
			'optional'
		);

		add_settings_field(
			'shopwp_collection_base_slug',
			__( 'Collection tag base', 'shopwp' ),
			array( $this, 'collection_tag_slug_input' ),
			'permalink',
			'optional'
		);

		$permalinks 					= Utils::get_permalinks();
		
		$this->product_permalink 		= $permalinks['products'];
		$this->collection_permalink 	= $permalinks['collections'];

		$this->DB_Settings_General 		= \ShopWP\Factories\DB\Settings_General_Factory::build();

	}

	public function product_tag_slug_input() {
		?>
		<input name="shopwp_product_base_slug" type="text" class="regular-text code" value="<?php echo \esc_attr( $this->product_permalink ); ?>" placeholder="<?php echo \esc_attr_x( 'product-tag', 'slug', 'shopwp' ); ?>" />
		<?php
	}

	public function collection_tag_slug_input() {
		?>
		<input name="shopwp_collection_base_slug" type="text" class="regular-text code" value="<?php echo \esc_attr( $this->collection_permalink ); ?>" placeholder="<?php echo \esc_attr_x( 'collection-tag', 'slug', 'shopwp' ); ?>" />
		<?php
	}

	public function settings() {
		/* translators: %s: Home URL */
		echo \wp_kses_post( \wpautop( sprintf( __( 'You can customize your ShopWP product and collection URLs here. You can adjust the URL base to be anything you want. For example, setting your products URL to <code>shop</code> will make your product links look like: <code>%sshop/sample-product/</code>. If you want to use a custom page for your product or collection listing pages, set the slug of your custom pages below.', 'shopwp' ), esc_url( home_url( '/' ) ) ) ) );

		$default_products_base_slug = 'products';
		$default_collections_base_slug = 'collections';

		if ( Utils::str_contains($default_products_base_slug, $this->product_permalink) ) {
			$products_is_using_shop_base = true;
		} else {
			$products_is_using_shop_base = false;
		}

		if ( Utils::str_contains($default_collections_base_slug, $this->collection_permalink) ) {
			$collections_is_using_shop_base = true;
		} else {
			$collections_is_using_shop_base = false;
		}

		?>
		<table class="form-table shopwp-permalink-structure">
			<tbody>
				<tr>
					<th><label><input name="product_permalink" type="radio" value="<?php echo \esc_attr( $default_products_base_slug ); ?>" class="product-permalink" <?php \checked($products_is_using_shop_base); ?> /> <?php \esc_html_e( 'Products default URL', 'shopwp' ); ?></label></th>
					<td><code><?php echo \esc_html( \home_url() ); ?>/<?php echo \esc_html( $default_products_base_slug ); ?>/sample-product/</code></td>
				</tr>
				<tr>
					<th><label><input name="product_permalink" id="shopwp_product_custom_selection" type="radio" value="custom" class="tog" <?php \checked(!$products_is_using_shop_base); ?> />
						<?php \esc_html_e( 'Products URL', 'shopwp' ); ?></label></th>
					<td>
						<code><?php echo \esc_html( \home_url() ); ?>/</code><input name="product_permalink_structure" id="shopwp_permalink_product_structure" type="text" value="<?php echo \esc_attr( $this->product_permalink ? $this->product_permalink : '' ); ?>" class="regular-text code"><code>/sample-product/</code> 
					</td>
				</tr>

				<tr>
					<th><label><input name="collection_permalink" type="radio" value="<?php echo \esc_attr( $default_collections_base_slug ); ?>" class="collection" <?php \checked($collections_is_using_shop_base); ?> /> <?php \esc_html_e( 'Collections default URL', 'shopwp' ); ?></label></th>
					<td><code><?php echo \esc_html( \home_url() ); ?>/<?php echo \esc_html( $default_collections_base_slug ); ?>/sample-collection/</code></td>
				</tr>
				<tr>
					<th><label><input name="collection_permalink" id="shopwp_collection_custom_selection" type="radio" value="custom" class="tog" <?php \checked(!$collections_is_using_shop_base); ?> />
						<?php \esc_html_e( 'Collections URL', 'shopwp' ); ?></label></th>
					<td>
						<code><?php echo \esc_html( \home_url() ); ?>/</code><input name="collection_permalink_structure" id="shopwp_permalink_collection_structure" type="text" value="<?php echo \esc_attr( $this->collection_permalink ? $this->collection_permalink : '' ); ?>" class="regular-text code"><code>/sample-collection/</code>
					</td>
				</tr>
			</tbody>
		</table>
		<?php \wp_nonce_field( 'shopwp-permalinks', 'shopwp-permalinks-nonce' ); ?>
		<script type="text/javascript">
			jQuery(function() {
				
				jQuery('.product-permalink').on( 'change', function() {
					jQuery('#shopwp_permalink_product_structure').val( jQuery( this ).val() );
				});
				jQuery('.collection-permalink').on( 'change', function() {
					jQuery('#shopwp_permalink_collection_structure').val( jQuery( this ).val() );
				});

				jQuery('#shopwp_permalink_product_structure').on( 'focus', function(){
					jQuery('#shopwp_product_custom_selection').trigger( 'click' );
				} );

				jQuery('#shopwp_permalink_collection_structure').on( 'focus', function(){
					jQuery('#shopwp_collection_custom_selection').trigger( 'click' );
				} );

			});
		</script>
		<?php
	}

	public function settings_save() {
		
		if ( ! is_admin() ) {
			return;
		}

		if ( isset( $_POST['permalink_structure'], $_POST['shopwp-permalinks-nonce'], $_POST['shopwp_product_base_slug']) && wp_verify_nonce( wp_unslash( $_POST['shopwp-permalinks-nonce'] ), 'shopwp-permalinks' ) ) {

			$DB_Settings_General = \ShopWP\Factories\DB\Settings_General_Factory::build();

			$product_base = isset( $_POST['product_permalink'] ) ? Utils::clean( wp_unslash( $_POST['product_permalink'] ) ) : ''; 

			if ( 'custom' === $product_base ) {
				if ( isset( $_POST['product_permalink_structure'] ) ) {
					$product_base = preg_replace( '#/+#', '/', '/' . str_replace( '#', '', trim( wp_unslash( $_POST['product_permalink_structure'] ) ) ) );
				} else {
					$product_base = '/';
				}

				// This is an invalid base structure and breaks pages.
				if ( '/%product_cat%/' === \trailingslashit( $product_base ) ) {
					$product_base = '/' . _x( 'product', 'slug', 'shopwp' ) . $product_base;
				}
			} elseif ( empty( $product_base ) ) {
				$product_base = _x( 'product', 'slug', 'shopwp' );
			}

			$DB_Settings_General->update_col('url_products', Utils::sanitize_permalink($product_base));
		}

		if ( isset( $_POST['permalink_structure'], $_POST['shopwp-permalinks-nonce'], $_POST['shopwp_collection_base_slug']) && wp_verify_nonce( wp_unslash( $_POST['shopwp-permalinks-nonce'] ), 'shopwp-permalinks' ) ) {

			$DB_Settings_General = \ShopWP\Factories\DB\Settings_General_Factory::build();

			// Generate collection base.
			$collection_base = isset( $_POST['collection_permalink'] ) ? Utils::clean( wp_unslash( $_POST['collection_permalink'] ) ) : '';

			if ( 'custom' === $collection_base ) {
				if ( isset( $_POST['collection_permalink_structure'] ) ) {
					$collection_base = preg_replace( '#/+#', '/', '/' . str_replace( '#', '', trim( wp_unslash( $_POST['collection_permalink_structure'] ) ) ) );
				} else {
					$collection_base = '/';
				}

				// This is an invalid base structure and breaks pages.
				if ( '/%collection_cat%/' === \trailingslashit( $collection_base ) ) {
					$collection_base = '/' . _x( 'collection', 'slug', 'shopwp' ) . $collection_base;
				}
			} elseif ( empty( $collection_base ) ) {
				$collection_base = _x( 'collection', 'slug', 'shopwp' );
			}

			$DB_Settings_General->update_col('url_collections', Utils::sanitize_permalink($collection_base));
		}

	}
}
