<?php

namespace ShopWP;

if (!defined('ABSPATH')) {
    exit();
}

class Config
{

    public function __construct() {}

    public function def($name, $value) {
        if (!defined($name)) {
            define($name, $value);
        }
    }

    public function init() {

        $this->def('SHOPWP_NEW_PLUGIN_VERSION', '8.6.1');
        $this->def('SHOPWP_ADMIN_REST_API_VERSION', '2024-01');
        $this->def('SHOPWP_ADMIN_GRAPHQL_API_VERSION', '2024-01');
        $this->def('SHOPWP_STOREFRONT_GRAPHQL_API_VERSION', '2024-01');
        $this->def('SHOPWP_COMPATIBILITY_PLUGIN_VERSION', '1.1.0');
        $this->def('SHOPWP_DOWNLOAD_NAME', 'ShopWP');
        $this->def('SHOPWP_PLUGIN_NAME_FULL', 'ShopWP');
        $this->def('SHOPWP_PLUGIN_NAME_FULL_PRO', 'ShopWP Pro');
        $this->def('SHOPWP_DOWNLOAD_ID', 35);
        $this->def('SHOPWP_FREE_BASENAME', 'wpshopify/shopwp.php');
        $this->def('SHOPWP_PRO_BASENAME', 'shopwp-pro/shopwp.php');
        $this->def('SHOPWP_SHOPIFY_HEADER_VERIFY_WEBHOOKS', 'HTTP_X_SHOPIFY_HMAC_SHA256');
        $this->def('SHOPWP_SHOPIFY_HEADER_API_CALL_LIMIT', 'HTTP_X_SHOPIFY_SHOP_API_CALL_LIMIT');
        $this->def('SHOPWP_SETTINGS_CONNECTION_OPTION_NAME', 'wps_settings_connection');
        $this->def('SHOPWP_SETTINGS_GENERAL_OPTION_NAME', 'wps_settings_general');
        $this->def('SHOPWP_SETTINGS_LICENSE_OPTION_NAME', 'wps_settings_license');
        $this->def('SHOPWP_TABLE_NAME_WP_OPTIONS', 'options');
        $this->def('SHOPWP_TABLE_NAME_WP_POSTS', 'posts');
        $this->def('SHOPWP_TABLE_NAME_WP_POSTMETA', 'postmeta');
        $this->def('SHOPWP_TABLE_NAME_WP_TERM_RELATIONSHIPS', 'term_relationships');
        $this->def('SHOPWP_TABLE_NAME_SETTINGS_LICENSE', 'wps_settings_license');
        $this->def('SHOPWP_TABLE_NAME_SETTINGS_GENERAL', 'wps_settings_general');
        $this->def('SHOPWP_TABLE_NAME_SETTINGS_CONNECTION', 'wps_settings_connection');
        $this->def('SHOPWP_TABLE_NAME_SETTINGS_SYNCING', 'wps_settings_syncing');
        $this->def('SHOPWP_TABLE_NAME_PRODUCTS', 'shopwp_products');
        $this->def('SHOPWP_TABLE_NAME_METAFIELDS', 'shopwp_metafields');
        $this->def('SHOPWP_TABLE_NAME_ORDERS', 'shopwp_orders');
        $this->def('SHOPWP_TABLE_NAME_OPTIONS', 'shopwp_options');
        $this->def('SHOPWP_TABLE_NAME_CUSTOMERS', 'shopwp_customers');
        $this->def('SHOPWP_TABLE_NAME_COLLECTS', 'shopwp_collects');
        $this->def('SHOPWP_TABLE_NAME_COLLECTIONS', 'shopwp_collections');
        $this->def('SHOPWP_TABLE_NAME_COLLECTIONS_SMART', 'shopwp_collections_smart');
        $this->def('SHOPWP_TABLE_NAME_COLLECTIONS_CUSTOM', 'shopwp_collections_custom');
        $this->def('SHOPWP_TABLE_NAME_TAGS', 'shopwp_tags');
        $this->def('SHOPWP_TABLE_NAME_SHOP', 'shopwp_shop');
        $this->def('SHOPWP_TABLE_NAME_IMAGES', 'shopwp_images');
        $this->def('SHOPWP_TABLE_NAME_VARIANTS', 'shopwp_variants');
        $this->def('SHOPWP_TABLE_MIGRATION_SUFFIX', '_migrate');
        $this->def('SHOPWP_TABLE_MIGRATION_SUFFIX_TESTS', '_migrate_tests');
        $this->def('SHOPWP_SHOPIFY_PAYLOAD_KEY', 'id');
        $this->def('SHOPWP_PRODUCTS_LOOKUP_KEY', 'productId');
        $this->def('SHOPWP_COLLECTIONS_LOOKUP_KEY', 'collectionId');
        $this->def('SHOPWP_PLUGIN_NAME', 'wps');
        $this->def('SHOPWP_NEW_PLUGIN_AUTHOR', 'ShopWP');
        $this->def('SHOPWP_PLUGIN_URL', plugin_dir_url(__DIR__));
        $this->def('SHOPWP_PLUGIN_DIR_PATH', plugin_dir_path(__DIR__));
        $this->def('SHOPWP_PLUGIN_ENV', 'https://wpshop.io');
        $this->def('SHOPWP_SHOPIFY_RATE_LIMIT', '39/40');
        $this->def('SHOPWP_LANGUAGES_FOLDER', 'languages');
        $this->def('SHOPWP_SHOPIFY_API_SLUG', 'shopwp');
        $this->def('SHOPWP_SHOPIFY_API_VERSION', 'v1');
        $this->def('SHOPWP_SHOPIFY_API_NAMESPACE', SHOPWP_SHOPIFY_API_SLUG . '/' . SHOPWP_SHOPIFY_API_VERSION);
        $this->def('SHOPWP_RELATIVE_TEMPLATE_DIR', 'public/templates');
        $this->def('SHOPWP_PRODUCTS_POST_TYPE_SLUG', 'wps_products');
        $this->def('SHOPWP_ORDERS_POST_TYPE_SLUG', 'wps_orders');
        $this->def('SHOPWP_COLLECTIONS_POST_TYPE_SLUG', 'wps_collections');
        $this->def('SHOPWP_PRODUCTS_DEFAULT_CONNECTIVE', 'OR');
        $this->def('SHOPWP_COLLECTIONS_DEFAULT_CONNECTIVE', 'OR');
        $this->def('SHOPWP_DEFAULT_PRODUCTS_PAGE_SLUG', 'products');
        $this->def('SHOPWP_DEFAULT_COLLECTIONS_PAGE_SLUG', 'collections');
        $this->def('SHOPWP_AVAILABLE_WEBHOOKS', [
            [
                'label' => 'Product added',
                'value' => 'PRODUCT_LISTINGS_ADD',
            ],
            [
                'label' => 'Product update',
                'value' => 'PRODUCT_LISTINGS_UPDATE',
            ],
            [
                'label' => 'Product removed',
                'value' => 'PRODUCT_LISTINGS_REMOVE',
            ],
            [
                'label' => 'Checkout create',
                'value' => 'CHECKOUTS_CREATE',
            ],
            [
                'label' => 'Collection added',
                'value' => 'COLLECTION_LISTINGS_ADD',
            ],
            [
                'label' => 'Collection update',
                'value' => 'COLLECTION_LISTINGS_UPDATE',
            ],
            [
                'label' => 'Collection removed',
                'value' => 'COLLECTION_LISTINGS_REMOVE',
            ],
            [
                'label' => 'Customer create',
                'value' => 'CUSTOMERS_CREATE',
            ],
            [
                'label' => 'Customer update',
                'value' => 'CUSTOMERS_UPDATE',
            ],
            [
                'label' => 'Order create',
                'value' => 'ORDERS_CREATE',
            ],
            [
                'label' => 'Order delete',
                'value' => 'ORDERS_DELETE',
            ],
            [
                'label' => 'Order edited',
                'value' => 'ORDERS_EDITED',
            ],
            [
                'label' => 'Shop update',
                'value' => 'SHOP_UPDATE',
            ]
        ]);
    }
}





