<?php

namespace ShopWP;

use ShopWP\Utils;
use ShopWP\CPT;

if (!defined('ABSPATH')) {
    exit();
}

class CPT_Model
{
 
    public function set_collection_model_defaults($collection, $settings)
    {

        $defaults = [
            'post_title' => isset($collection->title) ? $collection->title : '',
            'post_content' => isset($collection->descriptionHtml) ? $collection->descriptionHtml : (isset($collection->body_html) ? $collection->body_html : ''),
            'post_name' => isset($collection->handle) ? sanitize_title($collection->handle) : false,
            'post_date' => isset($collection->published_at) ? $collection->published_at : '',
            'post_type' => SHOPWP_COLLECTIONS_POST_TYPE_SLUG,
            'post_status' => 'publish',
            'post_author' => CPT::return_author_id(),
            'meta_input' => [
                'collection_id' => isset($collection->id) ? Utils::create_numeric_id($collection->id, 'Collection') : (isset($collection->collection_id) ? $collection->collection_id : '')
            ]
        ];

        if ($settings->sync_into_post_meta) {
            $defaults['meta_input']['collection_seo_title']             = isset($collection->seo) ? $collection->seo->title : false;
            $defaults['meta_input']['collection_seo_description']       = isset($collection->seo) ? $collection->seo->description : false;
            $defaults['meta_input']['collection_products_count']        = isset($collection->productsCount) ? $collection->productsCount : false;
            $defaults['meta_input']['collection_default_sort_order']    = isset($collection->sortOrder) ? $collection->sortOrder : false;
            $defaults['meta_input']['collection_updated_at']            = isset($collection->updatedAt) ? $collection->updatedAt : false;
        }

        return $defaults;
    }

    public function get_product_status($product, $settings) {
        if (isset($product->status)) {
            
            if ($product->status === 'ACTIVE') {
                $post_status = 'publish';

            } else if ($product->status === 'ARCHIVED') {
                $post_status = 'draft';

            } else if ($product->status === 'DRAFT') {
                $post_status = 'draft';
            }

        } else {
            $post_status = 'publish';
        }

        if (!empty($settings)) {

            if ($settings->sync_products_as_draft) {
                $post_status = 'draft';
            }
        }

        return $post_status;
    }

    public function set_product_model_defaults($product, $settings)
    {
        
        $post_status = $this->get_product_status($product, $settings);

        $product = Utils::convert_array_to_object($product);

        $info = [
            'post_title'        => isset($product->title) ? $product->title : '',
            'post_content'      => isset($product->descriptionHtml) ? $product->descriptionHtml : '',
            'post_name'         => isset($product->handle) ? $product->handle : sanitize_title($product->title),
            'post_date'         => isset($product->publishedAt) ? $product->publishedAt : '',
            'post_author'       => CPT::return_author_id(),
            'post_status'       => $post_status,
            'post_type'         => SHOPWP_PRODUCTS_POST_TYPE_SLUG,
            'meta_input'        => [
                'product_id' => isset($product->id) ? Utils::create_numeric_id($product->id) : ''
            ]
        ];

        if ($settings->sync_into_post_meta) {
            $info['meta_input']['product_seo_title']                    = isset($product->seo) ? $product->seo->title : '';
            $info['meta_input']['product_seo_description']              = isset($product->seo) ? $product->seo->description : '';
            $info['meta_input']['product_tags']                         = isset($product->tags) ? implode (", ", $product->tags) : '';
            $info['meta_input']['product_vendor']                       = isset($product->vendor) ? $product->vendor : '';
            $info['meta_input']['product_total_inventory']              = isset($product->totalInventory) ? $product->totalInventory : '';
            $info['meta_input']['product_total_variants']               = isset($product->totalVariants) ? $product->totalVariants : '';
            $info['meta_input']['product_type']                         = isset($product->productType) ? $product->productType : '';
            $info['meta_input']['product_is_gift_card']                 = isset($product->isGiftCard) ? $product->isGiftCard : '';
            $info['meta_input']['product_has_out_of_stock_variants']    = isset($product->onlineStoreUrl) ? $product->hasOutOfStockVariants : '';
            $info['meta_input']['product_shopify_url']                  = isset($product->onlineStoreUrl) ? $product->onlineStoreUrl : '';
            $info['meta_input']['product_requires_subscription']        = isset($product->requiresSellingPlan) ? $product->requiresSellingPlan : '';
        }

        return $info;

    }

    public function build_collections_model_for_update($collection, $post_id, $settings)
    {

        $collection_model = $this->set_collection_model_defaults($collection, $settings);
        
        $collection_model = $this->set_post_id_if_exists(
            $collection_model,
            $post_id
        );

        return $collection_model;
    }

    public function set_existing_product_categories($post_id, $product_model)
    {
        $categories = get_the_category($post_id);
        $cat_ids = [];

        if (empty($categories)) {
            return $product_model;
        }

        foreach ($categories as $category) {
            array_push($cat_ids, $category->cat_ID);
        }

        $product_model['post_category'] = $cat_ids;

        return $product_model;
    }

    public function set_existing_product_excerpt($post_id, $product_model)
    {
        if (!has_excerpt($post_id)) {
            return $product_model;
        }

        $product_model['post_excerpt'] = get_the_excerpt($post_id);

        return $product_model;
    }

    public function set_post_id_if_exists($model, $existing_post_id)
    {
        if (!empty($existing_post_id)) {
            $model['ID'] = $existing_post_id;
        }

        return $model;
    }

    public function set_full_sync_fields($product, $product_model) {

        if (!empty($product->metafields)) {
            foreach ($product->metafields as $metafield) {
                if ($metafield->key !== 'description_tag' && $metafield->key !== 'title_tag') {
                    $product_model['meta_input'][$metafield->key] = $metafield->value;
                }
            }
        }

        return $product_model;

    }

    public function build_products_model_for_update($product, $post_id = false, $full_sync = false, $settings = false)
    {

        $product_model = $this->set_product_model_defaults($product, $settings);

        $product_model = $this->set_post_id_if_exists($product_model, $post_id);

        $product_model = $this->set_existing_product_categories(
            $post_id,
            $product_model
        );

        $product_model = $this->set_existing_product_excerpt(
            $post_id,
            $product_model
        );

        if ($full_sync) {
            $product_model = $this->set_full_sync_fields(
                $product,
                $product_model
            );
        }

        return $product_model;
    }

    public function insert_or_update_product_post($product, $full_sync = false, $settings = false)
    {

        $numeric_id = Utils::create_numeric_id($product->id);
        
        $post_id = $this->get_post_id_by_item_id($numeric_id);

        $model = $this->build_products_model_for_update($product, $post_id, $full_sync, $settings);

        $post_id_updated = \wp_insert_post($model, true);

        if (!empty($product->tags)) {
            $result = \wp_set_object_terms($post_id_updated, $product->tags, 'shopify_tags');    

            if (is_wp_error($result)) {
                return [
                    'wp_error' => $result->get_error_message(),
                    'post_id' => $post_id_updated
                ];
            }
        }

        if (!empty($product->vendor)) {
            $result = \wp_set_object_terms($post_id_updated, $product->vendor, 'shopify_vendor', true);

            if (is_wp_error($result)) {
                return [
                    'wp_error' => $result->get_error_message(),
                    'post_id' => $post_id_updated
                ];
            }
        }

        if (!empty($product->productType)) {
            $result = \wp_set_object_terms($post_id_updated, $product->productType, 'shopify_product_type', true);

            if (is_wp_error($result)) {
                return [
                    'wp_error' => $result->get_error_message(),
                    'post_id' => $post_id_updated
                ];
            }
        }


        if (isset($product->__parentId)) {

            if (Utils::str_contains($product->__parentId, 'gid://shopify/Collection')) {

                $collection_id = str_replace('gid://shopify/Collection/', '', $product->__parentId);

                // Insert the collectionId into the product record 
                
                $result = \wp_set_object_terms($post_id_updated, $collection_id, 'shopify_collections', true);

                if (is_wp_error($result)) {
                    return [
                        'wp_error' => $result->get_error_message(),
                        'post_id' => $post_id_updated
                    ];
                }    
            }
            
        }

        if (is_array($post_id_updated)) {
            
            if (!empty($post_id_updated['wp_error'])) {

                $this->DB_Settings_Syncing->log($post_id_updated['wp_error'], 'error');

                return Utils::wp_error($post_id_updated['wp_error']);

            } else {
                return $post_id_updated['post_id'];
            }
        }

        return $post_id_updated;

    }

    public function insert_or_update_collection_post($collection, $settings = false) {

        $numeric_id = Utils::create_numeric_id($collection->id, 'Collection');

        $term = \get_term_by('slug', $numeric_id, 'shopify_collections');

        if (!empty($term)) {

            if (!\is_wp_error($term)) {

                $update = \wp_update_term($term->term_id, 'shopify_collections', [
                    'name' => $collection->title
                ] );

                if (\is_wp_error($update)) {
                    $this->DB_Settings_Syncing->log($update->get_error_message(), 'error');
                }
            }
        }

        $post_id = $this->get_post_id_by_item_id($numeric_id, 'collection');

        $model = $this->build_collections_model_for_update(
            $collection,
            $post_id,
            $settings
        );

        $post_id = wp_insert_post($model, true);

        if (is_array($post_id)) {
            if (!empty($post_id['wp_error'])) {
                return Utils::wp_error($post_id['wp_error']);

            } else {
                return $post_id['post_id'];
            }
        }

        return $post_id;
    }

    public function get_post_id_by_item_id($item_id, $type = 'product')
    {
        $post = \get_posts([
            'post_type'         => 'wps_' . $type . 's',
            'posts_per_page'    => 1,
            'meta_key'          => $type . '_id',
            'meta_value'        => $item_id,
            'fields'            => 'ids',
            'post_status'       => 'any'
        ]);

        if (empty($post)) {
            $post_id = false;
            
        } else {
            $post_id = $post[0];
        }

        return $post_id;
    }

}
