<?php

namespace ShopWP;

use ShopWP\Utils;
use ShopWP\Utils\Data as Utils_Data;
use ShopWP\Transients;

if (!defined('ABSPATH')) {
    exit();
}

class CPT
{
    public $DB_Settings_General;
    public $API_Settings_General;
    public $plugin_settings;

    public function __construct($DB_Settings_General, $API_Settings_General, $plugin_settings)
    {
        $this->DB_Settings_General      = $DB_Settings_General;
        $this->API_Settings_General     = $API_Settings_General;
        $this->plugin_settings          = $plugin_settings;
    }

    public static function return_author_id()
    {
        if (get_current_user_id() === 0) {
            $author_id = 1;
        } else {
            $author_id = get_current_user_id();
        }

        return intval($author_id);
    }

    public static function set_post_id($post, $post_id)
    {
        $post->post_id = $post_id;

        return $post;
    }

    public static function add_props($item, $props)
    {
        foreach ($props as $key => $value) {
            $item->{$key} = $value;
        }

        return $item;
    }

    public static function add_props_to_items($items, $props)
    {
        return array_map(function ($item) use ($props) {
            return self::add_props($item, $props);
        }, $items);
    }

    public function post_type_products($permalinks)
    {
        if (post_type_exists(SHOPWP_PRODUCTS_POST_TYPE_SLUG)) {
            return;
        }

        $rewrite_rules = [
            'slug' => $permalinks['products'],
            'with_front' => false,
            'feeds' => true,
        ];

        $labels = [
            'name'                  => __('Products', 'shopwp'),
            'singular_name'         => __('Product', 'shopwp'),
            'menu_name'             => __('Products', 'shopwp'),
            'new_item'              => __('Add New Product', 'shopwp'),
            'edit_item'             => __('Edit Product', 'shopwp'),
            'not_found'             => __('No Products found', 'shopwp'),
            'view_item'             => __('View product', 'shopwp'),
            'view_items'            => __('View products', 'shopwp'),
            'not_found_in_trash'    => __('No Products found in trash', 'shopwp'),
        ];

        $args = [
            'label'                 => __('Products', 'shopwp'),
            'description'           => __('Custom Post Type for Products', 'shopwp'),
            'labels' => $labels,
            'supports' => [
                'title',
                'editor',
                'page-attributes',
                'custom-fields',
                'thumbnail',
            ],
            'taxonomies' => ['shopify_tags', 'shopify_collections', 'shopify_vendor', 'shopify_product_type'],
            'hierarchical' => false,
            'public' => true,
            'show_ui' => true,
            'show_in_menu' => false,
            'menu_position' => 100,
            'menu_icon' => 'dashicons-megaphone',
            'show_in_admin_bar' => true,
            'show_in_nav_menus' => true,
            'can_export' => true,
            'show_in_rest' => true,
            'has_archive' => $this->plugin_settings['general']['enable_products_page'],
            'exclude_from_search' => false,
            'publicly_queryable' => true,
            'capability_type' => 'post',
            'rewrite' => $rewrite_rules,
            'capabilities' => [
                'create_posts' => false,
            ],
            'map_meta_cap' => true,
        ];

        register_post_type(
            SHOPWP_PRODUCTS_POST_TYPE_SLUG,
            apply_filters('shopwp_register_products_args', $args)
        );

        $fields = [
            'product_total_inventory',
            'product_total_variants',
            'product_id',
        ];

        $args = array('show_in_rest' => true, 'type' => 'string', 'single' => true);
        
        foreach($fields as $field){
            register_post_meta('wps_products', $field, $args);
        }

    }

    public function post_type_collections($permalinks)
    {
        if (post_type_exists(SHOPWP_COLLECTIONS_POST_TYPE_SLUG)) {
            return;
        }

        $rewrite_rules = [
            'slug' => $permalinks['collections'],
            'with_front' => false,
            'feeds' => true,
        ];
        
        $exclude_from_search = false;

        $labels = [
            'name'                  => __('Collections', 'shopwp'),
            'singular_name'         => __('Collection', 'shopwp'),
            'menu_name'             => __('Collections', 'shopwp'),
            'parent_item_colon'     => __('Parent Item:', 'shopwp'),
            'new_item'              => __('Add New Collection', 'shopwp'),
            'edit_item'             => __('Edit Collection', 'shopwp'),
            'not_found'             => __('No Collections found', 'shopwp'),
            'view_item'             => __('View collection', 'shopwp'),
            'view_items'            => __('View collections', 'shopwp'),
            'not_found_in_trash'    => __('No Collections found in trash', 'shopwp'),
        ];

        $args = [
            'label'                 => __('Collections', 'shopwp'),
            'description'           => __('Custom Post Type for Collections', 'shopwp'),
            'labels' => $labels,
            'supports' => [
                'title',
                'editor',
                'page-attributes',
                'custom-fields',
                'thumbnail',
            ],
            'hierarchical' => false,
            'public' => true,
            'show_ui' => true,
            'show_in_menu' => false,
            'menu_position' => 100,
            'menu_icon' => 'dashicons-megaphone',
            'show_in_admin_bar' => true,
            'show_in_nav_menus' => true,
            'can_export' => true,
            'has_archive' => $this->plugin_settings['general']['enable_collections_page'],
            'show_in_rest' => true,
            'exclude_from_search' => $exclude_from_search,
            'publicly_queryable' => true,
            'capability_type' => 'post',
            'rewrite' => $rewrite_rules,
            'capabilities' => [
                'create_posts' => false,
            ],
            'map_meta_cap' => true,
        ];

        register_post_type(
            SHOPWP_COLLECTIONS_POST_TYPE_SLUG,
            apply_filters('shopwp_register_collections_args', $args)
        );

        $fields = [
            'collection_id',
        ];

        $args = array('show_in_rest' => true, 'type' => 'string', 'single' => true);
        
        foreach($fields as $field){
            register_post_meta('wps_collections', $field, $args);
        }

    }

    public function post_type_layouts()
    {

        \register_post_type(
            'shopwp_shortcodes',
            [
                'label' => __('ShopWP Layouts', 'shopwp'),
                'description' => __('Allows for creating layouts using a visual builder to embed ShopWP products', 'shopwp'),
                'labels' => [
                    'name'                  => __('ShopWP Layouts', 'shopwp'),
                    'singular_name'         => __('Layout', 'shopwp'),
                    'add_new_item'          => __('Add new layout', 'shopwp'),
                    'add_new'               => __('Add new layout', 'shopwp'),
                    'edit_item'             => __('Edit layout', 'shopwp'),
                    'new_item'              => __('Add new layout', 'shopwp'),
                    'view_item'             => __('View layout', 'shopwp'),
                    'view_items'            => __('View layouts', 'shopwp'),
                    'search_items'          => __('Search layouts', 'shopwp'),
                    'not_found'             => __('No layout found', 'shopwp'),
                    'not_found_in_trash'    => __('No layout found in trash', 'shopwp'),
                    'all_items'             => __('All layouts', 'shopwp'),
                    'insert_into_item'      => __('Insert into layout', 'shopwp'),
                    'menu_name'             => __('layouts', 'shopwp'),
                    'item_published'        => __('Layout published', 'shopwp'),
                    'item_updated'          => __('Layout updated', 'shopwp'),
                ],
                'supports' => [
                    'title',
                    'editor',
                    'custom-fields'
                ],
                'template_lock' => false,
                'template' => [
                    [
                        'core/columns',
                        [
                            'align' => 'wide',
                            'locked' => true, 
                            'lock' => [
                                'move' => true, 
                                'remove' => true
                            ]
                        ],
                        [
                            [
                                'core/column',
                                [
                                    'locked' => true, 
                                    'lock' => [
                                        'move' => true, 
                                        'remove' => true
                                    ]
                                ],
                                [
                                    [
                                        'shopwp/products',
                                        [
                                            'locked' => true, 
                                            'lock' => [
                                                'move' => true, 
                                                'remove' => true
                                            ]
                                        ]
                                    ],
                                ],
                            ],
                        ],
                    ]
                ],
                'taxonomies' => [],
                'hierarchical' => false,
                'public' => true,
                'show_ui' => true,
                'show_in_menu' => false,
                'menu_position' => 100,
                'menu_icon' => 'dashicons-megaphone',
                'show_in_admin_bar' => true,
                'show_in_nav_menus' => true,
                'can_export' => false,
                'show_in_rest' => true,
                'has_archive' => false,
                'exclude_from_search' => true,
                'publicly_queryable' => false,
                'capability_type' => 'post',
                'rewrite' => [
                    'slug' => 'shopwp-shortcode',
                    'with_front' => false,
                    'feeds' => true,
                ],
                'map_meta_cap' => true,
            ]
        );

        \register_post_meta(
            'shopwp_shortcodes',
            'layout_type',
            [
                'auth_callback' => '__return_true',
                'show_in_rest'  => true,
                'single'        => true,
                'type'          => 'string',
                'default'       => 'shortcode',
            ]
        );

        \register_post_meta(
            'shopwp_shortcodes',
            'layout_shortcode_type',
            [
                'auth_callback' => '__return_true',
                'show_in_rest'  => true,
                'single'        => true,
                'type'          => 'string',
                'default'       => '[wps_products]',
            ]
        );

        \register_post_meta(
            'shopwp_shortcodes',
            'layout_preview_post_id',
            [
                'auth_callback' => '__return_true',
                'show_in_rest'  => true,
                'single'        => true,
                'type'          => 'integer',
                'default'       => 0,
            ]
        );

        \register_post_meta(
            'shopwp_shortcodes',
            'layout_preview_post_url',
            [
                'auth_callback' => '__return_true',
                'show_in_rest'  => true,
                'single'        => true,
                'type'          => 'string',
                'default'       => '',
            ]
        );

        \register_post_meta(
            'shopwp_shortcodes',
            'layout_preview_product_id',
            [
                'auth_callback' => '__return_true',
                'show_in_rest'  => true,
                'single'        => true,
                'type'          => 'integer',
                'default'       => 0,
            ]
        );

        \register_post_meta(
            'shopwp_shortcodes',
            'layout_preview_collection_id',
            [
                'auth_callback' => '__return_true',
                'show_in_rest'  => true,
                'single'        => true,
                'type'          => 'integer',
                'default'       => 0,
            ]
        );

        \register_post_meta(
            'shopwp_shortcodes',
            'layout_preview_collection_title',
            [
                'auth_callback' => '__return_true',
                'show_in_rest'  => true,
                'single'        => true,
                'type'          => 'string',
                'default'       => '',
            ]
        );

    }

    public function register_custom_taxonomies()
    {

        if ( !taxonomy_exists('shopify_tags') ) {

            register_taxonomy('shopify_tags', ['wps_products'], apply_filters('shopwp_register_shopify_product_tag_tax', [
                'label' => __('Shopify Tags', 'shopwp'),
                'hierarchical' => false,
                'public' => true,
                'description' => 'Shopify Tags',
                'show_admin_column' => true,
                'show_ui' => true,
                'show_in_rest' => true,
                'show_in_menu' => true,
                'rewrite' => ['slug' => 'product-tag'],
                'labels' => [
                    'name'                       => _x( 'Shopify Tags', 'taxonomy general name', 'shopwp' ),
                    'singular_name'              => _x( 'Shopify Tag', 'taxonomy singular name', 'shopwp' ),
                    'search_items'               => __( 'Search Shopify Tags', 'shopwp' ),
                    'popular_items'              => __( 'Popular Shopify Tags', 'shopwp' ),
                    'all_items'                  => __( 'All Shopify Tags', 'shopwp' ),
                    'edit_item'                  => __( 'Edit Shopify Tag', 'shopwp' ),
                    'update_item'                => __( 'Update Shopify Tag', 'shopwp' ),
                    'add_new_item'               => __( 'Add New Tag', 'shopwp' ),
                    'new_item_name'              => __( 'New Shopify Tag Name', 'shopwp' ),
                    'separate_items_with_commas' => __( 'Separate Shopify Tags with commas', 'shopwp' ),
                    'add_or_remove_items'        => __( 'Add or remove Shopify Tag', 'shopwp' ),
                    'choose_from_most_used'      => __( 'Choose from the most used Shopify Tag', 'shopwp' ),
                    'not_found'                  => __( 'No Shopify Tag found.', 'shopwp' ),
                    'menu_name'                  => __( 'Shopify Tags', 'shopwp' ),
                ]
            ]));

            register_taxonomy_for_object_type('shopify_tags', 'wps_products');
        }

        
        if ( !taxonomy_exists('shopify_vendor') ) {
            register_taxonomy('shopify_vendor', ['wps_products'], apply_filters('shopwp_register_shopify_product_vendor_tax', [
                'label' => __('Shopify Vendor', 'shopwp'),
                'hierarchical' => false,
                'public' => true,
                'description' => 'Shopify Vendor',
                'show_admin_column' => true,
                'show_ui' => true,
                'show_in_rest' => true,
                'show_in_menu' => true,
                'rewrite' => ['slug' => 'product-vendor'],
                'labels' => [
                    'name'                       => _x( 'Shopify Vendors', 'taxonomy general name', 'shopwp' ),
                    'singular_name'              => _x( 'Shopify Vendor', 'taxonomy singular name', 'shopwp' ),
                    'search_items'               => __( 'Search Shopify Vendors', 'shopwp' ),
                    'popular_items'              => __( 'Popular Shopify Vendors', 'shopwp' ),
                    'all_items'                  => __( 'All Shopify Vendors', 'shopwp' ),
                    'edit_item'                  => __( 'Edit Shopify Vendor', 'shopwp' ),
                    'update_item'                => __( 'Update Shopify Vendor', 'shopwp' ),
                    'add_new_item'               => __( 'Add New Vendor', 'shopwp' ),
                    'new_item_name'              => __( 'New Shopify Vendor Name', 'shopwp' ),
                    'separate_items_with_commas' => __( 'Separate Shopify Vendors with commas', 'shopwp' ),
                    'add_or_remove_items'        => __( 'Add or remove Shopify Vendor', 'shopwp' ),
                    'choose_from_most_used'      => __( 'Choose from the most used Shopify Vendor', 'shopwp' ),
                    'not_found'                  => __( 'No Shopify Vendor found.', 'shopwp' ),
                    'menu_name'                  => __( 'Shopify Vendors', 'shopwp' ),
                ]
            ]));

            register_taxonomy_for_object_type('shopify_vendor', 'wps_products');
        }


        if ( !taxonomy_exists('shopify_product_type') ) {
            register_taxonomy('shopify_product_type', ['wps_products'], apply_filters('shopwp_register_shopify_product_type_tax', [
                'label' => __('Shopify Product Type', 'shopwp'),
                'hierarchical' => false,
                'public' => true,
                'description' => 'Shopify Product Type',
                'show_admin_column' => true,
                'show_ui' => true,
                'show_in_rest' => true,
                'show_in_menu' => true,
                'rewrite' => ['slug' => 'product-type'],
                'labels' => [
                    'name'                       => _x( 'Shopify Product Types', 'taxonomy general name', 'shopwp' ),
                    'singular_name'              => _x( 'Shopify Product Type', 'taxonomy singular name', 'shopwp' ),
                    'search_items'               => __( 'Search Product Types', 'shopwp' ),
                    'popular_items'              => __( 'Popular Product Types', 'shopwp' ),
                    'all_items'                  => __( 'All Product Types', 'shopwp' ),
                    'edit_item'                  => __( 'Edit Product Type', 'shopwp' ),
                    'update_item'                => __( 'Update Product Type', 'shopwp' ),
                    'add_new_item'               => __( 'Add New Product Type', 'shopwp' ),
                    'new_item_name'              => __( 'New Product Type Name', 'shopwp' ),
                    'separate_items_with_commas' => __( 'Separate Product Types with commas', 'shopwp' ),
                    'add_or_remove_items'        => __( 'Add or remove Product Type', 'shopwp' ),
                    'choose_from_most_used'      => __( 'Choose from the most used Product Type', 'shopwp' ),
                    'not_found'                  => __( 'No Product Type found.', 'shopwp' ),
                    'menu_name'                  => __( 'Shopify Product Types', 'shopwp' ),
                ]
            ]));

            register_taxonomy_for_object_type('shopify_vendor', 'wps_products');
        }

        if ( !taxonomy_exists('shopify_collections') ) {

            register_taxonomy('shopify_collections', ['wps_products'], apply_filters('shopwp_register_shopify_collections_tax', [
                'label' => __('Shopify Collections', 'shopwp'),
                'hierarchical' => false,
                'public' => true,
                'description' => 'Shopify Collections',
                'show_admin_column' => true,
                'show_ui' => true,
                'show_in_rest' => true,
                'show_in_menu' => true,
                'rewrite' => ['slug' => 'collection'],
                'labels' => [
                    'name'                       => _x( 'Shopify Collections', 'taxonomy general name', 'shopwp' ),
                    'singular_name'              => _x( 'Shopify Collection', 'taxonomy singular name', 'shopwp' ),
                    'search_items'               => __( 'Search Collections', 'shopwp' ),
                    'popular_items'              => __( 'Popular Collections', 'shopwp' ),
                    'all_items'                  => __( 'All Collections', 'shopwp' ),
                    'edit_item'                  => __( 'Edit Collection', 'shopwp' ),
                    'update_item'                => __( 'Update Collection', 'shopwp' ),
                    'add_new_item'               => __( 'Add New Collection', 'shopwp' ),
                    'new_item_name'              => __( 'New Collection Name', 'shopwp' ),
                    'separate_items_with_commas' => __( 'Separate Shopify Collections with commas', 'shopwp' ),
                    'add_or_remove_items'        => __( 'Add or remove Shopify Collections', 'shopwp' ),
                    'choose_from_most_used'      => __( 'Choose from the most used Shopify Collections', 'shopwp' ),
                    'not_found'                  => __( 'No Shopify Collections found.', 'shopwp' ),
                    'menu_name'                  => __( 'Shopify Collections', 'shopwp' ),
                ]
            ]));

            register_taxonomy_for_object_type('shopify_collections', 'wps_products');
        }

    }


    public function hide_layout_builder_title() {
        if (current_user_can('subscriber')) {
            remove_post_type_support('shopwp_shortcodes', 'title');
        }
    }

    
    
 
    public function allowed_block_types_all( $allowed_blocks ) {

        $screen = \get_current_screen();

        if ($screen->post_type === 'wps_products' || $screen->post_type === 'wps_collections') {
            return [];
        }

        return $allowed_blocks;
    }


    public function change_default_layout_title_text($title) {
        
        $screen = \get_current_screen();

        if  ('shopwp_shortcodes' == $screen->post_type ) {
            $title = 'Enter a name for this layout';
        }

        return $title;
    }

    public function add_shortcode_column_to_post_list($columns) {
        $custom_col_order = [
            'cb' => $columns['cb'],
            'title' => $columns['title'],
            'shortcode' => __( 'Shortcode or layout type', 'shopwp' ),
            'date' => $columns['date']
        ];
        
        return $custom_col_order;
    }


    public function add_shortcode_column_value_to_post_list($column_key, $post_id) {

        $post = \get_post($post_id);

        $is_default_products_page_enabled       = $this->plugin_settings['general']['enable_products_page'];
        $is_default_collections_page_enabled    = $this->plugin_settings['general']['enable_collections_page'];

        if (empty($post->post_content)) {
            echo "<p>No shortcode found</p>";
            return;
        }

        $layout_type = get_post_meta($post->ID, 'layout_type', true);
        $layout_shortcode_type = get_post_meta($post->ID, 'layout_shortcode_type', true);

        if (empty($layout_shortcode_type)) {
            $layout_shortcode_type = '[wps_products]';
        }

        if ($post->post_status === 'draft') {
            $badge_disabled = 'badge-disabled';
        } else {
            $badge_disabled = '';
        }

        if ($layout_type === 'template-pdp') {
            echo '<p class="badge badge-pdp ' . $badge_disabled . '">PDP Template</p>';
            
            if ($post->post_status === 'draft') {
                echo '<span style="font-style:italic;position: relative;top: -10px;left: 10px;color: #7d7c7c;">(This template is not currently being used)</span>';
            }

        } else if ($layout_type === 'template-plp') {

            echo '<p class="badge badge-plp ' . $badge_disabled . '">PLP Template</p>';

            if (empty($is_default_products_page_enabled)) {
                echo '<span style="font-style:italic;position: relative;top: -10px;left: 10px;color: #7d7c7c;">(This layout won\'t be used because the <a href="/wp-admin/admin.php?page=wps-settings">default products page</a> is disabled.)</span>';
            } else {
                if ($post->post_status === 'draft') {
                    echo '<span style="font-style:italic;position: relative;top: -10px;left: 10px;color: #7d7c7c;">(This template is not currently being used)</span>';
                }
            }

        } else if ($layout_type === 'template-clp') {

            echo '<p class="badge badge-plp ' . $badge_disabled . '">CLP Template</p>';

            if (empty($is_default_collections_page_enabled)) {
                echo '<span style="font-style:italic;position: relative;top: -10px;left: 10px;color: #7d7c7c;">(This layout won\'t be used because the <a href="/wp-admin/admin.php?page=wps-settings">default collections page</a> is disabled.)</span>';
            } else {
                if ($post->post_status === 'draft') {
                    echo '<span style="font-style:italic;position: relative;top: -10px;left: 10px;color: #7d7c7c;">(This template is not currently being used)</span>';
                }
            }

            

        } else if ($layout_type === 'template-cdp') {

            echo '<p class="badge badge-pdp ' . $badge_disabled . '">CDP Template</p>';

            if ($post->post_status === 'draft') {
                echo '<span style="font-style:italic;position: relative;top: -10px;left: 10px;color: #7d7c7c;">(This template is not currently being used)</span>';
            }

        } else {
            

            if (empty($layout_shortcode_type)) {
                $type_to_split = "<!-- wp:shopwp/products ";

            } else {
                if ($layout_shortcode_type === '[wps_products]') {
                    $type_to_split = "<!-- wp:shopwp/products ";
                } else if ($layout_shortcode_type === '[wps_cart_icon]') {
                    $type_to_split = "<!-- wp:shopwp/cart-icon ";
                } else if ($layout_shortcode_type === '[wps_storefront]') {
                    $type_to_split = "<!-- wp:shopwp/storefront ";
                } else if ($layout_shortcode_type === '[wps_collections]') {
                    $type_to_split = "<!-- wp:shopwp/collections ";
                } else if ($layout_shortcode_type === '[wps_search]') {
                    $type_to_split = "<!-- wp:shopwp/search ";
                } else {
                    $type_to_split = "<!-- wp:shopwp/products ";
                }
            }

            $onlyconsonants = \explode($type_to_split, $post->post_content);

            if (empty($onlyconsonants)) {
                echo "<p>No shortcode found</p>";
                return;
            }

            if (!isset($onlyconsonants[1])) {
                echo "<p>No shortcode found</p>";
                return;
            }

            $okok = \explode('","', $onlyconsonants[1]);

            if (empty($okok)) {
                echo "<p>No shortcode found</p>";
                return;
            }

            $okok2 = \explode('{"settingsId":"', $okok[0]);

            if (empty($okok2)) {
                echo "<p>No shortcode found</p>";
                return;
            }

            if (empty($okok2[1])) {
                echo '<p>' . $layout_shortcode_type . '</p>';
                return;
            }

            $okok3 = \explode('","lock"', $okok2[1]);

            if (empty($okok3)) {
                echo "<p>No shortcode found</p>";
                return;
            }

            $decoded_settings = base64_decode($okok3[0]);

            if (empty($decoded_settings)) {
                echo '<p>' . $layout_shortcode_type . '</p>';
                return;
            }

            $product_settings_one = json_decode($decoded_settings, true);

            $product_settings = Utils::convert_settings_shape($product_settings_one);

            if (empty($product_settings)) {
                echo '<p>' . $layout_shortcode_type . '</p>';
                return;
            }        

            $shortcode_settings_string = '';

            foreach ($product_settings as $key => $value) {

                if ($key === 'query')  {
                    continue;
                }

                if ($key === 'collection_id' || $key === 'product_id' || $key === 'limit') {
                    if (empty($value)) {
                        continue;
                    }
                }

                if ($key === 'icon' && empty($value)) {
                    continue;
                }

                if ($key === 'collection' && $layout_shortcode_type === '[wps_collections]') {
                    continue;
                }

                if ($key === 'html_template_data') {

                    $value = str_replace('<product_title />', '<ProductTitle />', $value);
                    $value = str_replace('<product_pricing />', '<ProductPricing />', $value);
                    $value = str_replace('<product_description />', '<ProductDescription />', $value);
                    $value = str_replace('<product_images />', '<ProductImages />', $value);
                    $value = str_replace('<product_buy_button />', '<ProductBuyButton />', $value);
                    $value = str_replace('<reviews />', '<Reviews />', $value);

                    $value = base64_encode($value);

                }

                if (is_array($value)) {

                    $sub_data = '';

                    foreach ($value as $v) {
                        $sub_data .= $v . ',';
                    }

                    $sub_data = rtrim($sub_data, ',');

                    $shortcode_settings_string .= ' ' . $key . '="' . $sub_data . '"';

                } else {

                    if (is_bool($value)) {
                        if ($value === false) {
                            $value = 'false';
                        } else {
                            $value = 'true';
                        }
                    }

                    $shortcode_settings_string .= ' ' . $key . '="' . $value . '"';
                }

            }
            
            $split = explode(']', $layout_shortcode_type);

            if (empty($split)) {
                $sc = '[wps_products';
            } else {
                $sc = $split[0];
            }

            $default_shortcode = $sc . $shortcode_settings_string . ']';

            if ($column_key == 'shortcode') {
                echo '<p>' . $default_shortcode . '</p>';
            }            
        }

    }

    public function check_and_update_layout_type($data, $postarr) {

        // Check if it's a shopwp_shortcodes post type
        if ($data['post_type'] === 'shopwp_shortcodes') {

            $layout_type = get_post_meta( $postarr['ID'], 'layout_type', true );

            // Don't do anything if an empty layout or a simple shortcode layout
            if ( empty($layout_type) || 'shortcode' === $layout_type ) {
                return $data;
            }

            $post_status = $data['post_status'];

            // Only perform the check for published posts and when the layout type is not "shortcode"
            if ('publish' === $post_status && 'shortcode' !== $layout_type) {
                // Get the list of published posts with the same layout type
                $posts_with_same_layout_type = get_posts(
                    [
                        'post_type'      => 'shopwp_shortcodes',
                        'meta_key'       => 'layout_type',
                        'meta_value'     => $layout_type,
                        'order'          => 'ASC',
                        'posts_per_page' => -1, // Retrieve all published posts
                        'fields'         => 'ids', // Only retrieve the post IDs
                    ]
                );

                // Check if there are any published posts with the same layout type
                if ( !empty($posts_with_same_layout_type) ) {

                    // Find the post ID of the older published post
                    $older_post_id = $posts_with_same_layout_type[0];

                    if ( ! empty($older_post_id) && $older_post_id !== $postarr['ID'] ) {
                        
                        // Set the older post as a draft
                        wp_update_post(
                            [
                                'ID'          => $older_post_id,
                                'post_status' => 'draft',
                            ]
                        );
                    }
                }
            }
        }

        return $data;
    }

    public function maybe_flush_rewrite_rules() {

      if (!apply_filters('shopwp_flush_permalinks_after_sync', true)) {
         return;
      }

      if (get_option('shopwp_should_flush_rewrite_rules')) {
         flush_rewrite_rules();
         update_option('shopwp_should_flush_rewrite_rules', 0, false);
      }

    }    

    public function init($permalinks)
    {

        $this->post_type_products($permalinks);
        $this->post_type_collections($permalinks);
        $this->register_custom_taxonomies();
        $this->post_type_layouts();
        $this->maybe_flush_rewrite_rules();
        

        add_filter('allowed_block_types_all', [$this, 'allowed_block_types_all'], 10, 2);

        add_filter('enter_title_here', [$this, 'change_default_layout_title_text']);
        add_action('admin_init', [$this, 'hide_layout_builder_title']);
        add_action('wp_insert_post_data', [$this, 'check_and_update_layout_type'], 10, 2);
        add_filter('manage_shopwp_shortcodes_posts_columns', [$this, 'add_shortcode_column_to_post_list']);
        add_action('manage_shopwp_shortcodes_posts_custom_column', [$this, 'add_shortcode_column_value_to_post_list'], 10, 2);
    }
}
