<?php

namespace ShopWP;

use ShopWP\Utils;
use ShopWP\Options;
use ShopWP\Utils\Data;
use ShopWP\Utils\Server;

if (!defined('ABSPATH')) {
    exit();
}

class Frontend
{
    public $plugin_settings;
    public $Data_Bridge;

    public function __construct($plugin_settings, $Data_Bridge)
    {
        $this->plugin_settings = $plugin_settings;
        $this->Data_Bridge = $Data_Bridge;
    }

    public function public_scripts()
    {

        global $post;

        $runtime_url                    = SHOPWP_PLUGIN_URL . 'dist/runtime.js';
        $vendors_admin_url              = SHOPWP_PLUGIN_URL . 'dist/vendors-public.js';
        $main_url                       = SHOPWP_PLUGIN_URL . 'dist/public.js';
        $user_overrides_url             = SHOPWP_PLUGIN_URL . 'public/templates/javascript/user-overrides.js';

        $runtime_timestamp              = filemtime(SHOPWP_PLUGIN_DIR_PATH . 'dist/runtime.js');
        $public_vendors_timestamp       = filemtime(SHOPWP_PLUGIN_DIR_PATH . 'dist/vendors-public.js');
        $public_timestamp               = filemtime(SHOPWP_PLUGIN_DIR_PATH . 'dist/public.js');
        $user_overrides_timestamp       = filemtime(SHOPWP_PLUGIN_DIR_PATH . 'public/templates/javascript/user-overrides.js');

        wp_register_script('shopwp-runtime', $runtime_url, ['wp-hooks', 'wp-element'], $runtime_timestamp, true);
        wp_register_script('shopwp-vendors-public', $vendors_admin_url, ['wp-hooks', 'wp-element'], $public_vendors_timestamp, true);
        wp_register_script(
            'shopwp-public', 
            $main_url, 
            [
                'wp-hooks',
                'wp-element',
                'shopwp-runtime',
                'shopwp-vendors-public'
            ],
            $public_timestamp,
            true
        );

        if (empty($this->plugin_settings['shop'])) {
            $currency = 'USD';
        } else {
            $currency = $this->plugin_settings['shop']['currency'];
        }

        // Global plugin JS settings
        $this->Data_Bridge->add_settings_script('shopwp-public', false);

        wp_enqueue_script('shopwp-runtime');
        wp_enqueue_script('shopwp-vendors-public');
        wp_enqueue_script('shopwp-public');
        
        
        $overrides_filepath = trim(SHOPWP_PLUGIN_DIR_PATH . 'public/templates/javascript/user-overrides.js');

        $overrides_size = filesize($overrides_filepath);

        if ($overrides_size !== false and $overrides_size > 0) {
            wp_register_script('shopwp-user-overrides', $user_overrides_url, ['shopwp-public'], $user_overrides_timestamp, true);
            wp_enqueue_script('shopwp-user-overrides');
        }

        // CSS
        wp_enqueue_style('shopwp-styles-general', SHOPWP_PLUGIN_URL . 'dist/general.css', [], filemtime(SHOPWP_PLUGIN_DIR_PATH . 'dist/general.css'));
        wp_enqueue_style('shopwp-styles-layout', SHOPWP_PLUGIN_URL . 'dist/layout.css', [], filemtime(SHOPWP_PLUGIN_DIR_PATH . 'dist/layout.css'));
        wp_enqueue_style('shopwp-styles-cart', SHOPWP_PLUGIN_URL . 'dist/cart.css', [], filemtime(SHOPWP_PLUGIN_DIR_PATH . 'dist/cart.css'));
        wp_enqueue_style('shopwp-styles-buy-button', SHOPWP_PLUGIN_URL . 'dist/buyButton.css', [], filemtime(SHOPWP_PLUGIN_DIR_PATH . 'dist/buyButton.css'));
        wp_enqueue_style('shopwp-styles-images', SHOPWP_PLUGIN_URL . 'dist/images.css', [], filemtime(SHOPWP_PLUGIN_DIR_PATH . 'dist/images.css'));
        wp_enqueue_style('shopwp-styles-title', SHOPWP_PLUGIN_URL . 'dist/title.css', [], filemtime(SHOPWP_PLUGIN_DIR_PATH . 'dist/title.css'));
        wp_enqueue_style('shopwp-styles-description', SHOPWP_PLUGIN_URL . 'dist/description.css', [], filemtime(SHOPWP_PLUGIN_DIR_PATH . 'dist/description.css'));
        wp_enqueue_style('shopwp-styles-pricing', SHOPWP_PLUGIN_URL . 'dist/pricing.css', [], filemtime(SHOPWP_PLUGIN_DIR_PATH . 'dist/pricing.css'));
        wp_enqueue_style('shopwp-styles-storefront', SHOPWP_PLUGIN_URL . 'dist/storefront.css', [], filemtime(SHOPWP_PLUGIN_DIR_PATH . 'dist/storefront.css'));
        
        
    }

    public function defer_parsing_of_js($tag, $handle) {
        
        if ( 'shopwp-public' !== $handle ) {
            return $tag;
        }
        
        return str_replace( ' src', ' defer="defer" src', $tag );
    }


    public function css_body_class($classes)
    {
        $classes[] = 'shopwp';

        return $classes;
    }

    /*
    
    This fixes thew infamous "Cookie check failed" error documented here: https://docs.wpshop.io/guides/common-issues#cookie-check-failed
    Added: 5/5/23
    
    */
    public function sanitize_nonce($errors) {

        // Bail if rest_route isn't defined (shouldn't happen!)
        if ( empty( $GLOBALS['wp']->query_vars['rest_route'] ) ) {
            return $errors;
        }

        $route = ltrim( $GLOBALS['wp']->query_vars['rest_route'], '/' );

        // Ensure we're dealing with our REST request only
        if ( !Utils::str_contains($route, 'shopwp/v1')) {
            return $errors;
        }

        if ( !empty( $_SERVER['HTTP_X_WP_NONCE'] ) ) {

            $nonce = $_SERVER['HTTP_X_WP_NONCE'];

            if ( !\wp_verify_nonce($nonce, 'wp_rest') ) {
                // Nonce check failed, so create a new one.
                $_SERVER['HTTP_X_WP_NONCE'] = \wp_create_nonce('wp_rest');
            }
        }

        return $errors;
    }

    public function init()
    {
        if (\is_admin()) {
            return;
        }

        add_filter('body_class', [$this, 'css_body_class']);
        add_action('wp_enqueue_scripts', [$this, 'public_scripts']);
        add_filter('script_loader_tag', [$this, 'defer_parsing_of_js'], 10, 2 );
        add_filter('rest_authentication_errors', [$this, 'sanitize_nonce']);
    }
}