<?php

namespace ShopWP\DB;

use ShopWP\Utils;

if (!defined('ABSPATH')) {
    exit();
}

class Collects extends \ShopWP\DB
{
    
    public $table_name_suffix;
    public $table_name;
    public $version;
    public $primary_key;
    public $lookup_key;
    public $cache_group;
    public $type;

    public function __construct()
    {
        $this->table_name_suffix = SHOPWP_TABLE_NAME_COLLECTS;
        $this->table_name = $this->get_table_name();
        $this->version = '1.0';
        $this->primary_key = 'id';
        $this->lookup_key = 'collectId';
        $this->cache_group = 'shopwp_db_collects';
        $this->type = 'collect';
    }

    public function get_columns()
    {
        return [
            'id' => '%d',
            'collectId' => '%s',
            'productId' => '%s',
            'collectionId' => '%s',
            'position' => '%d'
        ];
    }

    public function cols_that_should_remain_ints()
    {
        return [
            'id',
            'position'
        ];
    }

    public function modify_options(
        $shopify_item,
        $item_lookup_key = SHOPWP_PRODUCTS_LOOKUP_KEY
    ) {
        return [
            'item' => $shopify_item,
            'item_lookup_key' => $item_lookup_key,
            'item_lookup_value' => $shopify_item->id,
            'prop_to_access' => 'collects',
            'change_type' => 'collect'
        ];
    }

    public function mod_before_change($collect)
    {
        $collect_copy = $this->copy($collect);
        $collect_copy = $this->maybe_rename_to_lookup_key($collect_copy);

        return $collect_copy;
    }

    public function insert_collect($collect)
    {
        return $this->insert($collect);
    }

    public function update_collect($collect)
    {
        return $this->update(
            $this->lookup_key,
            $this->get_lookup_value($collect),
            $collect
        );
    }

    public function delete_collect($collect)
    {
        return $this->delete_rows(
            $this->lookup_key,
            $this->get_lookup_value($collect)
        );
    }

    public function delete_collects_from_collection_id($collection_id)
    {
        return $this->delete_rows(
            SHOPWP_COLLECTIONS_LOOKUP_KEY,
            $collection_id
        );
    }

    public function get_collects_from_collection_id($collection_id)
    {
        return $this->get_rows(
            SHOPWP_COLLECTIONS_LOOKUP_KEY,
            $collection_id
        );
    }

    public function get_collects_from_product_id($product_id) {
		return $this->get_rows(SHOPWP_PRODUCTS_LOOKUP_KEY, $product_id);
	}

    public function get_published_collects($collects, $published_product_ids)
    {
        return array_filter($collects, function ($collect) use (
            $published_product_ids
        ) {
            return $this->is_collect_published(
                $collect,
                $published_product_ids
            );
        });
    }

    public function is_collect_published($collect, $published_product_ids)
    {
        if (!isset($collect->product_id)) {
            return false;
        }

        if (in_array($collect->product_id, $published_product_ids)) {
            return true;
        }

        return false;
    }

    // Need to do a comparison as some collects could have been found within
    // the batch of 250. So we just need to find the difference and then add that number.
    public function find_published_difference_to_add($items, $published_items)
    {
        $original_total = count($items);
        $after_filter_total = count($published_items);

        return $original_total - $after_filter_total;
    }

    public function create_table_query($table_name = false)
    {
        if (!$table_name) {
            $table_name = $this->table_name;
        }

        $collate = $this->collate();

        return "CREATE TABLE $table_name (
			id bigint(100) unsigned NOT NULL AUTO_INCREMENT,
			collectId varchar(255) DEFAULT '',
			productId varchar(255) DEFAULT '',
			collectionId varchar(255) DEFAULT '',
			position int(20) DEFAULT 0,
			PRIMARY KEY  (id)
		) ENGINE=InnoDB $collate";
    }
}
