<?php

namespace ShopWP\DB;

use ShopWP\CPT;
use ShopWP\Utils;

if (!defined('ABSPATH')) {
    exit();
}

class Metafields extends \ShopWP\DB
{
    public $table_name_suffix;
    public $table_name;
    public $version;
    public $primary_key;
    public $lookup_key;
    public $cache_group;
    public $type;

    public function __construct()
    {
        $this->table_name_suffix = SHOPWP_TABLE_NAME_METAFIELDS;
        $this->table_name = $this->get_table_name();
        $this->version = '1.0';
        $this->primary_key = 'id';
        $this->lookup_key = 'metafieldId';
        $this->cache_group = 'shopwp_db_metafields';
        $this->type = 'metafields';
    }

    public function get_columns()
    {        
        return [
            'id' => '%d',
            'metafieldId' => '%s',
            'productId' => '%s',
            'createdAt' => '%s',
            'definition' => '%s',
            'description' => '%s',
            'metaKey' => '%s',
            'legacyResourceId' => '%s',
            'namespace' => '%s',
            'owner' => '%s',
            'ownerType' => '%s',
            'reference' => '%s',
            'type' => '%s',
            'updatedAt' => '%s',
            'value' => '%s'
        ];
    }

    public function cols_that_should_remain_ints()
    {
        return [
            'id'
        ];

    }

    public function get_metafields()
    {
        return $this->get_all_rows();
    }

    public function delete_metafields_from_product_id($product_id)
    {
        return $this->delete_rows(SHOPWP_PRODUCTS_LOOKUP_KEY, $product_id);
    }

    public function mod_before_change($metafield)
    {

        $metafield_copy = $this->copy($metafield);

        if ($metafield_copy->productId) {
            $metafield_copy->productId = \str_replace("gid://shopify/Product/", "", $metafield_copy->productId);
        }

        if ($metafield_copy->metafieldId) {
            $metafield_copy->metafieldId = \str_replace("gid://shopify/Metafield/", "", $metafield_copy->metafieldId);
        }

        unset($metafield_copy->id);
        unset($metafield_copy->__parentId);

        return $metafield_copy;

    }

    public function create_table_query($table_name = false)
    {
        if (!$table_name) {
            $table_name = $this->table_name;
        }

        $collate = $this->collate();

        return "CREATE TABLE $table_name (
            id bigint(100) unsigned NOT NULL AUTO_INCREMENT,
            metafieldId varchar(255) DEFAULT '', 
            productId varchar(255) DEFAULT '', 
            createdAt datetime DEFAULT '0000-00-00 00:00:00',
            definition mediumtext DEFAULT '', 
            description mediumtext DEFAULT '', 
            metaKey varchar(255) DEFAULT '', 
            legacyResourceId varchar(255) DEFAULT '',
            namespace varchar(255) DEFAULT '',
            owner varchar(255) DEFAULT '', 
            ownerType varchar(255) DEFAULT '', 
            reference varchar(255) DEFAULT '', 
            type varchar(255) DEFAULT '', 
            updatedAt datetime DEFAULT '0000-00-00 00:00:00',
            value longtext DEFAULT '',
			PRIMARY KEY  (id)
		) ENGINE=InnoDB $collate";
    }
}
