<?php

namespace ShopWP\DB;

use ShopWP\Utils;

if (!defined('ABSPATH')) {
    exit();
}

class Options extends \ShopWP\DB
{

    public $table_name_suffix;
    public $table_name;
    public $version;
    public $primary_key;
    public $lookup_key;
    public $cache_group;
    public $type;

    public function __construct()
    {
        $this->table_name_suffix = SHOPWP_TABLE_NAME_OPTIONS;
        $this->table_name = $this->get_table_name();
        $this->version = '1.0';
        $this->primary_key = 'id';
        $this->lookup_key = 'optionId';
        $this->cache_group = 'shopwp_db_options';
        $this->type = 'option';
    }

    public function get_columns()
    {
        return [
            'id' => '%d',
            'optionId' => '%s',
            'productId' => '%s',
            'name' => '%s',
            'optionPosition' => '%d',
            'optionValues' => '%s'
        ];
    }

    public function cols_that_should_remain_ints()
    {
        return ['id', 'productId', 'optionPosition'];
    }

    public function modify_options(
        $shopify_item,
        $item_lookup_key = SHOPWP_PRODUCTS_LOOKUP_KEY
    ) {
        return [
            'item' => $shopify_item,
            'item_lookup_key' => $item_lookup_key,
            'item_lookup_value' => $shopify_item->id,
            'prop_to_access' => 'options',
            'change_type' => 'option'
        ];
    }

    public function mod_before_change($option)
    {

        $option_copy = $this->copy($option);
        $option_copy = $this->maybe_rename_to_lookup_key($option_copy);

        $option_copy->optionPosition = $option_copy->position;
        $option_copy->optionValues = $option_copy->values;

        if (isset($option_copy->productId)) {
            $option_copy->productId = \str_replace("gid://shopify/Product/", "", $option_copy->productId);
        }

        if (isset($option_copy->optionId)) {
            $option_copy->optionId = \str_replace("gid://shopify/ProductOption/", "", $option_copy->optionId);
        }

        unset($option_copy->position);
        unset($option_copy->values);
        unset($option_copy->__parentId);

        return $option_copy;

    }

    public function insert_option($option)
    {
        return $this->insert($option);
    }

    public function update_option($option)
    {
        return $this->update(
            $this->lookup_key,
            $this->get_lookup_value($option),
            $option
        );
    }

    public function delete_option($option)
    {
        return $this->delete_rows(
            $this->lookup_key,
            $this->get_lookup_value($option)
        );
    }

    public function delete_options_from_product_id($product_id)
    {
        return $this->delete_rows(SHOPWP_PRODUCTS_LOOKUP_KEY, $product_id);
    }

    public function get_options_from_product_id($product_id)
    {
        return $this->get_rows(SHOPWP_PRODUCTS_LOOKUP_KEY, $product_id, ['name', 'optionPosition', 'optionValues']);
    }

    public function unserialize_options($options) {
        return array_map(function($option) {

            $option->optionValues = maybe_unserialize($option->optionValues);

            return $option;

        }, $options);
    }

    public function create_table_query($table_name = false)
    {
        if (!$table_name) {
            $table_name = $this->table_name;
        }

        $collate = $this->collate();

        return "CREATE TABLE $table_name (
			id bigint(100) unsigned NOT NULL AUTO_INCREMENT,
			optionId longtext NOT NULL DEFAULT '',
			productId longtext DEFAULT '',
			name MEDIUMTEXT DEFAULT '',
			optionPosition int(20) DEFAULT 0,
			optionValues longtext DEFAULT '',
			PRIMARY KEY  (id)
		) ENGINE=InnoDB $collate";
    }
}
