<?php

namespace ShopWP\DB;

use ShopWP\Utils;
use ShopWP\Transients;
use ShopWP\CPT;

if (!defined('ABSPATH')) {
    exit();
}

class Orders extends \ShopWP\DB
{

    public $table_name_suffix;
    public $table_name;
    public $version;
    public $primary_key;
    public $lookup_key;
    public $cache_group;
    public $type;

    public function __construct()
    {
        // Table info
        $this->table_name_suffix = SHOPWP_TABLE_NAME_ORDERS;
        $this->table_name = $this->get_table_name();
        $this->version = '1.0';
        $this->primary_key = 'id';
        $this->lookup_key = 'order_id';
        $this->cache_group = 'shopwp_db_orders';

        // Used for hook identifiers within low level db methods like insert, update, etc
        $this->type = 'order';
    }

    public function get_columns()
    {
        return [
            'id'                                    => '%d',
            'orderId'                               => '%s',
            'postId'                                => '%d',
            'customerId'                            => '%s',
            'app'                                   => '%s',
            'billingAddress'                        => '%s',
            'billingAddressMatchesShippingAddress'  => '%d',
            'canMarkAsPaid'                         => '%d',
            'canNotifyCustomer'                     => '%d',
            'cancelReason'                          => '%s',
            'cancelledAt'                           => '%s',
            'capturable'                            => '%d',
            'cartDiscountAmountSet'                 => '%s',
            'channelInformation'                    => '%s',
            'clientIp'                              => '%s',
            'closed'                                => '%d',
            'closedAt'                              => '%s',
            'confirmed'                             => '%d',
            'createdAt'                             => '%s',
            'currencyCode'                          => '%s',
            'currentCartDiscountAmountSet'          => '%s',
            'currentSubtotalLineItemsQuantity'      => '%d',
            'currentSubtotalPriceSet'               => '%s',
            'currentTaxLines'                       => '%s',
            'currentTotalDiscountsSet'              => '%s',
            'currentTotalDutiesSet'                 => '%s',
            'currentTotalPriceSet'                  => '%s',
            'currentTotalTaxSet'                    => '%s',
            'currentTotalWeight'                    => '%s',
            'customAttributes'                      => '%s',
            'customerAcceptsMarketing'              => '%d',
            'customerJourneySummary'                => '%s',
            'customerLocale'                        => '%s',
            'discountCode'                          => '%s',
            'discountCodes'                         => '%s',
            'displayAddress'                        => '%s',
            'displayFinancialStatus'                => '%s',
            'displayFulfillmentStatus'              => '%s',
            'disputes'                              => '%s',
            'edited'                                => '%d',
            'email'                                 => '%s',
            'estimatedTaxes'                        => '%d',
            'fulfillable'                           => '%d',
            'fulfillments'                          => '%s',
            'fullyPaid'                             => '%d',
            'hasTimelineComment'                    => '%d',
            'legacyResourceId'                      => '%s',
            'merchantEditable'                      => '%d',
            'merchantEditableErrors'                => '%s',
            'metafield'                             => '%s',
            'name'                                  => '%s',
            'netPaymentSet'                         => '%s',
            'note'                                  => '%s',
            'originalTotalDutiesSet'                => '%s',
            'originalTotalPriceSet'                 => '%s',
            'paymentCollectionDetails'              => '%s',
            'paymentGatewayNames'                   => '%s',
            'paymentTerms'                          => '%s',
            'phone'                                 => '%s',
            'physicalLocation'                      => '%s',
            'presentmentCurrencyCode'               => '%s',
            'privateMetafield'                      => '%s',
            'processedAt'                           => '%s',
            'publication'                           => '%s',
            'refundDiscrepancySet'                  => '%s',
            'refundable'                            => '%d',
            'refunds'                               => '%s',
            'registeredSourceUrl'                   => '%s',
            'requiresShipping'                      => '%d',
            'restockable'                           => '%d',
            'riskLevel'                             => '%s',
            'risks'                                 => '%s',
            'shippingAddress'                       => '%s',
            'shippingLine'                          => '%s',
            'sourceIdentifier'                      => '%s',
            'subtotalLineItemsQuantity'             => '%d',
            'subtotalPriceSet'                      => '%s',
            'suggestedRefund'                       => '%s',
            'tags'                                  => '%s',
            'taxLines'                              => '%s',
            'taxesIncluded'                         => '%d',
            'test'                                  => '%d',
            'totalCapturableSet'                    => '%s',
            'totalDiscountsSet'                     => '%s',
            'totalOutstandingSet'                   => '%s',
            'totalPriceSet'                         => '%s',
            'totalReceivedSet'                      => '%s',
            'totalRefundedSet'                      => '%s',
            'totalRefundedShippingSet'              => '%s',
            'totalShippingPriceSet'                 => '%s',
            'totalTaxSet'                           => '%s',
            'totalTipReceivedSet'                   => '%s',
            'totalWeight'                           => '%s',
            'transactions'                          => '%s',
            'unpaid'                                => '%d',
            'updatedAt'                             => '%s',
        ];
    }

    public function cols_that_should_remain_ints()
    {
        return [
            'id',
            'postId',
            'currentSubtotalLineItemsQuantity',
            'estimatedTaxes',
            'subtotalLineItemsQuantity'
        ];
    }

    public function modify_options(
        $shopify_item,
        $item_lookup_key = SHOPWP_PRODUCTS_LOOKUP_KEY
    ) {
        return [
            'item' => $shopify_item,
            'item_lookup_key' => $item_lookup_key,
            'item_lookup_value' => $shopify_item->id,
            'prop_to_access' => 'orders',
            'change_type' => 'order',
        ];
    }

    public function mod_before_change($order)
    {

        $order_copy = $this->copy($order);
        $order_copy->orderId = $order_copy->id;
        
        $order_copy = $this->maybe_rename_to_lookup_key($order_copy);
        $order_copy = $this->add_customer_id_to_order($order_copy);

        if (empty($order_copy->cancelledAt)) {
            $order_copy->cancelledAt = '0000-00-00 00:00:00';
        }

        if (empty($order_copy->closedAt)) {
            $order_copy->closedAt = '0000-00-00 00:00:00';
        }

        if (empty($order_copy->createdAt)) {
            $order_copy->createdAt = '0000-00-00 00:00:00';
        }

        if (empty($order_copy->processedAt)) {
            $order_copy->processedAt = '0000-00-00 00:00:00';
        }

        if (empty($order_copy->updatedAt)) {
            $order_copy->updatedAt = '0000-00-00 00:00:00';
        }

        if (isset($order_copy->orderId)) {
            $order_copy->orderId = \str_replace("gid://shopify/Order/", "", $order_copy->orderId);
        }

        if (isset($order_copy->customerId)) {
            $order_copy->customerId = \str_replace("gid://shopify/Customer/", "", $order_copy->customerId);

        } else if (isset($order_copy->customer_id)) {
            $order_copy->customerId = \str_replace("gid://shopify/Customer/", "", $order_copy->customer_id);
        }
        
        unset($order_copy->order_id);
        unset($order_copy->customer_id);

        return $order_copy;
        
    }

    public function insert_order($order)
    {
        return $this->insert($order);
    }

   
    public function update_order($order)
    {
        return $this->update(
            $this->lookup_key,
            $this->get_lookup_value($order),
            $order
        );
    }

    public function delete_order($order)
    {
        return $this->delete_rows(
            $this->lookup_key,
            $this->get_lookup_value($order)
        );
    }

    public function get_orders()
    {
        return $this->get_all_rows();
    }

    public function delete_orders_from_product_id($product_id)
    {
        return $this->delete_rows('product_id', $product_id);
    }

    public function get_orders_from_product_id($product_id)
    {
        return $this->get_rows('product_id', $product_id);
    }

    public function add_customer_id_to_order($order)
    {
        $order = Utils::convert_array_to_object($order);

        if (
            Utils::has($order, 'customer') &&
            Utils::has($order->customer, 'id')
        ) {
            $order->customer_id = $order->customer->id;
        }

        return $order;
    }

    public function get_order($orderID = null)
    {
        global $wpdb;

        if ($orderID === null) {
            $orderID = get_the_ID();
        }

        $query = "SELECT orders.* FROM $this->table_name as orders WHERE orders.post_id = %d";

        return $wpdb->get_row($wpdb->prepare($query, $orderID));
    }

    /*

	Creates a table query string

	*/
    public function create_table_query($table_name = false)
    {
        if (!$table_name) {
            $table_name = $this->table_name;
        }

        $collate = $this->collate();

        return "CREATE TABLE $table_name (
            id bigint(100) unsigned NOT NULL AUTO_INCREMENT,
            orderId varchar(255) DEFAULT '',
            postId bigint(100) DEFAULT 0, 
            customerId varchar(255) DEFAULT '',
            app varchar(255) DEFAULT '',
            billingAddress longtext DEFAULT '',
            billingAddressMatchesShippingAddress tinyint(1) DEFAULT 0,
            canMarkAsPaid tinyint(1) DEFAULT 0,
            canNotifyCustomer tinyint(1) DEFAULT 0,
            cancelReason text,
            cancelledAt datetime DEFAULT '0000-00-00 00:00:00',
            capturable tinyint(1) DEFAULT 0,
            cartDiscountAmountSet varchar(255) DEFAULT '',
            channelInformation MEDIUMTEXT DEFAULT '',
            clientIp varchar(255) DEFAULT '',
            closed tinyint(1) DEFAULT 0,
            closedAt datetime DEFAULT '0000-00-00 00:00:00',
            confirmed tinyint(1) DEFAULT 0,
            createdAt datetime DEFAULT '0000-00-00 00:00:00',
            currencyCode varchar(255) DEFAULT '',
            currentCartDiscountAmountSet varchar(255) DEFAULT '',
            currentSubtotalLineItemsQuantity bigint(100) DEFAULT 0,
            currentSubtotalPriceSet varchar(255) DEFAULT '',
            currentTaxLines MEDIUMTEXT DEFAULT '',
            currentTotalDiscountsSet varchar(255) DEFAULT '',
            currentTotalDutiesSet varchar(255) DEFAULT '',
            currentTotalPriceSet varchar(255) DEFAULT '',
            currentTotalTaxSet varchar(255) DEFAULT '',
            currentTotalWeight varchar(255) DEFAULT '',
            customAttributes mediumtext DEFAULT '',
            customerAcceptsMarketing tinyint(1) DEFAULT 0,
            customerJourneySummary MEDIUMTEXT DEFAULT '',
            customerLocale varchar(255) DEFAULT '',
            discountCode varchar(255) DEFAULT '',
            discountCodes MEDIUMTEXT DEFAULT '',
            displayAddress longtext DEFAULT '',
            displayFinancialStatus varchar(255) DEFAULT '',
            displayFulfillmentStatus varchar(255) DEFAULT '',
            disputes MEDIUMTEXT DEFAULT '',
            edited tinyint(1) DEFAULT 0,
            email varchar(255) DEFAULT '',
            estimatedTaxes tinyint(1) DEFAULT 0,
            fulfillable tinyint(1) DEFAULT 0,
            fulfillments MEDIUMTEXT DEFAULT '',
            fullyPaid tinyint(1) DEFAULT 0,
            hasTimelineComment tinyint(1) DEFAULT 0,
            legacyResourceId varchar(255) DEFAULT '',
            merchantEditable tinyint(1) DEFAULT 0,
            merchantEditableErrors text,
            metafield MEDIUMTEXT DEFAULT '',
            name mediumtext DEFAULT '',
            netPaymentSet varchar(255) DEFAULT '',
            note longtext DEFAULT '',
            originalTotalDutiesSet varchar(255) DEFAULT '',
            originalTotalPriceSet varchar(255) DEFAULT '',
            paymentCollectionDetails mediumtext DEFAULT '',
            paymentGatewayNames mediumtext DEFAULT '',
            paymentTerms MEDIUMTEXT DEFAULT '',
            phone varchar(255) DEFAULT '',
            physicalLocation varchar(255) DEFAULT '',
            presentmentCurrencyCode varchar(255) DEFAULT '',
            privateMetafield text,
            processedAt datetime DEFAULT '0000-00-00 00:00:00',
            publication varchar(255) DEFAULT '',
            refundDiscrepancySet varchar(255) DEFAULT '',
            refundable tinyint(1) DEFAULT 0,
            refunds MEDIUMTEXT DEFAULT '',
            registeredSourceUrl longtext DEFAULT '',
            requiresShipping tinyint(1) DEFAULT 0,
            restockable tinyint(1) DEFAULT 0,
            riskLevel varchar(255) DEFAULT '',
            risks MEDIUMTEXT DEFAULT '',
            shippingAddress longtext DEFAULT '',
            shippingLine text,
            sourceIdentifier text,
            subtotalLineItemsQuantity bigint(100) DEFAULT 0,
            subtotalPriceSet varchar(255) DEFAULT '',
            suggestedRefund varchar(255) DEFAULT '',
            tags MEDIUMTEXT DEFAULT '',
            taxLines MEDIUMTEXT DEFAULT '',
            taxesIncluded tinyint(1) DEFAULT 0,
            test tinyint(1) DEFAULT 0,
            totalCapturableSet varchar(255) DEFAULT '',
            totalDiscountsSet varchar(255) DEFAULT '',
            totalOutstandingSet varchar(255) DEFAULT '',
            totalPriceSet varchar(255) DEFAULT '',
            totalReceivedSet varchar(255) DEFAULT '',
            totalRefundedSet varchar(255) DEFAULT '',
            totalRefundedShippingSet varchar(255) DEFAULT '',
            totalShippingPriceSet varchar(255) DEFAULT '',
            totalTaxSet varchar(255) DEFAULT '',
            totalTipReceivedSet varchar(255) DEFAULT '',
            totalWeight varchar(255) DEFAULT '',
            transactions MEDIUMTEXT DEFAULT '',
            unpaid tinyint(1) DEFAULT 0,
            updatedAt datetime DEFAULT '0000-00-00 00:00:00',
			PRIMARY KEY  (id)
		) ENGINE=InnoDB $collate";
    }
}
