<?php

namespace ShopWP\DB;

use ShopWP\Utils;
use ShopWP\Utils\Server;
use ShopWP\Utils\Data as Utils_Data;
use ShopWP\Transients;
use ShopWP\Options as Plugin_Options;

if (!defined('ABSPATH')) {
    exit();
}

class Settings_Syncing extends \ShopWP\DB
{
    public $table_name_suffix;
    public $table_name;
    public $version;
    public $primary_key;
    public $lookup_key;
    public $cache_group;
    public $type;
    public $default_primary_key_value;
    public $default_is_syncing;
    public $default_syncing_step_total;
    public $default_syncing_step_current;
    public $default_syncing_totals_shop;
    public $default_syncing_totals_products;
    public $default_syncing_totals_collections;
    public $default_syncing_totals_collects;
    public $default_syncing_totals_media;
    public $default_syncing_totals_customers;
    public $default_syncing_totals_orders;
    public $default_syncing_current_amounts_shop;
    public $default_syncing_current_amounts_products;
    public $default_syncing_current_amounts_collections;
    public $default_syncing_current_amounts_collects;
    public $default_syncing_current_amounts_media;
    public $default_syncing_current_amounts_customers;
    public $default_syncing_current_amounts_orders;
    public $default_syncing_start_time;
    public $default_syncing_end_time;
    public $default_syncing_errors;
    public $default_syncing_warnings;
    public $default_finished_media;
    public $default_published_product_ids;
    public $default_recently_syncd_media_ref;
    public $default_current_syncing_step_text;
    public $default_percent_completed_removal;
    public $expire_sync_from_server;
    public $default_syncing_totals_removal;
    public $default_expire_sync_from_server;
    public $default_currently_active_tool;
    
    public function __construct()
    {
        global $wpdb;

        $this->table_name_suffix = SHOPWP_TABLE_NAME_SETTINGS_SYNCING;
        $this->table_name = $this->get_table_name();
        $this->version = '1.0';
        $this->primary_key = 'id';
        $this->lookup_key = 'id';
        $this->cache_group = 'wps_db_syncing';
        $this->type = 'settings_syncing';

        $this->default_is_syncing                           = 0;
        $this->default_syncing_step_total                   = 0;
        $this->default_syncing_step_current                 = 0;

        $this->default_syncing_totals_shop                  = 0;
        $this->default_syncing_totals_products              = 0;
        $this->default_syncing_totals_collections           = 0;
        $this->default_syncing_totals_collects              = 0;
        $this->default_syncing_totals_media                 = 0;
        $this->default_syncing_totals_customers             = 0;
        $this->default_syncing_totals_orders                = 0;

        $this->default_syncing_current_amounts_shop         = 0;
        $this->default_syncing_current_amounts_collections  = 0;
        $this->default_syncing_current_amounts_products     = 0;
        $this->default_syncing_current_amounts_collects     = 0;
        $this->default_syncing_current_amounts_media        = 0;
        $this->default_syncing_current_amounts_customers    = 0;
        $this->default_syncing_current_amounts_orders       = 0;

        $this->default_syncing_start_time                   = 0;
        $this->default_syncing_end_time                     = 0;
        $this->default_syncing_errors                       = null;
        $this->default_syncing_warnings                     = null;
        $this->default_finished_media                       = 0;
        $this->default_published_product_ids                = '';
        $this->default_recently_syncd_media_ref             = '';
        $this->default_current_syncing_step_text            = __('Loading process ...', 'shopwp');
        $this->default_percent_completed_removal            = 0;
        $this->default_syncing_totals_removal               = 0;
        $this->default_expire_sync_from_server              = 0;
        $this->default_currently_active_tool                = '';

    }

    public function get_columns()
    {
        return [
            'id'                                    => '%d',
            'is_syncing'                            => '%d',
            'syncing_step_total'                    => '%d',
            'syncing_step_current'                  => '%d',
            'syncing_totals_shop'                   => '%d',
            'syncing_totals_products'               => '%d',
            'syncing_totals_collections'            => '%d',
            'syncing_totals_collects'               => '%d',
            'syncing_totals_media'                  => '%d',
            'syncing_totals_customers'              => '%d',
            'syncing_totals_orders'                 => '%d',
            'syncing_current_amounts_shop'          => '%d',
            'syncing_current_amounts_products'      => '%d',            
            'syncing_current_amounts_collections'   => '%d',
            'syncing_current_amounts_collects'      => '%d',
            'syncing_current_amounts_media'         => '%d',
            'syncing_current_amounts_customers'     => '%d',
            'syncing_current_amounts_orders'        => '%d',
            'syncing_start_time'                    => '%d',
            'syncing_end_time'                      => '%d',
            'syncing_errors'                        => '%s',
            'syncing_warnings'                      => '%s',
            'finished_media'                        => '%d',
            'published_product_ids'                 => '%s',
            'recently_syncd_media_ref'              => '%s',
            'current_syncing_step_text'             => '%s',
            'percent_completed_removal'             => '%d',
            'syncing_totals_removal'                => '%d',
            'bulk_op_id'                            => '%s',
            'bulk_op_webhook_subscription_id'       => '%s',
            'expire_sync_from_server'               => '%d',
            'currently_active_tool'                 => '%s'
        ];
    }

    public function cols_that_should_remain_ints()
    {
        return [
            'id',
            'syncing_step_total',
            'syncing_step_current',
            'syncing_totals_shop',
            'syncing_totals_products',
            'syncing_totals_collections',
            'syncing_totals_collects',
            'syncing_totals_media',
            'syncing_totals_customers',
            'syncing_totals_orders',
            'syncing_current_amounts_shop',
            'syncing_current_amounts_products',
            'syncing_current_amounts_collections',
            'syncing_current_amounts_collects',
            'syncing_current_amounts_media',
            'syncing_current_amounts_customers',
            'syncing_current_amounts_orders',
            'syncing_start_time',
            'syncing_end_time',
            'percent_completed_removal',
            'syncing_totals_removal'
        ];
    }

    public function get_column_defaults()
    {
        return [
            'is_syncing' => $this->default_is_syncing,

            'syncing_step_total' => $this->default_syncing_step_total,
            'syncing_step_current' => $this->default_syncing_step_current,

            'syncing_totals_shop' => $this->default_syncing_totals_shop,
            'syncing_totals_products' => $this->default_syncing_totals_products,
            'syncing_totals_collections' => $this->default_syncing_totals_collections,
            'syncing_totals_collects' => $this->default_syncing_totals_collects,
            'syncing_totals_media' => $this->default_syncing_totals_media,
            'syncing_totals_customers' => $this->default_syncing_totals_customers,
            'syncing_totals_orders' => $this->default_syncing_totals_orders,

            'syncing_current_amounts_shop' =>
                $this->default_syncing_current_amounts_shop,
            'syncing_current_amounts_products' =>
                $this->default_syncing_current_amounts_products,
            'syncing_current_amounts_collections' =>
                $this->default_syncing_current_amounts_collections,
            'syncing_current_amounts_collects' =>
                $this->default_syncing_current_amounts_collects,
            'syncing_current_amounts_media' =>
                $this->default_syncing_current_amounts_media,
            'syncing_current_amounts_customers' =>
                $this->default_syncing_current_amounts_customers,
            'syncing_current_amounts_orders' =>
                $this->default_syncing_current_amounts_orders,
            'syncing_start_time' => $this->default_syncing_start_time,
            'syncing_end_time' => $this->default_syncing_end_time,
            'syncing_errors' => $this->default_syncing_errors,
            'syncing_warnings' => $this->default_syncing_warnings,
            'finished_media' => $this->default_finished_media,
            'published_product_ids' => $this->default_published_product_ids,
            'recently_syncd_media_ref' =>
                $this->default_recently_syncd_media_ref,
            'current_syncing_step_text' =>
                $this->default_current_syncing_step_text,
            'percent_completed_removal' =>
                $this->default_percent_completed_removal,
            'syncing_totals_removal' =>
                $this->default_syncing_totals_removal,
            'bulk_op_id' => '',
            'bulk_op_webhook_subscription_id'               => '',
            'expire_sync_from_server'                       => $this->default_expire_sync_from_server,
            'currently_active_tool'                         => $this->default_currently_active_tool
        ];
    }

    public function syncing_totals($table_data)
    {
        return [
            'shop'              => $table_data->syncing_totals_shop,
            'collections'       => $table_data->syncing_totals_collections,
            'products'          => $table_data->syncing_totals_products,
            'collects'          => $table_data->syncing_totals_collects,
            'media'             => $table_data->syncing_totals_media,
            'customers'         => $table_data->syncing_totals_customers,
            'orders'            => $table_data->syncing_totals_orders,
        ];
    }

    public function syncing_current_amounts($table_data)
    {
        return [
            'shop'          => $table_data->syncing_current_amounts_shop,
            'products'      => $table_data->syncing_current_amounts_products,
            'collections'   => $table_data->syncing_current_amounts_collections,
            'collects'      => $table_data->syncing_current_amounts_collects,
            'media'         => $table_data->syncing_current_amounts_media,
            'customers'     => $table_data->syncing_current_amounts_customers,
            'orders'        => $table_data->syncing_current_amounts_orders,
        ];
    }

    public function get_total_if_set($type) {
        return isset($counts[$type]) ? $counts[$type] : 0;
    }

    public function update_total($type, $total) {
        return $this->update_col('syncing_totals_' . $type, $total);
    }

    public function set_syncing_totals($counts, $exclusions = [])
    {

        $shop           = $this->get_total_if_set('shop', $counts);
        $products       = $this->get_total_if_set('products', $counts);
        $collections    = $this->get_total_if_set('collections', $counts);
        $collects       = $this->get_total_if_set('collects', $counts);
        $webhooks       = $this->get_total_if_set('webhooks', $counts);
        $media          = $this->get_total_if_set('media', $counts);
        $customers      = $this->get_total_if_set('customers', $counts);
        $orders         = $this->get_total_if_set('orders', $counts);

        $this->log(__('Setting syncing totals', 'shopwp'));

        $operations = [
            'shop'           => $this->update_total('shop', $shop),
            'products'       => $this->update_total('products', $products),
            'collections'    => $this->update_total('collections', $collections),
            'collects'       => $this->update_total('collects', $collects),
            'webhooks'       => $this->update_total('webhooks', $webhooks),
            'media'          => $this->update_total('media', $media),
            'customers'      => $this->update_total('customers', $customers),
            'orders'         => $this->update_total('orders', $orders)
        ];

        if (is_wp_error($operations['shop'])) {
            return $operations['shop'];
        }

        if (is_wp_error($operations['products'])) {
            return $operations['products'];
        }

        if (is_wp_error($operations['collections'])) {
            return $operations['collections'];
        }

        if (is_wp_error($operations['collects'])) {
            return $operations['collects'];
        }

        if (is_wp_error($operations['webhooks'])) {
            return $operations['webhooks'];
        }

        if (is_wp_error($operations['media'])) {
            return $operations['media'];
        }

        if (is_wp_error($operations['customers'])) {
            return $operations['customers'];
        }

        if (is_wp_error($operations['orders'])) {
            return $operations['orders'];
        }

        return $operations;
    }

    public function reset_syncing_totals()
    {
        return [
            'shop'                          => $this->update_col('syncing_totals_shop', 0),
            'products'                      => $this->update_col('syncing_totals_products', 0),
            'collections'                   => $this->update_col('syncing_totals_collections', 0),
            'collects'                      => $this->update_col('syncing_totals_collects', 0),
            'media'                         => $this->update_col('syncing_totals_media', 0),
            'customers'                     => $this->update_col('syncing_totals_customers', 0),
            'orders'                        => $this->update_col('syncing_totals_orders', 0),
            'percent_completed_removal'     => $this->update_col('percent_completed_removal', 0),
            'syncing_totals_removal'        => $this->update_col('syncing_totals_removal', 0),
        ];
    }

    public function reset_syncing_timing()
    {
        return [
            'syncing_start_time' => $this->update_col('syncing_start_time', 0),
            'syncing_end_time' => $this->update_col('syncing_end_time', 0)
        ];
    }

    public function reset_syncing_published_product_ids()
    {
        return $this->update_col('published_product_ids', '');
    }

    public function reset_syncing_notices()
    {
        return [
            'syncing_errors' => $this->update_col('syncing_errors', false),
            'syncing_warnings' => $this->update_col('syncing_warnings', false),
        ];
    }

    public function get_syncing_errors()
    {
        $syncing_errors = $this->get_column_single('syncing_errors');

        if (
            Utils::array_not_empty($syncing_errors) &&
            isset($syncing_errors[0]->syncing_errors)
        ) {
            return $syncing_errors[0]->syncing_errors;
        } else {
            return false;
        }
    }

    public function get_syncing_warnings()
    {
        $syncing_warnings = $this->get_column_single('syncing_warnings');

        if (
            Utils::array_not_empty($syncing_warnings) &&
            isset($syncing_warnings[0]->syncing_warnings)
        ) {
            return $syncing_warnings[0]->syncing_warnings;
        } else {
            return false;
        }
    }

    public function syncing_notices()
    {
        return [
            'syncing_errors' => maybe_unserialize($this->get_syncing_errors()),
            'syncing_warnings' => maybe_unserialize(
                $this->get_syncing_warnings()
            ),
        ];
    }

    public function is_syncing_products()
    {
        $total_products = $this->col('syncing_totals_products', 'int');

        if ($total_products > 0) {
            return true;
        } else {
            return false;
        }
    }

    public function is_syncing_collections()
    {
        $collections = $this->col('syncing_totals_collections', 'int');

        if ($collections > 0) {
            return true;
        } else {
            return false;
        }
    }

    public function reset_syncing_current_amounts()
    {
        return [
            'shop'          => $this->update_col('syncing_current_amounts_shop', 0),
            'collections'   => $this->update_col('syncing_current_amounts_collections', 0),
            'products'      => $this->update_col('syncing_current_amounts_products', 0),
            'collects'      => $this->update_col('syncing_current_amounts_collects', 0),
            'media'         => $this->update_col('syncing_current_amounts_media', 0),
            'customers'     => $this->update_col('syncing_current_amounts_customers', 0),
            'orders'        => $this->update_col('syncing_current_amounts_orders', 0)
        ];
    }

    public function is_syncing()
    {
        $syncing_row = Utils::convert_array_to_object($this->get());

        if (!Utils::has($syncing_row, 'is_syncing')) {
            return false;
        }

        if ($syncing_row->is_syncing == 0 || $syncing_row->is_syncing == '0') {
            return false;
        } else {
            return true;
        }
    }

    public function toggle_syncing($syncing_status)
    {

        $result = $this->update_col('is_syncing', $syncing_status);

        return $result;
    }

    public function get_published_product_ids()
    {
        $published_product_ids = $this->get_column_single(
            'published_product_ids'
        );

        if (
            Utils::array_not_empty($published_product_ids) &&
            isset($published_product_ids[0]->published_product_ids)
        ) {
            $pub_ids = $published_product_ids[0]->published_product_ids;

            return maybe_unserialize($pub_ids);
        } else {
            return [];
        }
    }

    public function syncing_totals_smart_collections_actual()
    {
        $total_gross = $this->col('syncing_totals_collections', 'int');
        return $total_gross / 2;
    }

    public function increment_current_amount($key, $amount_to_increment = false)
    {
        global $wpdb;

        if (!$amount_to_increment) {
            $amount_to_increment = 1;
        }

        $query = 'UPDATE ' . $this->table_name . ' SET syncing_current_amounts_' . $key . ' = syncing_current_amounts_' . $key . ' + ' . $amount_to_increment;

        return $wpdb->query($query);
    }

    public function get_recently_syncd_media_ref()
    {
        $current_chunk_string = $this->col(
            'recently_syncd_media_ref',
            'string'
        );

        return maybe_unserialize($current_chunk_string);

    }

    public function get_current_syncing_step_text()
    {
        return $this->col('current_syncing_step_text', 'string');
    }

    public function calc_percent_completed($total_processed, $total_to_process)
    {

        $total_processed    = Utils_Data::add_totals($total_processed);
        $total_to_process   = Utils_Data::add_totals($total_to_process);

        if ($total_processed === 0 || $total_to_process === 0) {
            return 0;
        }

        $percent = ($total_processed / $total_to_process) * 100;

        if (!$percent) {
            return 0;
        }

        return ceil($percent);

    }

    public function increment_percent_completed_removal($total, $current_total)
    {
        if (!$total || $total === 0) {
            return $this->update_col('percent_completed_removal', 100);
        }

        $percent = ceil(($current_total / $total) * 100);

        $this->log(__('Removing any stale data: ', 'shopwp') . $percent . '%');

        if ($percent >= 100) {
            $this->log(__('Removing stale data ... done', 'shopwp'));
        }

        return $this->update_col('percent_completed_removal', $percent);
    }

    public function set_current_syncing_step_text($text)
    {
        return $this->update_col('current_syncing_step_text', $text);
    }

    public function set_syncing_totals_removal($total)
    {
        return $this->update_col('syncing_totals_removal', $total);
    }

    public function update_recently_syncd_media_ref($media_chunk)
    {
        if (empty($media_chunk)) {
            return;
        }

        $current_chunk = $this->get_recently_syncd_media_ref();

        if ($current_chunk === '') {
            $current_chunk = [];
        }

        $current_chunk[] = $media_chunk;

        return $this->update_col('recently_syncd_media_ref', maybe_serialize($current_chunk));
    }

    public function reset_finished_media()
    {
        return $this->update_col('finished_media', 0);
    }

    public function reset_syncing_step_text() {
        return $this->update_col('current_syncing_step_text', 'Starting new sync ...');
    }

    public function reset_syncing_active_tool() {
        return $this->update_col('currently_active_tool', '');
    }

    public function reset_syncing_product_ids() {
        return [
            'published_product_ids' => $this->reset_syncing_published_product_ids()
        ];
    }

    public function reset_syncing_refs()
    {
        return [
            'media_refs' => $this->update_col('recently_syncd_media_ref', '')
        ];
    }

    public function reset_all_syncing_totals()
    {
        return [
            $this->reset_syncing_current_amounts(),
            $this->reset_syncing_totals(),
            $this->reset_syncing_timing(),
            $this->reset_syncing_product_ids(),
            $this->reset_syncing_refs(),
        ];
    }

    public function reset_all_syncing_status()
    {
        return [
            $this->reset_finished_media(),
            $this->reset_syncing_step_text(),
            $this->reset_syncing_active_tool()
        ];
    }

    public function save_notice_and_expire_sync($WP_Error)
    {
        $this->save_notice($WP_Error);
        return $this->expire_sync();
    }

    public function expire_sync($cancel_auto_sync = false)
    {

        do_action('shopwp_expire_sync', $cancel_auto_sync);

        $result = [
            'syncing_status'                => $this->toggle_syncing(0),
            'reset_all_syncing_totals'      => $this->reset_all_syncing_totals(),
            'reset_all_syncing_status'      => $this->reset_all_syncing_status(),
            'delete_cache'                  => Transients::delete_plugin_cache()
        ];

        $errors = maybe_unserialize($this->get_syncing_errors());

        if (!empty($errors)) {
            $this->log(__('Syncing stopped with errors', 'shopwp'));
        }

        return $result;
    }

    public function prepare_notice_for_save(
        $current_notices,
        $error_message,
        $type
    ) {
        $current_notices = maybe_unserialize($current_notices);

        if (empty($current_notices)) {
            $current_notices = [];
        }

        $current_notices[$type][] = $error_message;

        return maybe_serialize($current_notices);
    }

    public function save_error($error_message)
    {
        $current_errors = $this->get_syncing_errors();

        $serialized_errors = $this->prepare_notice_for_save(
            $current_errors,
            $error_message,
            'error'
        );

        $saved_Results = $this->update_col('syncing_errors', $serialized_errors);

        return $saved_Results;
    }

    public function save_warning($error_message)
    {
        $current_warnings = $this->get_syncing_warnings();
        $serialized_warnings = $this->prepare_notice_for_save(
            $current_warnings,
            $error_message,
            'warning'
        );

        return $this->update_col('syncing_warnings', $serialized_warnings);
    }

    public function save_notice($maybe_wp_error, $type = 'error')
    {
        if (!is_wp_error($maybe_wp_error)) {
            if ($type === 'error') {
                return $this->save_error($maybe_wp_error);
            } elseif ($type === 'warning') {
                return $this->save_warning($maybe_wp_error);
            } else {
                return $this->save_error($maybe_wp_error);
            }
        }

        $error_message = $maybe_wp_error->get_error_message();
        $type = $maybe_wp_error->get_error_code();

        if ($error_message) {
            if ($type === 'error') {
                return $this->save_error($error_message);
            } elseif ($type === 'warning') {
                return $this->save_warning($error_message);
            } else {
                return $this->save_error($error_message);
            }
        }
    }

    public function maybe_save_warning_from_insert($result, $type, $identifier)
    {
        if ($result === false) {
            $this->save_warning('Unable to sync ' . $type . ': ' . $identifier);
        }
    }

    public function init()
    {
        return $this->init_table_defaults();
    }

    public function init_table_defaults()
    {
        $results = [];

        if (!$this->table_has_been_initialized('id')) {
            $results = $this->insert_default_values();
        }

        return $results;
    }

    public function api_error($error_message, $method, $line)
    {
        $this->save_error($error_message);

        $final_wp_error_obj = Utils::wp_error([
            'message_lookup'    => $error_message,
            'call_method'       => $method,
            'call_line'         => $line,
        ]);

        return $final_wp_error_obj;

    }

    public function server_error($error_message, $method, $line)
    {

        $final_error_obj = Utils::wp_error([
            'message_lookup'    => $error_message,
            'call_method'       => $method,
            'call_line'         => $line,
        ]);
        
        $this->save_notice_and_expire_sync($final_error_obj);

        return $final_error_obj;
        
    }

    public function delete_log($file = 'shopwp') {
        $Filesystem         = \ShopWP\Factories\Filesystem\Filesystem_Factory::build();
        $Filesystem_Obj     = $Filesystem->bootstrap_filesystem();
        $uploads_dir        = \wp_get_upload_dir();
        $log_debug_file     = $uploads_dir['basedir'] . '/' . $file . '.log';
        $Filesystem_Obj->delete_file($log_debug_file);        
    }

    public function log($msg, $type = 'info') {
        
        $bytes = Utils::log([
            'thing_to_log'  => $msg,
            'log_level'     => $type,
            'file'          => 'shopwp'
        ]);

        if ($type === 'warning') {
            $this->save_warning($msg);

        } else if ($type === 'error') {
            $this->save_error($msg);
        }

    }

    public function expired_from_server_issues($items, $method = false, $line = false)
    {
        $expired = false;

        if ($this->max_packet_size_reached($items)) {
            $expired = true;

            $wp_error = Utils::wp_error([
                'message_lookup' => 'max_allowed_packet',
                'call_method' => $method,
                'call_line' => $line,
            ]);

            $error_msg = $wp_error->get_error_message();

            $this->log($error_msg, 'error');
            $this->save_notice_and_expire_sync($wp_error);
            
        } else if (Utils::exceeds_max_post_body_size($items, $method, $line)) {
            $expired = true;

            $wp_error = Utils::wp_error([
                'message_lookup' => 'max_post_body_size',
                'call_method' => $method,
                'call_line' => $line,
            ]);
            
            $error_msg = $wp_error->get_error_message();

            $this->log($error_msg, 'error');
            $this->save_notice_and_expire_sync($wp_error);
        }

        return $expired;
    }

    public function create_table_query($table_name = false)
    {
        if (!$table_name) {
            $table_name = $this->table_name;
        }

        $collate = $this->collate();

        return "CREATE TABLE $table_name (
			id bigint(100) unsigned NOT NULL AUTO_INCREMENT,
			is_syncing tinyint(1) DEFAULT '{$this->default_is_syncing}',
            syncing_step_total bigint(100) unsigned DEFAULT  '{$this->default_syncing_step_total}',
			syncing_step_current bigint(100) unsigned DEFAULT '{$this->default_syncing_step_current}',
            syncing_totals_shop bigint(100) unsigned DEFAULT '{$this->default_syncing_totals_shop}',
            syncing_totals_products bigint(100) unsigned DEFAULT '{$this->default_syncing_totals_products}',
			syncing_totals_collections bigint(100) unsigned DEFAULT '{$this->default_syncing_totals_collections}',
			syncing_totals_collects bigint(100) unsigned DEFAULT '{$this->default_syncing_totals_collects}',
            syncing_totals_media bigint(100) unsigned DEFAULT '{$this->default_syncing_totals_media}',
            syncing_totals_customers bigint(100) unsigned DEFAULT '{$this->default_syncing_totals_customers}',
            syncing_totals_orders bigint(100) unsigned DEFAULT '{$this->default_syncing_totals_orders}',
            syncing_current_amounts_shop bigint(100) unsigned DEFAULT '{$this->default_syncing_current_amounts_shop}',
			syncing_current_amounts_collections bigint(100) unsigned DEFAULT '{$this->default_syncing_current_amounts_collections}',
			syncing_current_amounts_products bigint(100) unsigned DEFAULT '{$this->default_syncing_current_amounts_products}',
			syncing_current_amounts_collects bigint(100) unsigned DEFAULT '{$this->default_syncing_current_amounts_collects}',
            syncing_current_amounts_media bigint(100) unsigned DEFAULT '{$this->default_syncing_current_amounts_media}',
            syncing_current_amounts_customers bigint(100) unsigned DEFAULT '{$this->default_syncing_current_amounts_customers}',
            syncing_current_amounts_orders bigint(100) unsigned DEFAULT '{$this->default_syncing_current_amounts_orders}',
			syncing_start_time bigint(100) unsigned DEFAULT '{$this->default_syncing_start_time}',
			syncing_end_time bigint(100) unsigned DEFAULT '{$this->default_syncing_end_time}',
			syncing_errors LONGTEXT DEFAULT '{$this->default_syncing_errors}',
			syncing_warnings LONGTEXT DEFAULT '{$this->default_syncing_warnings}',
            finished_media tinyint(1) DEFAULT '{$this->default_finished_media}',
			published_product_ids longtext DEFAULT '{$this->default_published_product_ids}',
            recently_syncd_media_ref longtext DEFAULT '{$this->default_recently_syncd_media_ref}',
            current_syncing_step_text varchar(255) DEFAULT '{$this->default_current_syncing_step_text}',
            percent_completed_removal bigint(100) unsigned DEFAULT '{$this->default_percent_completed_removal}',
            syncing_totals_removal bigint(100) unsigned DEFAULT '{$this->default_syncing_totals_removal}',
            bulk_op_id varchar(255) DEFAULT '',
            bulk_op_webhook_subscription_id varchar(255) DEFAULT '',
            expire_sync_from_server tinyint(1) DEFAULT '{$this->default_expire_sync_from_server}',
            currently_active_tool varchar(255) DEFAULT '{$this->default_currently_active_tool}',
			PRIMARY KEY  (id)
		) ENGINE=InnoDB $collate";
    }
}
