<?php

namespace ShopWP\DB;

use ShopWP\Utils;

if (!defined('ABSPATH')) {
    exit();
}

class Shop extends \ShopWP\DB
{

    public $table_name_suffix;
    public $table_name;
    public $version;
    public $primary_key;
    public $lookup_key;
    public $cache_group;
    public $type;

    public function __construct()
    {
        global $wpdb;

        $this->table_name_suffix = SHOPWP_TABLE_NAME_SHOP;
        $this->table_name = $this->get_table_name();
        $this->version = '1.0';
        $this->primary_key = 'id';
        $this->lookup_key = 'id';
        $this->cache_group = 'shopwp_db_shop';
        $this->type = 'shop';
    }

    // Used for doing db_delta checks
    public function get_columns()
    {
        return [
            'id' => '%d',
            'billingAddress' => '%s',
            'channelDefinitionsForInstalledChannels' => '%s',
            'checkoutApiSupported' => '%d',
            'contactEmail' => '%s',
            'countriesInShippingZones' => '%s',
            'currencyCode' => '%s',
            'currencyFormats' => '%s',
            'customerAccounts' => '%s',
            'description' => '%s',
            'domains' => '%s',
            'email' => '%s',
            'enabledPresentmentCurrencies' => '%s',
            'features' => '%s',
            'ianaTimezone' => '%s',
            'limitedPendingOrderCount' => '%s',
            'metafield' => '%s',
            'myshopifyDomain' => '%s',
            'name' => '%s',
            'orderNumberFormatPrefix' => '%s',
            'orderNumberFormatSuffix' => '%s',
            'plan' => '%s',
            'primaryDomain' => '%s',
            'publicationCount' => '%d',
            'resourceLimits' => '%s',
            'searchFilters' => '%s',
            'setupRequired' => '%d',
            'shipsToCountries' => '%s',
            'taxShipping' => '%d',
            'taxesIncluded' => '%d',
            'timezoneAbbreviation' => '%s',
            'timezoneOffset' => '%s',
            'timezoneOffsetMinutes' => '%d',
            'translations' => '%s',
            'unitSystem' => '%s',
            'url' => '%s',
            'weightUnit' => '%s',
            'customerTags' => '%s',
            'orderTags' => '%s'
        ];
    }

    public function cols_that_should_remain_ints()
    {
        return ['id', 'publicationCount', 'timezoneOffsetMinutes'];
    }

    public function get_shop($column)
    {
        global $wpdb;

        // If not a string ...
        if (!is_string($column)) {
            return;
        }

        // If argument not apart of schema ...
        if (!\array_key_exists($column, $this->get_columns())) {
            return;
        }

        $query = "SELECT $column FROM {$this->table_name};";

        return $wpdb->get_results($query);
    }

    public function insert_shop($shop)
    {
        return $this->insert($shop);
    }

    public function domain()
    {
        $domain = $this->get_shop('domain');

        if (Utils::array_not_empty($domain)) {
            return $domain[0]->domain;
        } else {
            return false;
        }
    }

    public function update_shop($shopData)
    {
        $shop = $this->get_shop('id');

        if (empty($shop)) {
            return false;
        }

        return $this->update(
            $this->lookup_key,
            $this->get_shop('id')[0]->id,
            $shopData
        );
    }

    public function get_shop_name()
    {
        $shop_name = $this->col('name', 'string');

        if (empty($shop_name)) {
            return false;
        }

        return $shop_name;
    }

    public function create_table_query($table_name = false)
    {

        if (!$table_name) {
            $table_name = $this->table_name;
        }

        $collate = $this->collate();

        return "CREATE TABLE $table_name (
			id bigint(100) unsigned NOT NULL AUTO_INCREMENT,
            billingAddress varchar(255) DEFAULT '',
            channelDefinitionsForInstalledChannels varchar(255) DEFAULT '',
            checkoutApiSupported tinyint(1) DEFAULT 0,
            contactEmail varchar(255) DEFAULT '',
            countriesInShippingZones varchar(255) DEFAULT '',
            currencyCode varchar(255) DEFAULT '',
            currencyFormats varchar(255) DEFAULT '',
            customerAccounts varchar(255) DEFAULT '',
            description varchar(255) DEFAULT '',
            domains varchar(255) DEFAULT '',
            email varchar(255) DEFAULT '',
            enabledPresentmentCurrencies varchar(255) DEFAULT '',
            features varchar(255) DEFAULT '',
            ianaTimezone varchar(255) DEFAULT '',
            limitedPendingOrderCount varchar(255) DEFAULT '',
            metafield varchar(255) DEFAULT '',
            myshopifyDomain varchar(255) DEFAULT '',
            name varchar(255) DEFAULT '',
            orderNumberFormatPrefix varchar(255) DEFAULT '',
            orderNumberFormatSuffix varchar(255) DEFAULT '',
            plan varchar(255) DEFAULT '',
            primaryDomain varchar(255) DEFAULT '',
            publicationCount bigint(100) DEFAULT 0,
            resourceLimits varchar(255) DEFAULT '',
            searchFilters varchar(255) DEFAULT '',
            setupRequired tinyint(1) DEFAULT 0,
            shipsToCountries varchar(255) DEFAULT '',
            taxShipping tinyint(1) DEFAULT 0,
            taxesIncluded tinyint(1) DEFAULT 0,
            timezoneAbbreviation varchar(255) DEFAULT '',
            timezoneOffset varchar(255) DEFAULT '',
            timezoneOffsetMinutes bigint(100) DEFAULT 0,
            translations varchar(255) DEFAULT '',
            unitSystem varchar(255) DEFAULT '',
            url varchar(255) DEFAULT '',
            weightUnit varchar(255) DEFAULT '',
            customerTags varchar(255) DEFAULT '',
            orderTags varchar(255) DEFAULT '',
			PRIMARY KEY  (id)
		) ENGINE=InnoDB $collate";
    }
}