<?php

namespace ShopWP\Processing;

use ShopWP\Utils;
use ShopWP\Transients;

if (!defined('ABSPATH')) {
    exit();
}

class Database extends \ShopWP\Processing\Vendor_Background_Process
{
    protected $action = 'shopwp_background_processing_deletions';
    protected $Config;
    protected $DB_Settings_Connection;
    protected $DB_Settings_General;
    protected $DB_Settings_License;
    protected $DB_Settings_Syncing;
    protected $DB_Shop;
    protected $DB_Products;
    protected $DB_Collections;
    protected $DB_Collects;
    protected $DB_Images;
    protected $DB_Metafields;
    protected $DB_Customers;
    protected $DB_Orders;
    protected $DB_Tags;
    protected $DB_Options;
    protected $DB_Variants;
    protected $DB_Posts;
    protected $Transients;
    protected $API_Settings_License;
    protected $Compatibility;
    public $DB;
    public $total;
    public $current_total;

    public function __construct(
        $Config,
        $DB_Settings_Connection,
        $DB_Settings_General,
        $DB_Settings_License,
        $DB_Settings_Syncing,
        $DB_Shop,
        $DB_Products,
        $DB_Collections,
        $DB_Collects,
        $DB_Images,
        $DB_Metafields,
        $DB_Customers,
        $DB_Orders,
        $DB_Tags,
        $DB_Options,
        $DB_Variants,
        $DB_Posts,
        $Transients,
        $API_Settings_License,
        $Compatibility
    ) {

        $this->Config                   = $Config;
        $this->DB_Settings_Connection   = $DB_Settings_Connection;
        $this->DB_Settings_General      = $DB_Settings_General;
        $this->DB_Settings_License      = $DB_Settings_License;
        $this->DB_Settings_Syncing      = $DB_Settings_Syncing;
        $this->DB_Shop                  = $DB_Shop;
        $this->DB_Products              = $DB_Products;
        $this->DB_Collections           = $DB_Collections;
        $this->DB_Collects              = $DB_Collects;
        $this->DB_Images                = $DB_Images;
        $this->DB_Metafields            = $DB_Metafields;
        $this->DB_Customers             = $DB_Customers;
        $this->DB_Orders                = $DB_Orders;
        $this->DB_Tags                  = $DB_Tags;
        $this->DB_Options               = $DB_Options;
        $this->DB_Variants              = $DB_Variants;
        $this->DB_Posts                 = $DB_Posts;
        $this->Transients               = $Transients;
        $this->API_Settings_License     = $API_Settings_License;
        $this->DB                       = $DB_Settings_General; // Convenience
        $this->Compatibility            = $Compatibility;
        $this->total                    = 0;
        $this->current_total            = 0;

        parent::__construct($DB_Settings_Syncing);
    }

    /*

	When uninstalling the plugin

	*/
    public function uninstall_plugin()
    {
        $results = [];

        if (
            Utils::is_plugin_installed(SHOPWP_FREE_BASENAME) &&
            Utils::is_plugin_installed(SHOPWP_PRO_BASENAME)
        ) {
            return $results;
        }

        $images = $this->DB_Images->get_all_plugin_attachments();

        $results['delete_posts'] = $this->delete_posts();
        $results['drop_custom_tables'] = $this->drop_custom_tables();
        

        $results['delete_custom_options'] = Transients::delete_custom_options();
        $results['delete_plugin_cache'] = Transients::delete_plugin_cache();
        $results[
            'delete_compatibility'
        ] = $this->Compatibility->delete_compatibility_mu();

        $results['delete_images'] = $this->DB_Images->delete_all_images($images);
        
        return $results;
    }

    /*

	When uninstalling the plugin

	*/
    public function uninstall_plugin_multisite()
    {
        $results = [];
        $blog_ids = \get_sites(['fields' => 'ids']);

        foreach ($blog_ids as $site_blog_id) {

            switch_to_blog($site_blog_id);

            $results['blog_' . $site_blog_id] = $this->uninstall_plugin();

            restore_current_blog();
        }

        return $results;
    }

    /*

	Drop custom tables

	Tested

	NOT USING BACKGROUND PROCESS

	*/
    public function drop_custom_tables()
    {
        $results = [];

        $results[
            'settings_general'
        ] = $this->DB->delete_table(SHOPWP_TABLE_NAME_SETTINGS_GENERAL);

        $results[
            'settings_license'
        ] = $this->DB->delete_table(SHOPWP_TABLE_NAME_SETTINGS_LICENSE);

        $results[
            'settings_connection'
        ] = $this->DB->delete_table(SHOPWP_TABLE_NAME_SETTINGS_CONNECTION);

        $results[
            'settings_syncing'
        ] = $this->DB->delete_table(SHOPWP_TABLE_NAME_SETTINGS_SYNCING);

        $results[
            'collections_smart'
        ] = $this->DB->delete_table(SHOPWP_TABLE_NAME_COLLECTIONS_SMART);

        $results[
            'collections_custom'
        ] = $this->DB->delete_table(SHOPWP_TABLE_NAME_COLLECTIONS_CUSTOM);


        $results['shop']        = $this->DB->delete_table(SHOPWP_TABLE_NAME_SHOP);
        $results['products']    = $this->DB->delete_table(SHOPWP_TABLE_NAME_PRODUCTS);
        $results['collections'] = $this->DB->delete_table(SHOPWP_TABLE_NAME_COLLECTIONS);
        $results['images']      = $this->DB->delete_table(SHOPWP_TABLE_NAME_IMAGES);
        $results['metafields']  = $this->DB->delete_table(SHOPWP_TABLE_NAME_METAFIELDS);
        $results['customers']   = $this->DB->delete_table(SHOPWP_TABLE_NAME_CUSTOMERS);
        $results['orders']      = $this->DB->delete_table(SHOPWP_TABLE_NAME_ORDERS);
        $results['variants']    = $this->DB->delete_table(SHOPWP_TABLE_NAME_VARIANTS);
        $results['options']     = $this->DB->delete_table(SHOPWP_TABLE_NAME_OPTIONS);
        $results['tags']        = $this->DB->delete_table(SHOPWP_TABLE_NAME_TAGS);
        $results['collects']    = $this->DB->delete_table(SHOPWP_TABLE_NAME_COLLECTS);

        return Utils::return_only_error_messages(
            Utils::return_only_errors($results)
        );
    }

    /*

	Drop databases used during uninstall

	Tested

	*/
    public function delete_posts()
    {
        return $this->DB_Posts->delete_posts();
    }

    /*

	Deletes both synced data AND custom post types but no:

	- Connection data
	- License data

	*/
    public function delete_detail_pages()
    {

        $this->DB_Settings_Syncing->set_current_syncing_step_text('Removing synced detail pages ...');
        $this->DB_Settings_Syncing->set_syncing_totals_removal(2);

        $this->push_to_queue('DB_Posts');
        $this->push_to_queue('DB_Settings_Syncing');
        $this->push_to_queue('Transients');

        return $this->save()->dispatch();
    }

    private function get_current_table_name($class_object) {
        return str_replace('wps ', '', str_replace('_', ' ', $class_object->table_name_suffix));
    }

    /*

	Deletes only synced Shopify data. Keeps custom post types, license, etc.

	*/
    public function delete_synced_data($remove_all = false, $check_clear_before = true)
    {

        $this->DB_Settings_Syncing->set_current_syncing_step_text(__('Removing old data ...', 'shopwp'));

        if ($remove_all) {

            $this->DB_Settings_Syncing->set_syncing_totals_removal(12);

            $this->push_to_queue([
                'class' => 'DB_Posts'
            ]);

        } else {
            $this->DB_Settings_Syncing->set_syncing_totals_removal(11);
        }
        
        $this->push_to_queue([
            'class' => 'DB_Shop'
        ]);

        $this->push_to_queue([
            'class' => 'DB_Products'
        ]);

        $this->push_to_queue([
            'class' => 'DB_Collections'
        ]);

        $this->push_to_queue([
            'class' => 'DB_Collects'
        ]);

        $this->push_to_queue([
            'class' => 'DB_Images'
        ]);

        $this->push_to_queue([
            'class' => 'DB_Metafields'
        ]);

        $this->push_to_queue([
            'class' => 'DB_Customers'
        ]);

        $this->push_to_queue([
            'class' => 'DB_Orders'
        ]);

        $this->push_to_queue([
            'class' => 'DB_Tags'
        ]);

        $this->push_to_queue([
            'class' => 'DB_Options'
        ]);

        $this->push_to_queue([
            'class' => 'DB_Variants'
        ]);

        return $this->save()->dispatch();

    }    

    /*

	Override this method to perform any actions required during the async request.

	*/
    protected function task($data)
    {
        
        $this->total                = $this->DB_Settings_Syncing->col('syncing_totals_removal', 'int');
        $this->current_total        = $this->current_total + 1;

        if (empty($data)) {
            return false;
        }

        if ($data['class'] === 'DB_Posts') {
            $this->DB_Posts->delete_posts();

        } else {
            $class_object = $this->{$data['class']};
            $table_name = $this->get_current_table_name($class_object);

            $class_object->truncate();
        }

        $this->DB_Settings_Syncing->increment_percent_completed_removal($this->total, $this->current_total);

        return false;

    }

    /*

    ** Important ** Tables are only updated when # of cols are different

    */
    public function should_update_cols($new_cols, $current_cols)
    {
        if (Utils::new_cols_greater_than_old($new_cols, $current_cols)) {
            return true;
        }

        if (Utils::new_cols_less_than_old($new_cols, $current_cols)) {
            return true;
        }

        return false;
    }

    /*

	Find the difference between tables in the database
	and tables in the database schema. Used during plugin updates
	to dynamically update the database.

	*/
    public function get_table_delta()
    {
        $tables = [];
        $final_delta = [];

        $tables[] = $this->DB_Settings_License;
        $tables[] = $this->DB_Settings_Connection;
        $tables[] = $this->DB_Settings_General;
        $tables[] = $this->DB_Settings_Syncing;
        $tables[] = $this->DB_Shop;
        $tables[] = $this->DB_Products;
        $tables[] = $this->DB_Collections;
        $tables[] = $this->DB_Collects;
        $tables[] = $this->DB_Images;
        $tables[] = $this->DB_Metafields;
        $tables[] = $this->DB_Customers;
        $tables[] = $this->DB_Orders;
        $tables[] = $this->DB_Tags;
        $tables[] = $this->DB_Options;
        $tables[] = $this->DB_Variants;

        foreach ($tables as $key => $table) {

            // Contains full table name /w prefix
            $table_name = $table->get_table_name();
            $table_exists = $table->table_exists($table_name);

            if (is_wp_error($table_exists) || $table_exists === false) {
                
                // Create table since it doesn't exist
                $result = $table->create_table();

            } else {
                if ($this->should_update_cols($table->get_columns(), $table->get_columns_current($table_name))) {
                    $final_delta[$table_name] = $table;
                }
            }

        }

        return array_filter($final_delta);
    }

    /*

    Useful for creating new tables and updating existing tables to a new structure.
    Does NOT remove columns or delete tables.

    Requires that tables already exist.

    */
    public function sync_table_deltas()
    {

        // Next get all tables
        $tables = $this->get_table_delta();

        if (empty($tables)) {
            return;
        }

        require_once Utils::get_abs_admin_path('includes/upgrade.php');

        foreach ($tables as $table) {
            $results = \dbDelta($table->create_table_query($table->table_name));
        }

        return $results;
    }

    /*

	When the background process completes ...

	*/
    protected function complete()
    {

        parent::complete();

        $this->DB_Settings_Syncing->expire_sync();

    }
}
