<?php

namespace ShopWP\Processing;

use ShopWP\Utils;
use ShopWP\Transients;

if (!defined('ABSPATH')) {
    exit();
}

class Media extends \ShopWP\Processing\Vendor_Background_Process
{
    protected $action = 'shopwp_background_processing_media';

    protected $Config;
    protected $DB_Settings_Connection;
    protected $DB_Settings_General;
    protected $DB_Settings_Syncing;
    protected $DB_Images;

    public function __construct(
        $Config,
        $DB_Settings_Connection,
        $DB_Settings_General,
        $DB_Settings_Syncing,
        $DB_Images
    ) {

        $this->Config                   = $Config;
        $this->DB_Settings_Connection   = $DB_Settings_Connection;
        $this->DB_Settings_General      = $DB_Settings_General;
        $this->DB_Settings_Syncing      = $DB_Settings_Syncing;
        $this->DB_Images                = $DB_Images;

        parent::__construct($DB_Settings_Syncing);
    }

    public function delete_synced_images() {

        $this->DB_Settings_Syncing->set_current_syncing_step_text(__('Removing synced images from WordPress media library ...', 'shopwp'));
        $this->DB_Settings_Syncing->set_syncing_totals_removal(2);

        $images = $this->DB_Images->get_all_plugin_attachments();

        if (empty($images)) {

            $this->DB_Settings_Syncing->set_current_syncing_step_text(__('No images found to delete, stopping ...', 'shopwp'));
            $this->DB_Settings_Syncing->expire_sync();

            return [];
        }

        $new_count = \count($images);

        $this->DB_Settings_Syncing->update_col('syncing_totals_media', $new_count);

        foreach ($images as $image) {
            $this->push_to_queue($image);    
        }

        return $this->save()->dispatch();
    }

    protected function task($image)
    {

        $removal_result = $this->DB_Images->remove_image($image->ID);

        if (is_wp_error($removal_result)) {
            $this->DB_Settings_Syncing->log(__('Error while removing image: ', 'shopwp') . $image->ID . '. Error message: ' . $removal_result->get_error_message(), 'error');
        }

        $current_total = $this->DB_Settings_Syncing->col('syncing_current_amounts_media', 'int');

        $this->DB_Settings_Syncing->update_col('syncing_current_amounts_media', $current_total + 1);

        return false;

    }

    protected function complete()
    {
        parent::complete();

        $this->DB_Settings_Syncing->expire_sync();

    }
}
