<?php

namespace ShopWP\Render\Cart;

use ShopWP\Utils\Data;

if (!defined('ABSPATH')) {
    exit();
}

class Defaults
{
    public $plugin_settings;
    public $Render_Attributes;

    public function __construct($plugin_settings, $Render_Attributes)
    {
        $this->plugin_settings      = $plugin_settings;
        $this->Render_Attributes    = $Render_Attributes;
    }

    public function cart_icon($attrs = []) {
        return array_replace([
            'icon' => $this->Render_Attributes->attr($attrs, 'icon', false),
            'type' => $this->Render_Attributes->attr($attrs, 'type', 'inline'),
             'show_counter' => $this->Render_Attributes->attr(
                $attrs,
                'show_counter',
                true
            ),
        ], $attrs);
    }

    public function cart($attrs = []) {
        return array_replace([], $attrs);
    }

    public function all_attrs($attrs = [], $defaults = [])
    {

        // TODO: Remove below hook in 7.0
        // TODO: Remove the below isset calls in 7.0
        $settings = apply_filters('shopwp_cart_default_payload_settings', [
            'icon' => $this->Render_Attributes->attr($attrs, 'icon', false),
            'type' => $this->Render_Attributes->attr($attrs, 'type', 'inline'),
            'show_counter' => $this->Render_Attributes->attr(
                $attrs,
                'show_counter',
                true
            ),
            'data_type' => $this->Render_Attributes->attr(
                $attrs,
                'data_type',
                false
            ),
            'icon_color' => $this->Render_Attributes->attr(
                $attrs,
                'icon_color',
                isset($this->plugin_settings['general']['cart_icon_color']) ? $this->plugin_settings['general']['cart_icon_color'] : '#000'
            ),
            'inline_icon_color' => $this->Render_Attributes->attr(
                $attrs,
                'inline_icon_color',
                '#000'
            ),
            'background_color' => $this->Render_Attributes->attr(
                $attrs,
                'icon_color',
                isset($this->plugin_settings['general']['cart_icon_background_color']) ? $this->plugin_settings['general']['cart_icon_background_color'] : '#000'
            ),
            'counter_background_color' => $this->Render_Attributes->attr(
                $attrs,
                'icon_color',
                isset($this->plugin_settings['general']['cart_counter_background_color']) ? $this->plugin_settings['general']['cart_counter_background_color'] : '#6ae06a'
            ),
            'counter_text_color' => $this->Render_Attributes->attr(
                $attrs,
                'icon_color',
                isset($this->plugin_settings['general']['cart_counter_text_color']) ? $this->plugin_settings['general']['cart_counter_text_color'] : '#FFF'
            ),
            'show_inventory_levels' => $this->Render_Attributes->attr(
                $attrs,
                'show_inventory_levels',
                true
            ),
            'left_in_stock_threshold' => $this->Render_Attributes->attr($attrs, 'left_in_stock_threshold', 10),
            'left_in_stock_text' => $this->Render_Attributes->attr($attrs, 'left_in_stock_text', $defaults['n']['leftInStock']),
            'cart_title' => $this->Render_Attributes->attr(
                $attrs,
                'cart_title',
                $defaults['l']['cartTitle']
            ),
            'checkout_text' => $this->Render_Attributes->attr(
                $attrs,
                'checkout_text',
                $this->plugin_settings['general']['cart_checkout_button_text']
            ),
            'updating_text' => $this->Render_Attributes->attr(
                $attrs,
                'updating_text',
                $defaults['l']['updatingCart']
            ),
            'checkout_failed_message' => $this->Render_Attributes->attr(
                $attrs,
                'checkout_failed_message',
                $defaults['e']['checkoutFailed']
            ),
            'lineitem_remove_text' => $this->Render_Attributes->attr(
                $attrs,
                'lineitem_remove_text',
                $defaults['l']['remove']
            ),
            'lineitem_sale_label_text' => $this->Render_Attributes->attr(
                $attrs,
                'lineitem_sale_label_text',
                $defaults['l']['sale']
            ),
            'lineitems_disable_link' => $this->Render_Attributes->attr(
                $attrs,
                'lineitems_disable_link',
                false
            ),
            'lineitems_link_target' => $this->Render_Attributes->attr(
                $attrs,
                'lineitems_link_target',
                '_self'
            ),
            'lineitems_max_quantity' => $this->Render_Attributes->attr(
                $attrs,
                'lineitems_max_quantity',
                false
            ),
            'lineitems_min_quantity' => $this->Render_Attributes->attr(
                $attrs,
                'lineitems_min_quantity',
                1
            ),
            'lineitems_quantity_step' => $this->Render_Attributes->attr(
                $attrs,
                'lineitems_quantity_step',
                false
            ),
            'notes_label' => $this->Render_Attributes->attr(
                $attrs,
                'notes_label',
                $defaults['l']['checkoutNotes']
            ),
            'notes_update_frequency' => $this->Render_Attributes->attr(
                $attrs,
                'notes_update_frequency',
                1000
            ),
            'notes_placeholder' => $this->Render_Attributes->attr(
                $attrs,
                'notes_placeholder',
                $this->plugin_settings['general'][
                    'cart_notes_placeholder'
                ]
            ),
            'empty_cart_text' => $this->Render_Attributes->attr(
                $attrs,
                'empty_cart_text',
                $defaults['l']['yourCartEmpty']
            ),
            'subtotal_label_text' => $this->Render_Attributes->attr(
                $attrs,
                'subtotal_label_text',
                $defaults['l']['subtotal']
            ),
            'show_cart_close_icon' => $this->Render_Attributes->attr(
                $attrs,
                'show_cart_close_icon',
                true
            ),
            'show_cart_title' => $this->Render_Attributes->attr(
                $attrs,
                'show_cart_title',
                true
            ),
            'max_quantity' => $this->Render_Attributes->attr(
                $attrs,
                'max_quantity',
                !empty($this->plugin_settings['general']['cart_max_quantity']) ? $this->plugin_settings['general']['cart_max_quantity'] : false
            ),
            'language' => $this->Render_Attributes->attr(
                $attrs,
                'language',
                $this->plugin_settings['general']['language_code']
            ),
            'country' => $this->Render_Attributes->attr(
                $attrs,
                'country',
                $this->plugin_settings['general']['country_code']
            ),
            'currency' => $this->Render_Attributes->attr(
                $attrs,
                'currency',
                $this->plugin_settings['general']['currency_code']
            ),
            'checkout_button_text_color' => $this->Render_Attributes->attr(
                $attrs,
                'checkout_button_text_color',
                $this->plugin_settings['general']['checkout_button_text_color']
            )
        ]);

        return apply_filters('shopwp_cart_default_settings', $settings);
    }
}
