<?php

namespace ShopWP\Render;

use ShopWP\Utils;
use ShopWP\Utils\Data as Utils_Data;

if (!defined('ABSPATH')) {
    exit();
}

class Templates
{
    public $Template_Loader;

    public function __construct($Template_Loader)
    {
        $this->Template_Loader = $Template_Loader;
    }

    public function sanitize_user_data(
        $user_data = [],
        $type = false,
        $component_class = false
    ) {

        if ($component_class) {
            $combined_data = $component_class->$type($user_data);

        } else {
            $combined_data = $user_data;
        }

        $vals_formatted = Utils_Data::standardize_layout_data($combined_data);

        if ($type !== 'cart' && $type !== 'cart_icon' && $type !== 'translator' && $type !== 'reviews') {

            if ($type === 'collections') {

                if (!empty($vals_formatted['collections_title'])) {
                    $vals_formatted['collection']   = $vals_formatted['collections_title'];
                }
                
                $collections_query              = $component_class->create_collections_query($vals_formatted);
                $products_query                 = $component_class->create_product_query($vals_formatted);
            } else {

                if (empty($combined_data['query']) && $component_class) {
                    $query = $component_class->create_product_query($vals_formatted);

                } else {

                    if (isset($combined_data['query'])) {
                        $query = $combined_data['query'];
                    } else {
                        $query = false;
                    }
                }
            }

            if ($type === 'collections') {
                $vals_formatted['collections_query'] = $collections_query;
                $vals_formatted['query'] = $products_query;
            } else {
                $vals_formatted['query'] = $query;
            }

        }

      return $vals_formatted;
      
    }

    public function params_client_render($params)
    {
        return [
            'data' => $params,
            'path' => 'components/wrapper/wrapper',
            'name' => 'client',
        ];
    }

    public function set_and_get_template($params)
    {
        if (empty($params['full_path'])) {
            return $this->Template_Loader->set_template_data($params['data'])->get_template_part($params['path'], $params['name']);
            
        } else {
            return $this->Template_Loader->set_template_data($params['data'])->get_template_part($params['full_path']);
        }
    }

    public function load($params)
    {
        return $this->set_and_get_template(
            $this->params_client_render($params)
        );
    }

    public function process_gutenberg_blocks($blocks, $blocks_to_search, $default_settings, $is_collection) {
         
        $final_settings = [];
        $updated_blocks = [];

        foreach ($blocks as $block) {

            $block_name = $block['blockName'];

            if (in_array($block_name, $blocks_to_search)) {
                
                $settings_id = (array) json_decode(urldecode(base64_decode($block['attrs']['settingsId'])));

                $final_settings = array_merge($final_settings, $settings_id);

                $block_name_without_prefix = str_replace('shopwp/', '', $block_name);
                $block_name_without_hyphens = str_replace('-', '_', $block_name_without_prefix);
                $block_name_final = str_replace('images', 'gallery', $block_name_without_hyphens);

                if (Utils::str_contains($block_name_final, 'collection')) {
                    $css_selector = 'collections_dropzone_' . $block_name_final;

                } else {

                    $css_selector = 'dropzone_product_' . $block_name_final;

                    if ($is_collection && $block_name_final === 'products') {

                        $settings_id['collection'] = $default_settings['collection'];
                        $settings_id['collectionsQuery'] = $default_settings['collections_query'];
                        $settings_id['collectionId'] = $default_settings['collection_id'];
                        $settings_id['query'] = $default_settings['query'];

                        $boom = base64_encode(json_encode($settings_id));

                        $block['attrs']['settingsId'] = $boom;
                    }
                }

                // Replace innerHTML and innerContent with desired <div> tag
                $block['innerHTML'] = '<div id="' . $css_selector . '"></div>';
                $block['innerContent'] = ['<div id="' . $css_selector . '"></div>'];

                $final_settings[$css_selector] = '#' . $css_selector;
            }

            if (!empty($block['innerBlocks'])) {
                $nested_inner = $block['innerBlocks'];
                $inner_blocks = $this->process_gutenberg_blocks($nested_inner, $blocks_to_search, $default_settings, $is_collection);
                $final_settings = array_merge($final_settings, $inner_blocks['final_settings']);
                $block['innerBlocks'] = $inner_blocks['updated_blocks'];
            }

            $updated_blocks[] = $block;
        }

        return [
            'final_settings'    => $final_settings,
            'updated_blocks'    => $updated_blocks
        ];

    }

    public function find_block_name($block_name) {
        $block_name_without_prefix = str_replace('shopwp/', '', $block_name);
        $block_name_without_hyphens = str_replace('-', '_', $block_name_without_prefix);
        $block_name_final = str_replace('images', 'gallery', $block_name_without_hyphens);
        $block_name_final = str_replace('collection_', '', $block_name_final);

        return $block_name_final;
    }

    public function find_all_used_blocks($blocks) {

        $found_block_names = [];

        // $blockNames = array_column($blocks, 'blockName');
        // $result = array_column(array_column($blocks, "blockName"), "innerBlocks");

        array_walk_recursive(
            $blocks,
            function($leafNode, $key) use(&$found_block_names) {
                if ($key === 'blockName') {
                    $found_block_names[] = $leafNode;
                }
            }
        );


        return array_filter($found_block_names, function($name) {
            if (empty($name) || !str_contains($name, 'shopwp')) {
                return false;
            }

            return $name;
        });

    }

    public function sanitize_block_names($names) {

        return array_map(function($name) {
            return $this->find_block_name($name);
        }, $names);
        
    }

    public function customize_gutenberg_blocks($post_content, $blocks_to_search, $default_settings, $is_collection = false) {

        $parsed_content = \parse_blocks($post_content);
        $processed_blocks = $this->process_gutenberg_blocks($parsed_content, $blocks_to_search, $default_settings, $is_collection);

        $final_settings = array_merge($processed_blocks['final_settings'], $default_settings);
        $final_settings = Utils::convert_settings_shape($final_settings);

        /*
        
        Allows for detecting which collection blocks are being used. 
        Updates the final "excludes" settings depending on which blocks are used.
        
        */
        if ($is_collection) {
            $possible_collection_excludes = ['title', 'image', 'description', 'products'];
            $used_blocks = $this->find_all_used_blocks($parsed_content);
            $used_blocks_sanitized = $this->sanitize_block_names($used_blocks);

            $block_name_difference = array_diff($possible_collection_excludes, $used_blocks_sanitized);
            $final_settings['collections_excludes'] = array_values($block_name_difference);
        }

        $processed_blocks['final_settings'] = $final_settings;

        $updated_blocks = $processed_blocks['updated_blocks'];

        // Serialize the updated parsed content while preserving innerHTML and innerContent
        $updated_content = \serialize_blocks($updated_blocks, true);

        return [
            'post_content' => $updated_content,
            'settings' => $final_settings
        ];
    }
}
