<?php

namespace ShopWP\Render\Collections;

if (!defined('ABSPATH')) {
    exit();
}

use ShopWP\Utils\Data;

class Defaults
{
    public $Attrs;
    public $Products_Defaults;
    public $plugin_settings;

    public function __construct(
        $Render_Attributes,
        $Products_Defaults,
        $plugin_settings
    ) {
        $this->Attrs = $Render_Attributes;
        $this->Products_Defaults = $Products_Defaults;
        $this->plugin_settings = $plugin_settings;
    }

    public function create_product_query($user_atts) {
        return $this->Products_Defaults->create_product_query($user_atts);
    }

    public function array_map_assoc(callable $f, array $a) {
        return array_column(array_map($f, array_keys($a), $a), 1, 0);
    }

    public function maybe_prefix_attrs($user_atts) {
        return $this->array_map_assoc(function($k, $v) {
            if (substr($k, 0, strlen('collections_')) !== 'collections_') {
                                
                if (substr($k, 0, strlen('products_')) === 'products_') {

                    $parts = array_values(array_filter(explode('products_', $k), function($piece) {
                        if ($piece === '') {
                            return false;
                        }

                        return !empty($piece);
                    }));

                    if (!empty($parts)) {
                        return [$parts[0], $v];
                    }

                } else {
                    $new_key = 'collections_' . $k;

                    return [$new_key, $v];
                }

            } else {
                return [$k, $v];
            }
        }, $user_atts);

    }

    public function maybe_convert_collection_id_to_title($attrs) {

        if (isset($attrs['collection_id'])) {
            $collection_id = $attrs['collection_id'];

        } else if (isset($attrs['collections_collection_id'])) {
            $collection_id = $attrs['collections_collection_id'];

        } else {
            $collection_id = false;
        }

        if (!empty($collection_id)) {

            $posts = \get_posts([
                'post_type' => 'wps_collections',
                'post_status' => 'publish',
                'posts_per_page' => -1,
                'meta_query' => [
                    [
                        'key' => 'collection_id',
                        'value' => $attrs['collections_collection_id'],
                        'compare' => '=',
                    ],
                ],
            ]);

            if (!empty($posts)) {
                $attrs['collections_title'] = $posts[0]->post_title;
            }
        }

        return $attrs;

    }

    public function create_collections_query($user_atts)
    {

        if (empty($user_atts)) {
            return '*';
        }

        // Ensure only "collections_" params are passed in
        $user_atts =$this->maybe_prefix_attrs($user_atts);

        // Maybe convert collection_id to collections_title
        $user_atts =$this->maybe_convert_collection_id_to_title($user_atts);

        $filter_params = $this->Attrs->get_collections_filter_params_from_shortcode(
            Data::format_shortcode_attrs($user_atts)
        );

        if (!isset($user_atts['collections_connective'])) {
            if (empty($user_atts)) {
                $user_atts = [];
            }

            $user_atts['collections_connective'] = SHOPWP_COLLECTIONS_DEFAULT_CONNECTIVE;
        }

        // Allows the user to pass in a custom query instead
        if (isset($user_atts['collections_query'])) {
            $final = $user_atts['collections_query'];
        } else {
            $final = $this->Attrs->build_query(
                $filter_params,
                $user_atts
            );
        }

        return $final;

    }

    public function collections($attrs) {
        if (empty($attrs)) {
            return [];
        }
        return array_replace([], $attrs);
    }

    public function all_attrs($attrs = [], $defaults = [])
    {

        $default_products_query_params = [
            'sort_by' => 'collection_default',
            'reverse' => false,
            'page_size' => 20,
            'query' => '',
            'query_type' => 'collectionProducts'
        ];

        $product_settings = $this->Products_Defaults->all_attrs($default_products_query_params, $defaults);

        $settings = array_merge(
            $product_settings,
            [
                'collection' => false,
                'collections_query' => $this->Attrs->attr(
                    $attrs,
                    'collections_query',
                    '*'
                ),
                'collections_sort_by' => $this->Attrs->attr(
                    $attrs,
                    'collections_sort_by',
                    'TITLE'
                ),
                'collections_reverse' => $this->Attrs->attr(
                    $attrs,
                    'collections_reverse',
                    false
                ),
                'collections_page_size' => $this->Attrs->attr(
                    $attrs,
                    'collections_page_size',
                    10
                ),
                'collections_collection_type' => $this->Attrs->attr(
                    $attrs,
                    'collections_collection_type',
                    false
                ),
                'collections_images_sizing_toggle' => $this->Attrs->attr(
                    $attrs,
                    'collections_images_sizing_toggle',
                    0
                ),
                'collections_images_sizing_width' => $this->Attrs->attr(
                    $attrs,
                    'collections_images_sizing_width',
                    500
                ),
                'collections_images_sizing_height' => $this->Attrs->attr(
                    $attrs,
                    'collections_images_sizing_height',
                    300
                ),
                'collections_images_sizing_crop' => $this->Attrs->attr(
                    $attrs,
                    'collections_images_sizing_crop',
                    'center'
                ),
                'collections_images_sizing_scale' => $this->Attrs->attr(
                    $attrs,
                    'collections_images_sizing_scale',
                    0
                ),
                'collections_collection_id' => $this->Attrs->attr(
                    $attrs,
                    'collections_collection_id',
                    false
                ),
                'collections_post_id' => $this->Attrs->attr(
                    $attrs,
                    'collections_post_id',
                    false
                ),
                'collections_connective' => $this->Attrs->attr(
                    $attrs,
                    'collections_connective',
                    SHOPWP_COLLECTIONS_DEFAULT_CONNECTIVE
                ),
                'collections_title' => $this->Attrs->attr($attrs, 'collections_title', false),
                'collections_collection' => $this->Attrs->attr(
                    $attrs,
                    'collections_title',
                    false
                ),
                'collections_items_per_row' => $this->Attrs->attr(
                    $attrs,
                    'collections_items_per_row',
                    3
                ),
                'collections_grid_column_gap' => $this->Attrs->attr(
                    $attrs,
                    'collections_grid_column_gap',
                    '20px'
                ),
                'collections_limit' => $this->Attrs->attr($attrs, 'collections_limit', false),
                'collections_post_meta' => $this->Attrs->attr(
                    $attrs,
                    'collections_post_meta',
                    false
                ),
                'collections_excludes' => $this->Attrs->attr($attrs, 'collections_excludes', is_singular('wps_collections') ? [] : [
                    'products',
                ]),
                'collections_pagination' => $this->Attrs->attr(
                    $attrs,
                    'collections_pagination',
                    true
                ),
                'collections_dropzone_page_size' => $this->Attrs->attr(
                    $attrs,
                    'collections_dropzone_page_size',
                    false
                ),
                'collections_dropzone_load_more' => $this->Attrs->attr(
                    $attrs,
                    'collections_dropzone_load_more',
                    false
                ),
                'collections_skip_initial_render' => $this->Attrs->attr(
                    $attrs,
                    'collections_skip_initial_render',
                    false
                ),
                'collections_dropzone_collection_title' => $this->Attrs->attr(
                    $attrs,
                    'collections_dropzone_collection_title',
                    false
                ),
                'collections_dropzone_collection_image' => $this->Attrs->attr(
                    $attrs,
                    'collections_dropzone_collection_image',
                    false
                ),
                'collections_dropzone_collection_description' => $this->Attrs->attr(
                    $attrs,
                    'collections_dropzone_collection_description',
                    false
                ),
                'dropzone_product_products' => $this->Attrs->attr(
                    $attrs,
                    'dropzone_product_products',
                    false
                ),
                'collections_infinite_scroll' => $this->Attrs->attr(
                    $attrs,
                    'collections_infinite_scroll',
                    false
                ),
                'collections_infinite_scroll_offset' => $this->Attrs->attr(
                    $attrs,
                    'collections_infinite_scroll_offset',
                    -200
                ),
                'collections_query_type' => $this->Attrs->attr(
                    $attrs,
                    'collections_query_type',
                    'collections'
                ),
                'collections_is_singular' => is_singular(SHOPWP_COLLECTIONS_POST_TYPE_SLUG),
                'collections_link_to' => $this->Attrs->attr(
                    $attrs,
                    'collections_link_to',
                    'wordpress'
                ),
                'collections_link_target' => $this->Attrs->attr(
                    $attrs,
                    'collections_link_target',
                    '_self'
                ),
                'collections_no_results_text' => $this->Attrs->attr(
                    $attrs,
                    'collections_no_results_text',
                    $defaults['n']['noCollectionsLeft']
                ),
                'collections_type' => $this->Attrs->attr(
                    $attrs,
                    'type',
                    'collection'
                ),
                'collections_sorting' => $this->Attrs->attr(
                    $attrs,
                    'collections_sorting',
                    false
                ),
                'collections_pagination_load_more_text' => $this->Attrs->attr(
                    $attrs, 
                    'collections_pagination_load_more_text', 
                    $defaults['l']['loadMore']
                ),
                'collections_pagination_load_more_button_color' => $this->Attrs->attr($attrs, 'collections_pagination_load_more_button_color', $this->plugin_settings['general']['pagination_load_more_button_color']),
                'collections_title_color' => $this->Attrs->attr($attrs, 'collections_title_color', '#000'),
                'collections_title_type_font_family' => $this->Attrs->attr($attrs, 'collections_title_type_font_family', false),
                'collections_title_type_font_size' => $this->Attrs->attr($attrs, 'collections_title_type_font_size', '32px'),
                'collections_title_type_font_weight' => $this->Attrs->attr($attrs, 'collections_title_type_font_weight', false),
                'collections_title_type_text_transform' => $this->Attrs->attr($attrs, 'collections_title_type_text_transform', false),
                'collections_title_type_font_style' => $this->Attrs->attr($attrs, 'collections_title_type_font_style', false),
                'collections_title_type_text_decoration' => $this->Attrs->attr($attrs, 'collections_title_type_text_decoration', false),
                'collections_title_type_line_height' => $this->Attrs->attr($attrs, 'collections_title_type_line_height', false),
                'collections_title_type_letter_spacing' => $this->Attrs->attr($attrs, 'collections_title_type_letter_spacing', false),
                'collections_description_length' => $this->Attrs->attr($attrs, 'collections_description_length', false),
                'collections_description_color' => $this->Attrs->attr($attrs, 'collections_description_color', '#000'),
                'collections_description_type_font_family' => $this->Attrs->attr($attrs, 'collections_description_type_font_family', false),
                'collections_description_type_font_size' => $this->Attrs->attr($attrs, 'collections_description_type_font_size', '20px'),
                'collections_description_type_font_weight' => $this->Attrs->attr($attrs, 'collections_description_type_font_weight', false),
                'collections_description_type_text_transform' => $this->Attrs->attr($attrs, 'collections_description_type_text_transform', false),
                'collections_description_type_font_style' => $this->Attrs->attr($attrs, 'collections_description_type_font_style', false),
                'collections_description_type_text_decoration' => $this->Attrs->attr($attrs, 'collections_description_type_text_decoration', false),
                'collections_description_type_line_height' => $this->Attrs->attr($attrs, 'collections_description_type_line_height', 1.6),
                'collections_description_type_letter_spacing' => $this->Attrs->attr($attrs, 'collections_description_type_letter_spacing', false),
            ]
        );

        return apply_filters('shopwp_collections_default_settings', $settings);
    }
}
