<?php

namespace ShopWP\Render\Storefront;

defined('ABSPATH') ?: exit();

use ShopWP\Utils\Data;

class Defaults
{
    public $Attrs;
    public $Products_Defaults;

    public function __construct($Render_Attributes, $Products_Defaults)
    {
        $this->Attrs                = $Render_Attributes;
        $this->Products_Defaults    = $Products_Defaults;
    }

    public function create_product_query($user_atts) {
        return $this->Products_Defaults->create_product_query($user_atts);
    }

    public function storefront($attrs) {

        // Here we can customize certain attributes, depending on others, without adding unnecessary logic to our JS
        $attrs = $this->Products_Defaults->apply_conditional_logic_to_settings($attrs);

        return array_replace([], $attrs);
    }

    public function all_attrs($attrs = [], $defaults = [])
    {
        return apply_filters('shopwp_storefront_default_settings', array_merge(
            $this->Products_Defaults->all_attrs($attrs, $defaults),
            [
                'query' => $this->Attrs->attr($attrs, 'query', '*'),
                'sort_by' => $this->Attrs->attr(
                    $attrs,
                    'sort_by',
                    'TITLE'
                ),
                'reverse' => $this->Attrs->attr(
                    $attrs,
                    'reverse',
                    false
                ),
                'page_size' => $this->Attrs->attr(
                    $attrs,
                    'page_size',
                    10
                ),
                'show_tags' => $this->Attrs->attr(
                    $attrs,
                    'show_tags',
                    true
                ),
                'show_vendors' => $this->Attrs->attr(
                    $attrs,
                    'show_vendors',
                    true
                ),
                'show_types' => $this->Attrs->attr(
                    $attrs,
                    'show_types',
                    true
                ),
                'show_collections' => $this->Attrs->attr(
                    $attrs,
                    'show_collections',
                    true
                ),
                'show_price' => $this->Attrs->attr(
                    $attrs,
                    'show_price',
                    true
                ),
                'show_selections' => $this->Attrs->attr(
                    $attrs,
                    'show_selections',
                    true
                ),
                'show_sorting' => $this->Attrs->attr(
                    $attrs,
                    'show_sorting',
                    true
                ),
                'show_pagination' => $this->Attrs->attr(
                    $attrs,
                    'show_pagination',
                    true
                ),
                'show_options_heading' => $this->Attrs->attr(
                    $attrs,
                    'show_options_heading',
                    true
                ),
                'dropzone_payload' => $this->Attrs->attr(
                    $attrs,
                    'dropzone_payload',
                    false
                ),
                'dropzone_options' => $this->Attrs->attr(
                    $attrs,
                    'dropzone_options',
                    false
                ),
                'dropzone_selections' => $this->Attrs->attr(
                    $attrs,
                    'dropzone_selections',
                    false
                ),
                'dropzone_sorting' => $this->Attrs->attr(
                    $attrs,
                    'dropzone_sorting',
                    false
                ),
                'dropzone_page_size' => $this->Attrs->attr(
                    $attrs,
                    'dropzone_page_size',
                    false
                ),            
                'dropzone_heading' => $this->Attrs->attr(
                    $attrs,
                    'dropzone_heading',
                    false
                ),
                'dropzone_load_more' => $this->Attrs->attr(
                    $attrs,
                    'dropzone_load_more',
                    true
                ),
                'dropzone_loader' => $this->Attrs->attr(
                    $attrs,
                    'dropzone_loader',
                    false
                ),
                'dropzone_notices' => $this->Attrs->attr(
                    $attrs,
                    'dropzone_notices',
                    false
                ),
                'pagination' => $this->Attrs->attr(
                    $attrs,
                    'pagination',
                    true
                ),
                'no_results_text' => $this->Attrs->attr(
                    $attrs,
                    'no_results_text',
                    $defaults['n']['noResultsFound']
                ),
                'excludes' => $this->Attrs->attr($attrs, 'excludes', [
                    'description',
                ]),
                'items_per_row' => $this->Attrs->attr(
                    $attrs,
                    'items_per_row',
                    3
                ),
                'limit' => $this->Attrs->attr($attrs, 'limit', false),
                'skip_initial_render' => $this->Attrs->attr(
                    $attrs,
                    'skip_initial_render',
                    false
                ),
                'infinite_scroll' => $this->Attrs->attr(
                    $attrs,
                    'infinite_scroll',
                    false
                ),
                'infinite_scroll_offset' => $this->Attrs->attr(
                    $attrs,
                    'infinite_scroll_offset',
                    -200
                ),
                'query_type' => $this->Attrs->attr(
                    $attrs,
                    'query_type',
                    'products'
                ),
                'price' => $this->Attrs->attr(
                    $attrs,
                    'price',
                    false
                ),
                'connective' => $this->Attrs->attr(
                    $attrs,
                    'connective',
                    SHOPWP_PRODUCTS_DEFAULT_CONNECTIVE
                ),
                'filter_option_open_on_load' => $this->Attrs->attr(
                    $attrs,
                    'filter_option_open_on_load',
                    'tags'
                ),
                'type' => $this->Attrs->attr(
                    $attrs,
                    'type',
                    'storefront'
                ),
                'no_filter_group_found_text' => $this->Attrs->attr(
                    $attrs,
                    'no_filter_group_found_text',
                    $defaults['n']['noResultsFound']
                ),
                'clear_filter_selections_text' => $this->Attrs->attr(
                    $attrs,
                    'clear_filter_selections_text',
                    $defaults['l']['clearAll']
                ),
                'selections_available_for_sale_text' => $this->Attrs->attr(
                    $attrs,
                    'selections_available_for_sale_text',
                    $defaults['l']['availSale']
                ),
                'load_more_collections_busy_text' => $this->Attrs->attr(
                    $attrs,
                    'load_more_collections_busy_text',
                    $defaults['l']['loading']
                ),
                'load_more_collections_text' => $this->Attrs->attr(
                    $attrs,
                    'load_more_collections_text',
                    $defaults['l']['seeMore']
                ),
                'collections_heading' => $this->Attrs->attr(
                    $attrs,
                    'collections_heading',
                    $defaults['l']['collections']
                ),
                'price_heading' => $this->Attrs->attr(
                    $attrs,
                    'price_heading',
                    $defaults['l']['price']
                ),
                'tags_heading' => $this->Attrs->attr(
                    $attrs,
                    'tags_heading',
                    $defaults['l']['tags']
                ),
                'types_heading' => $this->Attrs->attr(
                    $attrs,
                    'types_heading',
                    $defaults['l']['types']
                ),
                'vendors_heading' => $this->Attrs->attr(
                    $attrs,
                    'vendors_heading',
                    $defaults['l']['vendors']
                ),
                'with_sorting' => $this->Attrs->attr(
                    $attrs,
                    'with_sorting',
                    true
                ),
                'with_page_size' => $this->Attrs->attr(
                    $attrs,
                    'with_page_size',
                    true
                ),
                'variant_label_text_color' => $this->Attrs->attr($attrs,'variant_label_text_color', '#000'),
            ]
        ));

    }
}
