<?php

defined('ABSPATH') ?: die();

get_header('wpshopify');

$Collections = ShopWP\Factories\Render\Collections\Collections_Factory::build();
$Settings = ShopWP\Factories\DB\Settings_General_Factory::build();
$selection_collections = $Settings->col('sync_by_collections', 'string');

$selection_collections_pretty = maybe_unserialize($selection_collections);

$params = [
   'collections_link_target' => '_blank',
   'collections_link_to' => 'wordpress',
   'collections_connective' => 'OR',
];

// Only show collections that the user has selected in the syncing settings
if (empty($selection_collections_pretty)) {
   $title_query = false;
   $titles = false;

   $collection_posts = get_posts([
      'post_type' => 'wps_collections',
      'posts_per_page' => -1
   ]);

   $last_key = array_key_last($collection_posts);

   if ($last_key) {
      $id_of_last_item = $collection_posts[$last_key]->ID;
   } else {
      $id_of_last_item = false;
   }

   $final_query_string = array_reduce($collection_posts, function($prev, $curr) use($id_of_last_item) {

      if (empty($curr->collection_id)) {
         return $prev;
      }

      if ($curr->ID === $id_of_last_item) {
         $prev = $prev . 'id:' . $curr->collection_id;

      } else {
         $prev = $prev . 'id:' . $curr->collection_id . ' OR ';
      }

      return $prev;

   }, '');

   if (empty($final_query_string)) {
      $params['collections_query'] = 'zzzzzzzzzzzzzz';
   } else {
      $params['collections_query'] = $final_query_string;
   }
   

} else {
   $titles = array_reduce($selection_collections_pretty, function($prev, $curr) {
      array_push($prev, $curr['title']);

      return $prev;
   }, []);

   $params['collections_title'] = $titles ? $titles : false;

}


$posts_with_template = get_posts([
   'meta_key' => 'layout_type',
   'meta_value' => 'template-clp',
   'post_type' => 'shopwp_shortcodes',
   'post_status' => 'publish',
   'posts_per_page' => -1
]);

?>

<style>

   .wps-container {
      max-width: 1200px;
      width: 100%;
      margin: 0 auto 50px auto;
   }
   
   .wps-breadcrumbs {
      max-width: 1200px;
      margin: 0 auto;
   }

   .wps-breadcrumbs-name {
      text-transform: capitalize;
   }

   .wps-heading {
      text-align: center;
   }

</style>

<section class="wps-container">

   <?= do_action('shopwp_breadcrumbs') ?>

   <div class="wps-collections-all">

   <?php 
   
      if (empty($posts_with_template)) { ?>
         
         <header class="wps-collections-header">
            <h1 class="wps-heading">
               Collections
            </h1>
         </header>
            
         <?= do_action('shopwp_breadcrumbs') ?>

         <div class="wps-products-all">
            <?php 
               
               $Collections->collections(apply_filters('shopwp_collections_all_args', apply_filters('wps_collections_all_args', $params)));

            ?>
         </div>

      <?php } else {
         echo $posts_with_template[0]->post_content;
      } ?>

   </div>

</section>

<?php get_footer('wpshopify');
