<?php

defined('ABSPATH') ?: die();

get_header('wpshopify');

global $post;

$Collections = ShopWP\Factories\Render\Collections\Collections_Factory::build();
$Products = ShopWP\Factories\Render\Products\Products_Factory::build();
$Settings = ShopWP\Factories\DB\Settings_General_Factory::build();
$Templates = ShopWP\Factories\Render\Templates_Factory::build();

$Queries = ShopWP\Factories\API\Admin\Webhooks\Queries_Factory::build();

$products_per_page = $Settings->col('num_posts', 'int');
$post_id = $post->ID;
$collection_id = get_post_meta($post_id, 'collection_id', true);

$posts_with_template = get_posts([
   'meta_key' => 'layout_type',
   'meta_value' => 'template-cdp',
   'post_type' => 'shopwp_shortcodes',
   'post_status' => 'publish',
   'posts_per_page' => -1
]);

?>

<style>
   .wps-breadcrumbs {
      max-width: 1200px;
      margin: 0 auto;
   }

   .wps-breadcrumbs-name {
         text-transform: capitalize;
      }
      
   .wps-collection-single-content {
      max-width: 1200px;
      margin: 0 auto;
      display: flex;
      flex-direction: column;
   }

   .wps-collections-wrapper {
      width: 100%;
      max-width: 100%;      
   }

   #collection_products {
      margin-top: 2em;
   }

   #collection_products_sorting {
      margin-top: 20px;
      display: flex;
      justify-content: flex-end;
   }

</style>

<section class="wps-collections-wrapper wps-container">
   <?= do_action('shopwp_breadcrumbs') ?>

   <div class="wps-collection-single row">

      <?php if (empty($posts_with_template)) { ?>
      
      <div class="wps-collection-single-content col">

         <div id="collection_image"></div>
         <div id="collection_title"></div>
         <div id="collection_description"></div>
         <div id="collection_products_sorting"></div>
         <div id="collection_products"></div>

         <?php 
            
            $Collections->collections(
               apply_filters('shopwp_collections_single_args', [
                  'collections_query'                    => 'id:"' . $collection_id . '"',
                  'dropzone_collection_image'            => '#collection_image',
                  'dropzone_collection_title'            => '#collection_title',
                  'dropzone_collection_description'      => '#collection_description',
                  'dropzone_collection_products'         => '#collection_products',
                  'dropzone_collection_products_sorting' => '#collection_products_sorting',
                  'collections_excludes'                 => false,
                  'excludes'                             => ['description'],
                  'collections_infinite_scroll'          => false,
                  'sort_by'                              => 'collection_default',
                  'with_sorting'                         => true,
               ])
            );
         
         ?>

      </div>

      <?php } else {

         $default_settings = [
            'collections_link_to'      => 'none',
            'collections_excludes'     => [],
            'collections_post_id'      => $post_id,
            'collections_pagination'   => false,
            'collections_limit'        => 1,
            'collection'               => [$post->post_title],
            'collection_id'            => $collection_id,
            'collections_query'        => 'id:"' . $collection_id . '"',
            'query'                    => 'id:"' . $collection_id . '"'
         ];

         $blocks_to_search = [
            'shopwp/collection-image',
            'shopwp/collection-title',
            'shopwp/collection-description',
            'shopwp/products',
         ];

         $block_data = $Templates->customize_gutenberg_blocks($posts_with_template[0]->post_content, $blocks_to_search, $default_settings, true);

         echo $block_data['post_content'];

         $collections_settings = $block_data['settings'];

         $Collections->collections(
            apply_filters('shopwp_collections_single_args', $collections_settings)
         );

      }

      ?>

   </div>
</section>

<?php 

get_footer('wpshopify');