<?php

defined('ABSPATH') ?: die();

get_header('wpshopify');

$Products = ShopWP\Factories\Render\Products\Products_Factory::build();
$Settings = ShopWP\Factories\DB\Settings_General_Factory::build();

$paged_var = get_query_var('paged');
$paged = $paged_var ? $paged_var : 1;
$num_posts = $Settings->col('num_posts', 'int');

if (empty($num_posts) || $num_posts >= 250) {
   $num_posts = 10; // reasonable default
}

$is_showing_heading = $Settings->col(
    'products_heading_toggle',
    'bool'
);

$description_toggle = $Settings->col(
    'products_plp_descriptions_toggle',
    'bool'
);

if (!$description_toggle) {
    $products_args = [
        'excludes' => ['description'],
    ];
} else {
    $products_args = [
        'excludes' => [],
    ];
}

$posts_ids = get_posts([
    'post_type' => 'wps_products',
    'posts_per_page' => -1,
    'fields' => 'ids',
]);

if (!empty($posts_ids)) {
   $post_ids = array_values(array_slice($posts_ids, -250, 250, true));
   $products_args['post_id'] = $post_ids;

} else {

   // Force no products to show if empty post_ids
   $products_args['product_id'] = 999999991;

}

$products_args['connective']           = SHOPWP_PRODUCTS_DEFAULT_CONNECTIVE;
$products_args['page_size']            = $num_posts;
$products_args['available_for_sale']   = true;
$products_args['show_featured_only']   = true;

$posts_with_template = get_posts([
   'meta_key' => 'layout_type',
   'meta_value' => 'template-plp',
   'post_type' => 'shopwp_shortcodes',
   'post_status' => 'publish',
   'posts_per_page' => -1
]);

?>

<style>

   .wps-container {
      max-width: 1200px;
      width: 100%;
      margin: 0 auto 50px auto;
   }
   
   .wps-breadcrumbs {
      max-width: 1200px;
      margin: 0 auto;
   }

   .wps-breadcrumbs-name {
         text-transform: capitalize;
      }

   .wps-products-wrapper {
      display: flex;
      padding: 2em 0;
   }

   .wps-pagination {
      text-align: center;
      padding: 2em 0;
      font-size: 20px;      
   }

   .wps-products-content {
      flex: 1;
   }

   .wps-products-sidebar {
      width: 30%;
   }

   .wps-heading {
      text-align: center;
   }
</style>

<section class="wps-products-wrapper wps-container">

   <div class="wps-products-content">
      
   <?php 
   
      if (empty($posts_with_template)) { ?>

         <header class="wps-products-header">
            <h1 class="swp-mb20 wps-heading">
               Products
            </h1>
         </header>
            
         <?= do_action('shopwp_breadcrumbs') ?>

         <div class="wps-products-all">
            <?php $Products->products(apply_filters('shopwp_products_all_args', $products_args)); ?>
         </div>

      <?php } else {

         echo $posts_with_template[0]->post_content;
      } ?>

   </div>

   <?php if (apply_filters('shopwp_products_show_sidebar', false)) { ?>
      <div class="wps-sidebar wps-products-sidebar">
         <?= get_sidebar('wpshopify') ?>
      </div>
   <?php } ?>

</section>

<?php get_footer('wpshopify');
