<?php

use ShopWP\Utils;

defined('ABSPATH') ?: die();

get_header('wpshopify');

global $post;

$Products            = ShopWP\Factories\Render\Products\Products_Factory::build();
$Reviews             = ShopWP\Factories\Render\Reviews\Reviews_Factory::build();
$Settings_General    = new ShopWP\DB\Settings_General;
$DB_Products         = ShopWP\Factories\DB\Products_Factory::build();
$Templates           = ShopWP\Factories\Render\Templates_Factory::build();


$show_reviews_below_product = $Settings_General->col('yotpo_reviews', 'bool');

if (empty($post)) {
   $post_id = false;

} else {
   $post_id = $post->ID;
}

$product_id    = get_post_meta($post_id, 'product_id', true);
$product_d     = $DB_Products->get_product_from_product_id($product_id);

if (empty($product_d)) {
   $prices        = false;
   $feat_image    = false;

} else {
   $prices        = maybe_unserialize($product_d[0]->priceRangeV2);
   $feat_image    = maybe_unserialize($product_d[0]->featuredImage);
}

$can_show_reviews = defined('SHOPWP_DOWNLOAD_ID_YOTPO_REVIEWS_EXTENSION') && $show_reviews_below_product && $product_id ? true : false;

$posts_with_template = get_posts([
   'meta_key' => 'layout_type',
   'meta_value' => 'template-pdp',
   'post_type' => 'shopwp_shortcodes',
   'post_status' => 'publish',
   'posts_per_page' => -1
]);

$product_settings = [
   'dropzone_product_buy_button' => '#product_buy_button',
   'dropzone_product_title' => '#product_title',
   'dropzone_product_description' => '#product_description',
   'dropzone_product_pricing' => '#product_pricing',
   'dropzone_product_gallery' => '#product_gallery',
   'link_to' => 'none',
   'excludes' => [],
   'product_id' => $product_id,
   'pagination' => false,
   'show_out_of_stock_variants' => false,
   'limit' => 1,
   'select_first_variant' => true,
];

?>

<section class="wps-container" itemtype="https://schema.org/Product" itemscope>

   <?php if (!empty($product_d)) { ?>
      <meta itemprop="name" content="<?= $product_d[0]->title; ?>" />
      <meta itemprop="description" content="<?= $product_d[0]->description; ?>" />
   <?php } ?>

   <?php if (!empty($feat_image)) { ?>
      <link itemprop="image" href="<?= $feat_image->url ?>" />
   <?php } ?>
   
   <div itemprop="offers" itemtype="https://schema.org/Offer" itemscope>
      <link itemprop="url" href="https://example.com/anvil" />
      <meta itemprop="availability" content="https://schema.org/InStock" />

      <?php if (!empty($prices)) { ?>
         <meta itemprop="priceCurrency" content="<?= $prices->minVariantPrice->currencyCode; ?>" />
         <meta itemprop="price" content="<?= $prices->minVariantPrice->amount; ?>" />
      <?php } ?>
      
   </div>

   <style>

      .wps-container {
         max-width: 1200px;
         width: 100%;
         margin: 0 auto 50px auto;
      }

      .single-wps_products .wps-breadcrumbs + .wps-product-single {
         margin-top: 0;
      }

      .single-wps_products .wps-product-single {
         margin-top: 1em;
         margin-bottom: 0;
         display: flex;
         flex-direction: column;
      }

      .single-wps_products .wps-product-single-inner {
         display: flex;
      }

      .single-wps_products .wps-product-single + [data-wpshopify-component] {
         margin-top: -40px;
      }

      .single-wps_products .wps-product-single-content,
      .single-wps_products .wps-product-single-gallery {
         width: 50%;
         max-width: 50%;
         flex: 0 0 50%;
      }

      .single-wps_products .wps-product-single-content {
         padding: 0em 0 2em 2em;
         width: calc(50% - 4em);
         max-width: calc(50% - 4em);
         flex: 0 0 calc(50% - 4em);
      }

      .single-wps_products .wps-items {
         width: 100%;
         max-width: 100%;
      }

      .wp-block-columns {
         width: 100%;
         align-items: normal !important;
         box-sizing: border-box;
         display: flex;
         flex-wrap: wrap !important;
      }      

      @media (min-width: 800px) {
         .wp-block-columns {
            flex-wrap: nowrap !important;
         }
         .wp-block-column {
            flex-basis: 0;
            flex-grow: 1;
            min-width: 0;
            overflow-wrap: break-word;
            word-break: break-word;            
         }

         .wp-block-columns {
            gap: 1.5rem
         }
      }

      @media (max-width: 800px) {
         .wp-block-columns:not(.is-not-stacked-on-mobile) > .wp-block-column {
            flex-basis: 100% !important;
         }

         .single-wps_products .wps-product-single-inner {
            flex-direction: column;
            align-items: center;
         }

         .single-wps_products .wps-product-single-content,
         .single-wps_products .wps-product-single-gallery {
            width: 80%;
            max-width: 80%;
            flex: 0 0 80%;
         }

         .single-wps_products .wps-product-single-content {
            padding: 0em 0 2em 2em;
            width: calc(80% - 4em);
            max-width: calc(80% - 4em);
            flex: 0 0 calc(80% - 4em);
         }                  
      }      

      @media (max-width: 600px) {

         .single-wps_products .wps-product-single + [data-wpshopify-component] {
            margin-top: 0;
         }

         .single-wps_products .wps-product-single {
            flex-direction: column;
         }

         .single-wps_products .wps-container {
            padding: 0 1em;
         }

         .single-wps_products .wps-product-single-content,
         .single-wps_products .wps-product-single-gallery {
            width: 100%;
            max-width: 100%;
            padding: 0;
            flex: none;
         }

         .single-wps_products .wps-product-single .wps-product-image-wrapper .wps-product-image {
            margin: 0 auto;
            display: block;      
         }

      }

   </style>

   <?= do_action('shopwp_breadcrumbs') ?>
   
   <div class="wps-product-single">

   <?php if (empty($posts_with_template)) { ?>

      <div class="wps-product-single-inner">

         <div class="wps-product-single-gallery">
            <div id="product_gallery"></div>
         </div>

         <div class="wps-product-single-content">
            
            <?php if ($can_show_reviews) { ?>
               <div id="product_review_rating" itemscope="" itemprop="review" itemtype="https://schema.org/Review"></div>
            <?php } ?>
            
            <div id="product_title"></div>
            <div id="product_pricing"></div>
            <div id="product_description"></div>
            <div id="product_buy_button"></div>

         </div>

      </div>
      
   <?php } else { 

      $default_settings = [
         'link_to' => 'none',
         'excludes' => [],
         'product_id' => $product_id,
         'pagination' => false,
         'limit' => 1,
         'query' => 'id:' . $product_id
      ];

      $blocks_to_search = [
         'shopwp/images',
         'shopwp/title',
         'shopwp/buy-button',
         'shopwp/description',
         'shopwp/pricing'
      ];

      $block_data = $Templates->customize_gutenberg_blocks($posts_with_template[0]->post_content, $blocks_to_search, $default_settings);

      echo $block_data['post_content'];

      $product_settings = $block_data['settings'];

      ?>
   <?php } ?>

   </div>

   <?php 

   $Products->products(
      apply_filters('shopwp_products_single_args', $product_settings)
   );

   if ($can_show_reviews) { ?>

   <style>
      [data-wpshopify-component-type="reviews"] {
         margin-top: 40px;
      }
   </style>
      <?php
      $Reviews->reviews(
         apply_filters('shopwp_products_single_reviews_args', [
            'product_id' => $product_id,
            'dropzone_rating' => '#product_review_rating',
            'dropzone_listing' => '#product_reviews',
            'show_rating' => true,
            'reviews_shown' => 30,
            'show_create_new' => $show_reviews_below_product ? true : false,
            'show_listing' => $show_reviews_below_product
         ])
      );
   }

   ?>

</section>

<?php 

get_footer('wpshopify');